require 'mongo'
require 'mongo-import'
require 'bson'
require 'active_support/time'
include Mongo
include MongoImport

module MongoUtility
  HOST = "localhost"
  PORT = 27017

  def removeCollection(collectionName, dbName)
    @db = Mongo::Connection.new(HOST, PORT).db(dbName)
    @db.collection(collectionName).remove
    puts "[removeCollection] removed All documents for " + collectionName
  end

  # def updateDocument(collectionName, dbName, objId, columnName, value)
  #   @db = Mongo::Connection.new(HOST, PORT).db(dbName)
  #   documents = @db.collection(collectionName)
  #   documents.update({:_id => BSON::ObjectId(objId)}, {"$set" => {columnName => value}})
  # end

  def updateDocument(collectionName, dbName, objId, columnName, value)
    @db = Mongo::Connection.new(HOST, PORT).db(dbName)
    documents = @db.collection(collectionName)
    documents.update({:_id => BSON::ObjectId(objId)}, {"$set" => {columnName => value}})
  end

  def removeDocument(collectionName, dbName, assigningAuthorityValue, uniqueIdValue)
    @db = Mongo::Connection.new(HOST, PORT).db(dbName)
    documents = @db.collection(collectionName).sort({})
    documents.remove({"patientIdentifier.uniqueId" => uniqueIdValue, "patientIdentifier.assigningAuthority"=> assigningAuthorityValue})
  end

  def insertCollection(collectionName, dbName, fileName)
    snapshot fileName, :db => dbName, :collection => collectionName, :path => 'spec/snapshots'
    puts "[insertCollection] imported Collection " + collectionName
  end

  def insertDumpCollection(collectionName, dbName, filePath)
    command = 'mongorestore --collection ' + collectionName + ' --db ' + dbName + ' ' + filePath
    system command
    #puts "[insertCollection] imported Collection " + collectionName
  end

  def restoreCollection(collectionName, dbName, fileName)
    removeCollection(collectionName, dbName)
    insertDumpCollection(collectionName, dbName, fileName)
    puts "[restoreCollection] reset Collection " + collectionName

  end

  def insertDocument(collectionName, dbName, document)
    db = Mongo::Connection.new(HOST, PORT).db(dbName)
    collection = db.collection(collectionName)
    collection.insert(document)
  end

  def resetCollection(collectionName, dbName, fileName)
    removeCollection(collectionName, dbName)
    insertCollection(collectionName, dbName, fileName)
    puts "[resetCollection] reset Collection " + collectionName
  end

  def getFieldByObjId(collectionName, dbName, objId, columnName)
    @db = Mongo::Connection.new(HOST, PORT).db(dbName)
    document = @db.collection(collectionName).find({:_id => BSON::ObjectId(objId)}, :fields => [columnName]).to_a
    #puts "[getDocumentByObjId ] " + document.to_s

    for record in document
      puts "record[columnName]=" + record[columnName].to_s
      return record[columnName]
    end
  end

  def getDocumentByObjId(collectionName, dbName, objId, columnName)
    @db = Mongo::Connection.new(HOST, PORT).db(dbName)
    document = @db.collection(collectionName).find({:_id => BSON::ObjectId(objId)}, :fields => [columnName]).to_a
    #puts "[getDocumentByObjId ] " + document.to_s

    return document
  end

  def getDocumentByPatientIdentifier(collectionName, dbName, assigningAuthorityValue, uniqueIdValue, columnName)
    @db = Mongo::Connection.new(HOST, PORT).db(dbName)
    document = @db.collection(collectionName).find({"patientIdentifier.uniqueId" => uniqueIdValue, "patientIdentifier.assigningAuthority"=> assigningAuthorityValue}, :fields => [columnName])

    return document
  end

  def verifyNotificationDeleted(collectionName, dbName, objId, columnName)
    document = getDocumentByObjId(collectionName, dbName, objId, columnName)
    isDeleted = true

    for record in document
      if record[columnName] == false
        isDeleted = false
        break
      end
    end

    #puts "[verifyNotificationDeleted ] objId: " + objId + " - columnName: " + columnName + " IsDeleted: " + isDeleted.to_s
    return isDeleted

  end

  def verifyDocumentsDeleted(collectionName, dbName, assigningAuthorityValue, uniqueIdValue, columnName)
    document = getDocumentByPatientIdentifier(collectionName, dbName, assigningAuthorityValue, uniqueIdValue, columnName)
    isDeleted = true

    for record in document
      #puts "record[columnName] = " + record[columnName].to_s

      if record[columnName] == false
        isDeleted = false
        break
      end
    end

    return isDeleted

  end

  def retrieveThisFieldInDocumentWithMultiRows(collectionName, dbName, assigningAuthorityValue, uniqueIdValue, columnName)
    document = getDocumentByPatientIdentifier(collectionName, dbName, assigningAuthorityValue, uniqueIdValue, columnName)
    outPut = ""
    document.each { |record|
      #puts record[columnName]
      if record[columnName] != nil then
        outPut = record[columnName] + ',' + outPut.to_s
      else
        output = "" + "," + output.to_s
      end
    }
    #puts "outPut=" + outPut.to_s
    return outPut

  end

  def retrieveThisFieldInDocumentWithMultiRowsAsArray(collectionName, dbName, assigningAuthorityValue, uniqueIdValue, columnName)
    document = getDocumentByPatientIdentifier(collectionName, dbName, assigningAuthorityValue, uniqueIdValue, columnName)
    outPut = Array.new
    document.each { |record|
      if record[columnName] != nil then
        outPut.push(record[columnName])
      end
    }
    return outPut
  end

  def retrieveThisFieldInDocument(collectionName, dbName, assigningAuthorityValue, uniqueIdValue, columnName)
    document = getDocumentByPatientIdentifier(collectionName, dbName, assigningAuthorityValue, uniqueIdValue, columnName)

    for record in document
      puts "record[columnName]=" + record[columnName].to_s
      return record[columnName]
    end
  end

  def getTotalDocumentCountByPatientIdentifier(collectionName, dbName, assigningAuthorityValue, uniqueIdValue)
    @db = Mongo::Connection.new(HOST, PORT).db(dbName)
    document = @db.collection(collectionName).find({"patientIdentifier.assigningAuthority"=> assigningAuthorityValue, "patientIdentifier.uniqueId" => uniqueIdValue})

    return document.count
  end

  def retreiveAllDocumentsInCollection(collectionName, dbName)
    documentArray = []
    db = Mongo::Connection.new(HOST, PORT).db(dbName)
    documents = db.collection(collectionName).find()
    documents.each do | document |
      documentArray << document
    end

    puts documentArray.to_s

    return documentArray
  end

  def removeAllDocuments()
    removeCollection("fitness", "healthfactordb")
    removeCollection("fitnessgoals", "healthfactordb")
    removeCollection("chemlabs", "medicalhistorydb")
    removeCollection("chemlabsbundles", "medicalhistorydb")
    removeCollection("vitalsigns", "medicalhistorydb")
    removeCollection("vitalsignsbundles", "medicalhistorydb")
    removeCollection("mood", "medicalhistorydb")

    removeCollection("notifications", "notificationsdb")
    removeCollection("reminders", "remindersdb")
    removeCollection("notificationPreferences", "notificationsdb")
    removeCollection("preferences", "notificationsdb")
  end

  def setDefaultPreferencesForPatient(uniqueId)
    @db = Mongo::Connection.new(HOST, PORT).db("notificationsdb")
    documents = @db.collection("preferences")
    document = getDocumentByPatientIdentifier("preferences", "notificationsdb", "EDIPI", uniqueId, "timeZone")
    if(document.next() == nil)
      documents.insert({"patientIdentifier" => { "uniqueId" => uniqueId, "assigningAuthority"=> "EDIPI" }, "timeZone" => "(-05:00) America/New_York (Eastern)", "emailAddress" => ""})
    end
  end

  def setDefaultPreferencesForPatientWithEmail(uniqueId)
    @db = Mongo::Connection.new(HOST, PORT).db("notificationsdb")
    documents = @db.collection("preferences")
    document = getDocumentByPatientIdentifier("preferences", "notificationsdb", "EDIPI", uniqueId, "timeZone")
    if(document.next() == nil)
      documents.insert({"patientIdentifier" => { "uniqueId" => uniqueId, "assigningAuthority"=> "EDIPI" }, "timeZone" => "(-05:00) America/New_York (Eastern)", "emailAddress" => "test@test.com"})
    end
  end

  def setDefaultNotificationPreferencesForPatient(uniqueId, firstName, lastName)
    @db = Mongo::Connection.new(HOST, PORT).db("notificationsdb")
    documents = @db.collection("notificationPreferences")
    document = getDocumentByPatientIdentifier("notificationPreferences", "notificationsdb", "EDIPI", uniqueId, "optInSelected")
    if(document.next() == nil)
      documents.insert({"patientIdentifier" => { "uniqueId" => uniqueId, "assigningAuthority"=> "EDIPI" }, "firstName" => firstName, "lastName" => lastName, "optInSelected" => true, "dateOfBirth" => {"format" => "MMM dd, yyyy", "value" => "Jan 19, 1900", "valid" => true}})
    end
  end

end