require_relative '../module/DateUtility'

class CalendarPicker
  include DateUtility

  MONTHS = [\
      "January",
      "February",
      "March",
      "April",
      "May",
      "June",
      "July",
      "August",
      "September",
      "October",
      "November",
      "December"
  ]

  def initialize(driver)
    @driver = driver
  end

  def clickNthCalendarPicker(nThCalendar)
    elements = @driver.elements(:css => "i.icon-calendar")
    elements[nThCalendar - 1].when_present
    elements[nThCalendar - 1].click
  end

  def isNthCalendarDisplayed(nThCalendar)
    elements = @driver.elements(:css => "ul.dropdown-menu")
    elements[nThCalendar - 1].when_present
    return elements[nThCalendar - 1].visible?
  end

  def getVisibleCalendar()
    calendar = nil
    elements =  @driver.elements(:css => "ul.dropdown-menu")
    count = elements.length
    for i in 0..( count - 1 )
      if elements[i].visible?
        calendar = elements[i]
      end
    end

    return calendar
  end

  def clickPreviousButton()
    element = getVisibleCalendar()
    previousMonthBtn = element.button(:css =>  "button[ng-click='move(-1)']")
    previousMonthBtn.click
  end

  def clickNextButton()
    element = getVisibleCalendar()
    element.button(:css =>  "button[ng-click='move(1)']").click
  end

  def clickTitleToggleButton()
    element =  getVisibleCalendar()
    element.button(:css => "button[ng-click='toggleMode()']").click
  end

  def getTitleToggleButtonText()
    element =  getVisibleCalendar()
    return element.button(:css => "button[ng-click='toggleMode()']").text
  end

  def getCurrentDay()
    element = getVisibleCalendar()

    begin
      day = element.button(:css => "table[role='grid'] tbody tr button.btn-info")
      if day.text != nil
        #puts "current Day " + day.text
        return day.text
      else
        return ""
      end
    rescue Exception=>e
      return ""
    end


  end

  def getCurrentMonth()
    headerText= getTitleToggleButtonText()
    monthAndYear = headerText.split(" ")
    return monthAndYear[0]
  end

  def getcurrentYear()
    headerText= getTitleToggleButtonText()
    monthAndYear = headerText.split(" ")
    return monthAndYear[1]
  end


  def pickDay(day)
    element =  getVisibleCalendar()
    buttons = element.buttons(:css => "table[role='grid'] tbody tr button")
    count = buttons.length
    for i in (day.to_i - 1)..(count - 1) do
      #puts "## i  " + i.to_s + "  =>" + buttons[i].text

      if(buttons[i].text == day.to_s)
        disabled = buttons[i].attribute_value("disabled")

        if (buttons[i].attribute_value("disabled") == "true")
          puts "button "  + buttons[i].text + " is disabled."
        else
          #puts ("button clicked : " + buttons[i].text)
          buttons[i].click
        end

        break
      end
    end

    return;
  end

  def pickMonth(month)
    element =  getVisibleCalendar()
    buttons = element.buttons(:css => "table[role='grid'] tbody tr button")
    count = buttons.length

    for i in 0..11 do
      #puts ("button Text:  " + buttons[i].text)

      if(buttons[i].text == month.to_s)

        if (buttons[i].attribute_value("disabled") == "true")
          #puts "button "  + buttons[i].text + " is disabled."
        else
          #puts ("button clicked : " + buttons[i].text)
          buttons[i].click
        end
        break
      end
    end


    return;
  end

  def isDayEnabled(day)
    isEnabled = false
    element =  getVisibleCalendar()
    buttons = element.buttons(:css => "table[role='grid'] tbody tr button")
    count = buttons.length
    #puts "buttons count " + count.to_s

    for i in (day.to_i - 1)..(count - 1) do
      #puts ("button " + buttons[i].text)

      if(buttons[i].text == day.to_s)
        if(buttons[i].attribute_value("disabled") == nil)
          isEnabled = true
        elsif (buttons[i].attribute_value("disabled") == "true")
          #puts "button "  + buttons[i].text + " is disabled."
        end

        break
      end

    end

    return isEnabled
  end

  def isMonthOrYearEnabled(text)
    isEnabled = false
    element =  getVisibleCalendar()
    buttons = element.buttons(:css => "table[role='grid'] tbody tr button")
    count = buttons.length
    #puts "buttons count " + count.to_s

    for i in 0..(count - 1) do
      #puts ("button " + buttons[i].text)

      if (buttons[i].text == text.to_s and buttons[i].attribute_value("disabled") == "true")
        #puts "button "  + buttons[i].text + " is disabled."
        break
      elsif (buttons[i].text == text.to_s  and  buttons[i].attribute_value("disabled") == nil)
        isEnabled = true
        break
      end

    end

    return isEnabled
  end


  def areAllMonthsDisplayed()
    isDisplayed = true
    element =  getVisibleCalendar()
    monthButtons = element.buttons(:css => "table[role='grid'] tbody tr button")
    count = monthButtons.length

    for i in 0..(count - 1) do
      if monthButtons[i].text != MONTHS[i]
        isDisplayed = false
        #puts ("Month " + monthButton[i]  + "  != " + MONTHS[i])
        break
      end
    end

    return isDisplayed

  end

  def flipToYear(year)

    #puts "Currnet year : "  + getcurrentYear()
    yearDifference = getcurrentYear().to_i - year.to_i
    #puts "Year Difference : " + yearDifference.to_s

    clickTitleToggleButton

    if (yearDifference < 0)
      0.downto( yearDifference + 1 ) {clickNextButton}
    elsif yearDifference > 0
      for i in 0..(yearDifference - 1) do
        clickPreviousButton()
      end
    end
  end

  def flipToMonth(month)
    month = MONTHS.index(month) + 1
    #puts "Month in # " + month.to_s
    monthDifference = MONTHS.index(getCurrentMonth()).to_i - month.to_i

    #puts "Month Difference: " + monthDifference.to_s

    if monthDifference < 0
      monthDifference.downto(0){  clickNextButton  }
    elsif monthDiff > 0
      for i in 0..(monthDiff - 1) do
        clickPreviousButton()
      end
    end
  end

  def pick(dateStr)
    flipToYear(getYear(dateStr))
    pickMonth(getMonthOfYear(dateStr))

    #puts "Selected Month and Year " + getTitleToggleButtonText

    pickDay(getDay(dateStr))

  end

end