require 'page-object'

class UserGuide
  include PageObject
  h1(:userGuideHeader, :css => 'h1')

  FEATURES_PATH = "/features/"
  GENERAL_USE_PATH = "/general-use/"
  USER_MENU_FEATURES_PATH = "/user-menu-features/"

  GENERAL_USE_LINKS = ["to-use-this-app", "new-users", "navigation", "self-entered-data"]
  FEATURES_LINKS = ["home", "fitness-activity", "health", "support", "learn-more", "notifications"]
  USER_MENU_FEATURES_LINKS = ["about", "help", "login-logout", "launchpad"]
  APP_ABBREVIATIONS = ["app-abbreviations"]

  def clickLink(path)
    cssPath = "a[href='#" + path + "']"
    self.class.link(:link, :css => cssPath)
    self.link_element.click

  end

  def clickBackToTop()
    cssPath = "a[href='#/top']"
    self.class.link(:link, :css => cssPath)
    self.link_element.click
  end

  def getGeneralUsePath(part)
    return GENERAL_USE_PATH + part;
  end

  def getFeaturesPath(part)
    return FEATURES_PATH + part
  end

  def getUserMenuFeaturesPath(part)
    return USER_MENU_FEATURES_PATH + part
  end

  def getFeatureLinks()
    return FEATURES_LINKS
  end

  def getGeneralUseLinks()
    return GENERAL_USE_LINKS
  end

  def getUserMenuLinks()
    return USER_MENU_FEATURES_LINKS
  end

  def isLinkVisible(path)
    cssPath = "a[href='#" + path + "']"
    self.class.link(:link, :css => cssPath)
    return self.link_element.visible?
  end

  def isContentDisplayedOnPage(idStr)
    element = self.class.element(:contentElement, :id => idStr)

    maxY = @driver.execute_script("return document.querySelector('html').offsetHeight - window.innerHeight;").to_i
    elementY = element.location.y
    elementY = elementY > maxY ? maxY : elementY
    currentY = @driver.execute_script("return window.scrollY;").to_i

    return elementY < currentY + 15 && elementY > currentY - 15
  end

  def getHeaderContent(path)
    element = element(:h3, :id => path)
    return element.text
  end

  #####################
  # GENERAL USE
  def getToUseAppContent()
    self.class.element(:appContent, :p, :css => "div.terms-of-cond > section:nth-of-type(1) > section:nth-of-type(1) > p:nth-child(2)")
    return self.appContent_element.text
  end

  def getNewUsersContent()
    self.class.element(:appNewUserContent, :p, :css => "div.terms-of-cond > section:nth-of-type(1) > section:nth-of-type(2) > p:nth-child(2)")
    return self.appNewUserContent_element.text
  end

  def getNavigationParaText(subSectionIndex)
    self.class.element(:navigationParagrph, :p, :css =>  "div.terms-of-cond > section:nth-of-type(1) > section:nth-of-type(3) > p:nth-of-type(" + subSectionIndex.to_s + ")")
    return self.navigationParagrph_element.text
  end

  def getNavigationListElements(subSectionIndex)
    self.class.element(:navigationList, :ul, :css =>  "div.terms-of-cond > section:nth-of-type(1) > section:nth-of-type(3) > ul:nth-of-type(" + subSectionIndex.to_s + ")")
    return self.navigationList_element.text
  end

  #####################
  # USER MENU FEATURES
  # All sections in user guide under 'User Menu Features' are just header + one paragraph, so very repetitive
  def getUserMenuFeatureContent(subSectionIndex)
    self.class.element(:userMenuContent, :p, :css => "div.terms-of-cond > section:nth-of-type(3) > section:nth-of-type(" + subSectionIndex.to_s + ") > p:nth-child(2)")
    return self.userMenuContent_element.text
  end

  #####################
  # MAIN FEATURES
  # Helper to get first paragraph under a given feature sub-section
  def getFeatureParagraphText(subSectionIndex, paragraphIndex)
    self.class.element(:featureParagrph, :p, :css => "div.terms-of-cond > section:nth-of-type(2) > section:nth-of-type(" + subSectionIndex.to_s + ") > p:nth-of-type(" + paragraphIndex.to_s + ")")
    return self.featureParagrph_element.text
  end

  def getFeatureListElements(subSectionIndex, listIndex)
    self.class.element(:featureList, :ul, :css => "div.terms-of-cond > section:nth-of-type(2) > section:nth-of-type(" + subSectionIndex.to_s + ") > ul:nth-of-type(" + listIndex.to_s + ")")

    return self.featureList_element.text
  end

  #####################
  # NOTIFICATIONS
  def getNotificationsParaText(listIndex)
    self.class.element(:notificationPragraph, :p, :css => "div.terms-of-cond > section:nth-of-type(2) > section:nth-of-type(6) > p:nth-of-type(" + listIndex.to_s + ")")
    return self.notificationPragraph_element.text
  end

  #####################
  # ABBREVIATION TABLE
  def getAbbreviationTableRow(rowNumber)
    self.class.element(:tableRow, :tr, :css => "div.terms-of-cond section:nth-of-type(4) table tbody tr:nth-of-type(" + (rowNumber + 1).to_s + ")")
    return self.tableRow_element.text
  end

end