require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'

describe '[Story #CREHAB-12: FitHeart - About]', :regression => true do
  include MongoUtility

  app_title = "VA FitHeart"
  va_fitHeart = "VA FitHeart"
  project_team = "Project Team"
  references = "References"
  acknowledgements = "Acknowledgements"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @lp = @site.launchpad
    @rightNav = @site.rightMenu
    @modal = @site.modal
    @about = @site.about

  end

  after(:all) do
    @site.quitDriver
  end

  context "#AC #CREHAB-85|TC #CREHAB-189: About Selection for FitHeart" do
    it ". When a user selects About from the User Menu, the system displays the About information in a modal screen" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      @landing.appTitle_element.when_visible
      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @rightNav.rightMenu_element.when_visible

      @rightNav.rightMenu
      @rightNav.menuHeader_element.when_visible

      @rightNav.about_element.when_visible
      @rightNav.about

      @about.title_element.when_visible
      Watir::Wait.until {@about.title_element.text == "About"}
      expect(@about.title_element.text).to eq("About")
      expect(@about.version_element.text).to eq("VA FitHeart - v1.0.0")

      expect(@about.isAccordionExpanded(va_fitHeart)).to eq(false)
      expect(@about.isAccordionExpanded(project_team)).to eq(false)
      expect(@about.isAccordionExpanded(references)).to eq(false)
      expect(@about.isAccordionExpanded(acknowledgements)).to eq(false)

      expect(@about.isRightArrowIconDisplayed(va_fitHeart)).to eq(true)
      expect(@about.isRightArrowIconDisplayed(project_team)).to eq(true)
      expect(@about.isRightArrowIconDisplayed(references)).to eq(true)
      expect(@about.isRightArrowIconDisplayed(acknowledgements)).to eq(true)

      expect(@about.isAccordionExpanded(acknowledgements)).to eq(false)

    end
  end

  context "AC #CREHAB-93|TC #CREHAB-189: VA FitHeart Accordion for About " do
    it ". Accordion header: VA Fit Heart" do
      @about.vaFitHeart
      Watir::Wait.until {@about.isAccordionExpanded(va_fitHeart)}
      expect(@about.isAccordionExpanded(va_fitHeart)).to eq(true)
      expect(@about.isAccordionExpanded(project_team)).to eq(false)
      expect(@about.isAccordionExpanded(references)).to eq(false)
      expect(@about.isAccordionExpanded(acknowledgements)).to eq(false)

      expect(@about.getAccordionLabelByHeading(va_fitHeart)).to eq("VA FitHeart")
    end

    it ". Text" do
      expect(@about.getAccordionTextByHeading(va_fitHeart)).to eq("VA FitHeart is designed to help people with heart conditions improve their heart health by increasing physical fitness and providing tools for a healthy lifestyle. This mobile application is not intended to replace professional medical diagnosis or treatment. The main part of the program is designed to resemble a cardiac rehabilitation program. Typical programs guide people through education and fitness training over 12 weeks.")
    end
  end

  context "AC #CREHAB-98|TC CREHAB-189: Project Team Accordion for About" do
    it ". Accordion header " do
      @about.projectTeam
      Watir::Wait.until {@about.isAccordionExpanded(project_team)}
      expect(@about.isAccordionExpanded(va_fitHeart)).to eq(false)
      expect(@about.isAccordionExpanded(project_team)).to eq(true)
      expect(@about.isAccordionExpanded(references)).to eq(false)
      expect(@about.isAccordionExpanded(acknowledgements)).to eq(false)

      expect(@about.getAccordionLabelByHeading(project_team)).to eq("Project Team")
    end

    it ". Text" do
      expect(@about.getAccordionTextByHeading("Project Team")).to eq("Alexis Beatty, MD\nMary Whooley, MD\nYoshimi Fukuoka, RN, PhD\nJudith Moskowitz, PhD, MPH")
    end

  end

  context "AC #CREHAB-101|TC CREHAB-189: References Accordion for About" do
    it ". Accordion header " do
      @about.references
      Watir::Wait.until {@about.isAccordionExpanded(references)}
      expect(@about.isAccordionExpanded(va_fitHeart)).to eq(false)
      expect(@about.isAccordionExpanded(project_team)).to eq(false)
      expect(@about.isAccordionExpanded(references)).to eq(true)
      expect(@about.isAccordionExpanded(acknowledgements)).to eq(false)

      expect(@about.getAccordionLabelByHeading(references)).to eq("References")
    end

    it ". Text" do
      expect(@about.getAccordionTextByHeading(references)).to eq("Smith SC, Jr., Benjamin EJ, Bonow RO, Braun LT, Creager MA, Franklin BA, et al. AHA/ACCF Secondary Prevention and Risk Reduction Therapy for Patients With Coronary and Other Atherosclerotic Vascular Disease: 2011 Update: A Guideline From the American Heart Association and American College of Cardiology Foundation. Circulation. 2011;124(22):2458-73. PMID: 22052934.\nBeatty AL, Fukuoka Y, Whooley MA. Using Mobile Technology for Cardiac Rehabilitation: A Review and Framework for Development and Evaluation. Journal of the American Heart Association. 2013. 2: e000568.")
    end

  end


  context "AC #CREHAB-104|TC CREHAB-189: Acknowledgements Accordion for About" do
    it ". Accordion header " do
      @about.acknowledgements
      Watir::Wait.until {@about.isAccordionExpanded(acknowledgements)}
      expect(@about.isAccordionExpanded(va_fitHeart)).to eq(false)
      expect(@about.isAccordionExpanded(project_team)).to eq(false)
      expect(@about.isAccordionExpanded(references)).to eq(false)
      expect(@about.isAccordionExpanded(acknowledgements)).to eq(true)

      expect(@about.getAccordionLabelByHeading(acknowledgements)).to eq("Acknowledgements")
    end

    it ". Text" do
      expect(@about.getAccordionTextByHeading(acknowledgements)).to eq("VA FitHeart was supported by the following organizations and funding sources:\nUniversity of California, San Francisco (UCSF) is a leading university dedicated to promoting health worldwide through advanced biomedical research, graduate-level education in the life sciences and health professions, and high-quality patient care.\nNational Institutes of Health (NIH) This project was supported by NIH/NCRR UL1 RR024131, NIH/ NCATS KL2TR000143, and NIH/NHLBI F32HL110518. Its contents are solely the responsibility of the authors and do not necessarily represent the official views of the NIH.\nDepartment of Veterans Affairs")
    end

  end

  context "AC #CREHAB-109|TC CREHAB-189" do
    it ". Link Label: EULA" do
      expect(@about.eula_element.text).to eq("EULA")
    end

    it ". Selecting the EULA opens the EULA in a new window" do
      @about.eula
      @site.switchWindowToLast

      Watir::Wait.until {@eula.eulaTitle_element.text == "End User License Agreement"}
      @eula.eulaTitle_element.when_visible
      expect(@eula.eulaTitle_element.text).to eq("End User License Agreement")

      @site.closeBrowser

      @site.switchWindowToFirst
    end
  end

  context "AC #CREHAB-133|TC CREHAB-189: OK Button for About" do
    it ". OK button" do
      expect(@about.ok_element.text).to eq("OK")
    end

    it ". Selecting the OK button closes the modal and returns the user focus to the About menu button" do
      @about.ok
      @about.ok_element.wait_while_present
      expect(@rightNav.isAboutFocused).to eq(true)
    end

  end

end
