require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../rspec_helper'

describe '[Story #CREHAB-10: Fitheart - App Options]
          [Story #CREHAB-375: Adjust Health feature label in navigation]', :regression => true do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @lp = @site.launchpad
    @rightNav = @site.rightMenu
    @leftNav = @site.leftMenu
    @modal = @site.modal

  end

  after(:all) do
    @site.quitDriver
  end

  context "#AC #CREHAB-151|TC #CREHAB-186: App Options Menu for FitHeart
           #AC #CREHAB-503|TC #CREHAB-505: Left Panel from Health/Mood to Health" do
    it ". When an authenticated user views the app header a menu icon is visible in the upper left hand corner" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      @landing.appTitle_element.when_visible

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @leftNav.leftMenu_element.when_visible

    end

    it ". When the user selects the menu icon, a left panel is opened and displays the elements" do
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.title_element.text).to eq("App Options\nClose App Options Menu")

      expect(@leftNav.home_element.visible?).to eq(true)
      expect(@leftNav.home_element.text).to eq("Home\nselected")
      expect(@leftNav.fitness_activity_element.visible?).to eq(true)
      expect(@leftNav.fitness_activity_element.text).to eq("Fitness Activity")
      expect(@leftNav.health_element.visible?).to eq(true)
      expect(@leftNav.health_element.text).to eq("Health")
      expect(@leftNav.support_element.visible?).to eq(true)
      expect(@leftNav.support_element.text).to eq("Support")
      expect(@leftNav.learnMore_element.visible?).to eq(true)
      expect(@leftNav.learnMore_element.text).to eq("Learn More")
      expect(@leftNav.notifications_element.visible?).to eq(true)
      expect(@leftNav.notifications_element.text).to eq("Notifications")

      expect(@leftNav.getNthOption(1)).to eq("Home\nselected")
      expect(@leftNav.getNthOption(2)).to eq("Fitness Activity")
      expect(@leftNav.getNthOption(3)).to eq("Health")
      expect(@leftNav.getNthOption(4)).to eq("Support")
      expect(@leftNav.getNthOption(5)).to eq("Learn More")
      expect(@leftNav.getNthOption(6)).to eq("Notifications")

      @leftNav.leftMenu

    end

  end

end
