require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../rspec_helper'

describe '[Story #CREHAB-3 : FitHeart - Login/Logout ]
          [Story #CREHAB-14:  Return to Launchpad]
          [Story #CREHAB-5: FitHeart - App on Launchpad]
          [Story #CREAHB-926: Updated Launcpahd Icon]', :acceptance => true do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @lp = @site.launchpad
    @rightNav = @site.rightMenu
    @leftNav = @site.leftMenu
    @modal = @site.modal

    setDefaultPreferencesForPatient("D123401")
  end

  after(:all) do
    @site.quitDriver
  end

  context "AC #CREHAB-149|TC #CREHAB-184: User Login/Lotout Feature for FitHeart
           AC #CREHAB-141|TC #CREHAB-191: Launchpad from User Menu for FitHeart
           AC #CREHAB-172|TC #CREHAB-196: FitHeart - App on Launchpad" do
    it ". DS Logon" do
      Watir::Wait.until {@eula.accept_element.exists?}
      @eula.accept_element.when_visible
      @eula.accept

      @landing.login_element.when_visible
      @landing.login

      Watir::Wait.until {@login.login_element.exists?}

      expect(@login.appTitle).to eq("Department of Veterans Affairs")

      @login.login_with("zztest.patient01", "pass", "D123401")
      #goal modal

      if (@modal.no_element.exists?)
        @modal.no
        @modal.no_element.when_not_visible
      end

      @landing.appTitle_element.when_visible
      expect(@landing.appTitle).to eq(app_title)

      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

    end

    it ". After authenticated in the app, the user will be able to open another app and not have to re-authenticate.  When the uer selects Launchpad from the User Menu, the system closes the app and returns to the Launchpad.  The authentication status is retained.  a button labled VA FitHeart is displayed in Launchpad" do
      @rightNav.rightMenu_element.when_visible
      @rightNav.rightMenu
      Watir::Wait.until {@rightNav.launchpad?}
      @rightNav.launchpad_element.when_visible
      @rightNav.launchpad

      #accept Launchpad EULA
      @lp.lpAccept_element.when_visible
      @lp.lpAccept

      Watir::Wait.until { @lp.appTitle == "Launchpad"}
      expect(@lp.appTitle).to eq("Launchpad")

      @lp.footer_element.when_visible
      expect(@lp.footer).to eq("Logged in as patient01, zztest")

      expect(@lp.fitHeart_element.text).to eq("VA FitHeart\nTrack cardiac-related fitness and health activities.")

      @lp.fitHeart

      if (@modal.no_element.exists?)
        @modal.no
        @modal.no_element.when_not_visible
      end

      Watir::Wait.until { @landing.appTitle == app_title}
      @landing.appTitle_element.when_visible
      expect(@landing.appTitle).to eq(app_title)

    end

    it ".  When the user selects Logout, the user is logged out of all VA apps and returned to the Launchpad in an authenticated state."  do
      @rightNav.rightMenu_element.when_visible
      @rightNav.rightMenu
      Watir::Wait.until {@rightNav.logout?}
      @rightNav.logout_element.when_visible
      @rightNav.logout

      Watir::Wait.until { @lp.appTitle == "Launchpad"}
      expect(@lp.appTitle).to eq("Launchpad")

      expect(@lp.footer).to eq("Not logged in")
    end

  end

  context "#AC CREHAB-1071|TC CREHAB-1078: Change of FitHeart Icon" do
    it ". When a user opens the Launchpad, the FitHeart icon (attached) is shown for the FitHeart selection box." do
      @lp.fitHeart_element.when_visible
      expect(@lp.fitHeartIcon_element.visible?).to eq(true)
      @lp.fitHeart

      Watir::Wait.until { @landing.login_element.exists?}
      expect(@landing.login_element.visible?).to eq(true)

    end
  end


end
