require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'

describe '[Story #CREHAB-11: Fitheart - User Menu]
          [Story #CREHAB-13: Fitheart - Help]', :regression => true  do
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @lp = @site.launchpad
    @rightNav = @site.rightMenu
    @modal = @site.modal
    @userGuide = @site.userGuide

  end

  after(:all) do
    @site.quitDriver
  end

  context "#AC #CREHAB-154|TC #CREHAB-188: User Menu Icon for FitHeart" do
    it ". When an authenticated user views the app header a user menu icon is visible in the upper right hand corner" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @rightNav.rightMenu_element.when_visible

    end

    it ". When the user selects the user menu icon, a right panel is opened and displays the menu elements" do
      @rightNav.rightMenu_element.when_visible
      @rightNav.rightMenu

      @rightNav.menuHeader_element.when_visible
      expect(@rightNav.menuHeader_element.text).to eq("User Menu\nClose User Menu")
      @rightNav.about_element.when_visible
      expect(@rightNav.about_element.text).to eq("About")

      @rightNav.help_element.when_visible
      expect(@rightNav.help_element.text).to eq("Help")

      @rightNav.launchpad_element.when_visible
      expect(@rightNav.launchpad_element.text).to eq("Launchpad")

      @rightNav.logout_element.when_visible
      expect(@rightNav.logout_element.text).to eq("Logout")

      expect(@rightNav.getNthUserMenu(1)).to eq("About")
      expect(@rightNav.getNthUserMenu(2)).to eq("Help")
      expect(@rightNav.getNthNavMenu(1)).to eq("Launchpad")
      expect(@rightNav.getNthNavMenu(2)).to eq("Logout")

      @rightNav.rightMenu

    end

  end

  context "AC #CREHAB-155|TC CREHAB-190: User Menu - Help for FitHeart" do
    it ". When a user selects Help from the User Menu, the system displays the Help information in a modal screen" do
      @rightNav.rightMenu_element.when_visible
      @rightNav.rightMenu

      @rightNav.menuHeader_element.when_visible
      expect(@rightNav.menuHeader_element.text).to eq("User Menu\nClose User Menu")

      @rightNav.help_element.when_visible
      expect(@rightNav.help_element.text).to eq("Help")

      @rightNav.help

      @modal.modalTitle_element.when_visible
      Watir::Wait.until {@modal.modalTitle_element.text == "Help"}
      expect(@modal.modalTitle_element.text).to eq("Help")

      expect(@modal.section1_element.text).to eq("Help Desk (toll free): (877)  Weekdays 7 a.m. - 7 p.m. (CT)")
      expect(@modal.section2_element.text).to eq("General Feedback to VA")

      @modal.generalFeedbackToVA

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == "https://"}
      expect(@site.getCurrentURL).to eq("https://")

      @site.closeBrowser
      @site.switchWindowToFirst

    end

    it ". Link label: User Guide <Link to user guide>" do
      expect(@modal.section3_element.text).to eq("User Guide")
      @modal.userGuide

      @site.switchWindowToLast

      Watir::Wait.until {@userGuide.userGuideHeader_element.exists?}
      @userGuide.userGuideHeader_element.when_visible
      expect(@userGuide.userGuideHeader_element.text).to eq("VA FitHeart User Guide")

      @site.closeBrowser

      @site.switchWindowToFirst

    end

    it ". Help OK button " do
      @modal.ok_element.when_visible
      @modal.ok
      @modal.ok_element.when_not_visible

      expect(@modal.isModalPopUpPresent).to eq(false)
    end

  end

end
