#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'

describe '[Story #CREHAB-391: User Guide]', :acceptance => true do
include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @lp = @site.launchpad
    @rightNav = @site.rightMenu
    @modal = @site.modal
    @userGuide = @site.userGuide

  end

  after(:all) do
    @site.quitDriver
  end

  context "#AC #CREHAB-1190|TC CREAHB-1211: User Guide Text" do
    it ". Update user guide to include attached version with Notification section" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @rightNav.rightMenu_element.when_visible
      @rightNav.rightMenu

      @rightNav.help_element.when_visible
      @rightNav.help

      @modal.modalTitle_element.when_visible
      Watir::Wait.until {@modal.modalTitle_element.text == "Help"}
      expect(@modal.modalTitle_element.text).to eq("Help")

      expect(@modal.section3_element.text).to eq("User Guide")
      @modal.userGuide

      @site.switchWindowToLast

      Watir::Wait.until {@userGuide.userGuideHeader_element.exists?}
      @userGuide.userGuideHeader_element.when_visible
      expect(@userGuide.userGuideHeader_element.text).to eq("VA FitHeart User Guide")
    end

    it ". General Use" do
      GENERAL_USE_LINKS = ["to-use-this-app", "new-users", "navigation", "self-entered-data"]
      #puts "Check User Guide: General Use - TO USE THIS APP"
      section = "to-use-this-app"
      path = @userGuide.getGeneralUsePath(section)
      headerContent = "To Use This App"
      bodyContent = "This app requires a Premium (Level 2) DS Logon account so that access can be controlled. You can go to the DoD Self-Service Access Center (DEERS) to find out more information about DS Logon. Before you access the app features, you must also complete a Right of Access (ROA) form and agree to the End User License Agreement (EULA). You only need to sign one Right of Access form to use any VA mobile application. The ROA form covers your request to receive your information from VA systems. All apps require that you agree to a separate EULA, and for web apps you will need to agree to the EULA on each device and browser you use to access the app. If you need to complete either of these documents they will be presented to you."
      @userGuide.clickLink(path)
      expect(@userGuide.getHeaderContent(path).include?(headerContent)).to eq(true)
      expect(@userGuide.getToUseAppContent).to eq(bodyContent)

     # puts "Check User Guide: General Use - NEW USERS"
      section = "new-users"
      path = @userGuide.getGeneralUsePath(section)
      headerContent = "New Users"
      bodyContent =  "The first time you open the app, after logging in, you will be asked to confirm your Notification settings. Settings include establishing your preferred time zone and email address for Notification delivery, and electing to receive notices from Your Care Team. After you save your Notification settings, the standard VA FitHeart home page is shown and the app features in the App Options (left panel) menu become available."
      @userGuide.clickLink(path)
      expect(@userGuide.getHeaderContent(path).include?(headerContent)).to eq(true)
      expect(@userGuide.getNewUsersContent).to eq(bodyContent)

      #puts "Check User Guide: General Use - NAVIGATION"

      expectedFirstPara = "App functionality is shown in two menus: the App Options Menu (menu icon in the upper left corner) and the User Menu (menu icon in the upper right corner)."
      expectedSecondPara = "App Options menu: The app menu includes a list of the features available, including:"
      expectedThirdPara = "User Menu: The user menu allows you to manage your interaction with the app, and includes the following functions:"
      expectedListOne = ["Home", "Fitness Activity", "Health", "Support", "Learn More", "Notifications"].join("\n")
      expectedListTwo = ["About", "Help", "Login/Logout", "Launchpad"].join("\n")

      section = "navigation"
      path = @userGuide.getGeneralUsePath(section)
      headerContent = "Navigation"

      @userGuide.clickLink(path)
      expect(@userGuide.getHeaderContent(path).include?(headerContent)).to eq(true)
      expect(@userGuide.getNavigationParaText(1)).to eq(expectedFirstPara)
      expect(@userGuide.getNavigationParaText(2)).to eq(expectedSecondPara)
      expect(@userGuide.getNavigationParaText(3)).to eq(expectedThirdPara)
      expect(@userGuide.getNavigationListElements(1)).to eq(expectedListOne)
      expect(@userGuide.getNavigationListElements(2)).to eq(expectedListTwo)

    end

    it ". Main Features" do
      firstParagraphHash = {
            "home" => "The Home feature allows you to view your Weekly Activity Goals and set new weekly goal targets for Minutes, Steps and Miles. Each goal displays a progress gauge indicating your progress for the active goal period. The Notification feature allows you to view a count of unread notifications received in the last thirty days, and provides quick access to the notification section.",
            "fitness-activity" => "The Fitness Activity feature allows you to record your fitness activities. Entries that fall within a target goal date range are logged as progress against any activity targets you have set. You may also specify additional information about an activity you have completed, including the activity type (walking, swimming, gardening, etc.) and the intensity level. For fitness activity entries you can:",
            "health" => "The Health feature allows you to enter and track information about a variety of health-related items over time, including:",
            "support" => "The Support feature allows you access various methods of support in the provided topic areas: Mindfulness, People Like You, Family and Friends, and Health Providers:",
            "learn-more" => "The Learn More feature allows you access to quick tips and information on ways to improve the condition of your heart via Fitness, Health, Nutrition and Mood.",
            "notifications" => "Notifications include messages from two primary sources: the Veteran's Administration and VA FitHeart app."

           }

      sectionsListHash = {
          "home" => "",
          "fitness-activity" => ["View and filter a list of entries", "Add or edit an entry", "Save or Cancel an entry"],
          "health" => ["Blood Pressure and Pulse", "Weight", "Glucose", "Cholesterol", "Mood"],
          "support" => ["The Mindfulness option provides guidance how to reduce stress through meditation.", "The People Like You option provides suggestions for talking to people like you about heart disease through an online support group.", "The Family and Friends provides suggestions for receiving support and encouragement from your peers.", "The Health Providers option provides ways to prepare for when you talk to your health care provider about your health or related concerns."],
          "learn-more" => "",
          "notifications" => ["Veteran's Administration notifications include short messages, instructions, encouragements, and educational items on topics such as appointments, health and wellness tips, and suggestions to use other apps. Receipt of these notices is optional, and you may turn them off or on at any time.",
                              "VA FitHeart notices include reminder messages that are generated by the Health feature of the app."]
      }

      secondSectionsListHash = {
          "home" => "",
          "fitness-activity" => "",
          "health" =>  ["View and filter a list of entries", "View a graphical representation of a set of entries", "Add or edit an entry", "Save, Cancel or Delete an entry", "Set up a notification to receive motivational health messages or reminders about the specific health area."],
          "support" => "",
          "learn-more" => "",
          "notifications" => ""
      }


      def checkFeatureHelper(section, sectionIndex, paragraphHash, listHash, secondListHash)
        puts "Check User Guide: Features - " + section.dup.upcase

        expectedHeader = section.dup
        expectedHeader.sub! "-", " "
        expectedHeader = "2." + sectionIndex.to_s + " " +  expectedHeader
        expectedList = ""
        secondExpectedList = ""

        count = listHash[section].length
        for j in 0..(count - 1)
          expectedList = expectedList + listHash[section][j]

          if(j < count - 1)
            expectedList = expectedList + "\n"
          end
        end

        count = secondListHash[section].length
        for i in 0..(count - 1)
          secondExpectedList = secondExpectedList + secondListHash[section][i]

          if(i < count - 1)
            secondExpectedList = secondExpectedList + "\n"
          end
        end

        expectedFirstPara = paragraphHash[section]

        sectionList = ""
        secondSectionList = ""

        if listHash[section] != ""
          sectionList = @userGuide.getFeatureListElements(sectionIndex, 1)
        end

        if secondListHash[section] != ""
          secondSectionList = @userGuide.getFeatureListElements(sectionIndex, 2)
        end

        path = @userGuide.getFeaturesPath(section)
        @userGuide.clickLink(path)

        expect(@userGuide.getHeaderContent(path).downcase).to eq(expectedHeader)
        expect(@userGuide.getFeatureParagraphText(sectionIndex, 1)).to eq(expectedFirstPara)
        expect(sectionList).to eq(expectedList)
        expect(secondSectionList).to eq(secondExpectedList)

        if (section == "notifications")
          expect(@userGuide.getNotificationsParaText(2)).to eq("You may update your Notifications preferences by selecting the Settings button (gear icon) inside the Notification feature. Setting options include your time zone preference, notification email address, and the ability to opt-in to health messages provided by the VA.")
          expect(@userGuide.getNotificationsParaText(3)).to eq("When you log into the app, you will see a count of your unread notifications received within the last 30 days. You may access your notifications by clicking the Notification button on the landing page, or you may navigate to the Notification feature using the left navigation panel menu. To refine the view, you may filter the notification list by date range and read or unread status. All notifications are retained in the Notification feature list until you elect to remove them from the list using the Delete feature, which is accessed within the detailed view of the Notification itself.")
        end

        sleep 1
      end

      @userGuide.getFeatureLinks.each_with_index do |section, index|
          # index + 1 because Ruby arrays start at index 0
        checkFeatureHelper(section, index + 1, firstParagraphHash, sectionsListHash, secondSectionsListHash)
      end
    end

    it ". User Menu Features" do
      map = {
          "about"         =>   "The About feature provides general information about the App, including the version number, and a link to the End User License Agreement.",
          "help"          =>   "The Help feature provides support contact information and link to this User Guide.",
          "login-logout"  =>   "The Login/Logout feature allows you to enable or disable your authentication session for ALL VA apps. Note that once you logout you will need to login again to use any app and you will no longer see an indicator that you are logged in at the bottom right of the app.",
          "launchpad"     =>   "The Launchpad feature allows you to quickly access the apps included on the VA Launchpad, including apps that you may not be aware of that the VA is providing to help improve your health and healthcare."
        }

      def checkUserMenuContentHelper(section, sectionIndex, map)
        puts "Check User Guide: User Menu Features - " + section.dup.upcase

        expectedContent = map[section]
        expectedHeader = section.dup

        #login/logout section has path with '-' but header with '/' between "login" and "logout" parts
        if expectedHeader.include?("login")
          expectedHeader.sub! "-", "/"
        end

        path = @userGuide.getUserMenuFeaturesPath(section)
        @userGuide.clickLink(path)

        expect(@userGuide.getHeaderContent(path).downcase.include?(expectedHeader)).to eq(true)
        expect(@userGuide.getUserMenuFeatureContent(sectionIndex)).to eq(expectedContent)
        @userGuide.clickBackToTop

        sleep 1
      end

      map.keys.each_with_index do |section, index|
        checkUserMenuContentHelper(section, index + 1, map)
      end

    end

    it ". App Abbreviations" do

      tableMap = {
          "bpm"      =>  "beats per minute",
          "lb/lbs"   =>  "pound/pounds",
          "mg/dl"    =>  "milligrams per deciliter",
          "min/mins" =>  "minute/minutes",
          "mmHg"     =>  "millimeters of mercury"
      }
      path = "/app-abbreviations"
      expect(@userGuide.isLinkVisible(path)).to eq(true)
      @userGuide.clickLink(path)

      def checkTableRowContent(abbreviation, rowIndex, map)
        row = @userGuide.getAbbreviationTableRow(rowIndex)
        expectedContent = abbreviation + " " + map[abbreviation]
        expect(row).to eq(expectedContent)
      end

      tableMap.keys.each_with_index do |rowAbbreviation, index|
        # we want to start checking table content starting at row 2
        checkTableRowContent(rowAbbreviation, index, tableMap)
      end

    end

  end

end
