#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../rspec_helper'


describe '[Story #CREHAB-416: Fitness Activity - Congratulatory Message on Save]', :acceptance => true do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeCollection("fitness", "healthfactordb")
    removeCollection("fitnessgoals", "healthfactordb")

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @error = @site.error
    @goal = @site.goal
    @fitness_listview = @site.fitness_listview
    @fitness_activity = @site.fitness_activity

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1,"%m/%d/%Y")


    @goal_reached_congratulatory_messages = [\
                                          "Amazing job!",
                                          "Fantastic!",
                                          "Awesome!",
                                          "You did it!",
                                          "Excellent!",
                                          "Congratulations!"]

    @goal_reached_education_messages = [\
                                          "You reached your goal!",
                                          "You’re taking an important step to improve your heart health.",
                                          "Keep up the good work by being active on most days.",
                                          "Reward yourself for a job well done.",
                                          "Reaching your goals can help improve your mood.",
                                          "Remember what you did to help you achieve your goal to use in the future.",
                                          "Share your good news with family and friends.",
                                          "Think about ways that you can do a little bit more next week.",
                                          "Being active can improve your health and mood. Plus, it’s fun!",
                                          "Make a commitment to staying active.",
                                          "Vary your fitness routine to help keep you engaged.",
                                          "Staying active is important for a healthy heart.",
                                          "Make a plan to stay active long-term.",
                                          "Continue to set new goals and stay active!",
                                          "To reduce stay engaged, consider switching between different forms of exercise. ",
                                          "Congratulations on your achievement!",
                                          "Motivate others to be active with you. ",
                                          "Noticing how good exercise makes you feel can motivate you in the future",
                                          "Great job! Remember, regular exercise is crucial for good health.",
                                          "You have taken a step towards a healthier lifestyle. Celebrate your achievement."]

    @standard_congratulatory_messages = [\
                                        "Well done!",
                                        "Way to go!",
                                        "Keep it up!",
                                        "Nice job!"]

    @standard_education_messages = [\
                                  "Safety first! Let your doctor know if you have chest pain or trouble breathing.",
                                  "Being active is an important part of heart health.",
                                  "A healthy diet can reduce your risk of heart problems.",
                                  "SMART goals are Specific, Measureable, Attainable, Relevant, and Timely.",
                                  "Try to replace junk foods like chips and sweets with vegetables and fruits!",
                                  "Try to get at least 150 minutes/week of physical activity.",
                                  "Try to include 4-5 servings of fruits and vegetables in your daily diet.",
                                  "Choose oil-based salad dressings (Italian) over the creamier options (Ranch).",
                                  "Pushing yourself to work up a sweat can improve your heart health.",
                                  "Limit your portion sizes and salt intake, and try to avoid high fat and sugary foods.",
                                  "It can help to have a fitness buddy. Do you have friends or family who can help?",
                                  "Engaging in at least moderate intensity activities is best for your heart.",
                                  "Drink less than 36 ounces/week of sugary beverages (like soda or juice).",
                                  "Use olive oil to cook instead of butter.",
                                  "Being active can improve your health and mood. Plus, it’s fun!",
                                  "Taking the stairs or parking your car far away can help increase physical activity.",
                                  "Did you know that red meats have more unhealthy fat than chicken and fish?",
                                  "Beans, peas, lentils, and nuts are excellent sources of protein, vitamins, and fiber.",
                                  "Whole grains contain fiber, which helps control cholesterol and glucose levels!",
                                  "Varying your fitness routine can help keep you engaged.",
                                  "Egg whites contain protein while the yolk has fat. Limit yolks to two per week.",
                                  "Salt makes your heart work harder. Eat less salt to lower your blood pressure.",
                                  "Setting goals can help you stay healthy. Are you meeting your fitness goals?",
                                  "Remember, both good diet and exercise will help you stay healthy!",
                                  "Did you know that people who are more physically active live longer?",
                                  "You can count how many steps you take in a day to measure your activity.",
                                  "If you are overweight, losing weight is an important step towards better health.",
                                  "Try to eat 3 or more servings of whole grains (oats, brown rice, barley) per day.",
                                  "Reducing sedentary time (like watching TV) can improve your health.",
                                  "Making a plan to stay active over the long-term can help you to stay healthy.",
                                  "	Look at nutrition labels on foods to make healthy food choices."]
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      @landing.appTitle_element.when_visible

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      @leftNav.fitness_activity_element.when_visible
      @leftNav.fitness_activity

      @fitness_listview.primaryHeader_element.when_visible
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

    end
  end

  context "#AC CREHAB-1176|TC CREHAB-1188: Standard Message Modal
           #AC CREHAB-1175|TC CREHAB-1188: Goal Reached Modal"  do
    it ". If the user has no goals set, display Standard message modal" do
      date = getDateNthDaysFromNow(-12, "%m/%d/%Y")
      time =  "12:30 PM"
      duration = "90"
      steps = "5000"
      distance = "12345.1"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType = "standard"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)
    end

    it ". If the save does not result in the user meeting any goal targets, display Standard message modal" do
      #set weekly goal
      resetCollection("fitnessgoals", "healthfactordb", "weekly_goals2")
      sixDaysAgo = Time.now - 6.days
      now = Time.now
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc7", "startDate", sixDaysAgo)
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc7", "endDate", now)
      @site.refreshBrowser

      #"target" : { "minutes" : 500, "steps" : 50000, "miles" : 50 }

      date = getDateNthDaysFromNow(0, "%m/%d/%Y")
      time =  getNthMinFromNow(-10)
      duration = "90"
      steps = "5000"
      distance = "5"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType = "standard"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

    end

    it ". if the save updates a goal where the target has already been met, display Standard message modal" do
      #distance goal reached
      date = getDateNthDaysFromNow(-1, "%m/%d/%Y")
      time =  getNthMinFromNow(-8)
      duration = "90"
      steps = "3000"
      distance = "45"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType = "goal reached"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

      #target goal has already met
      date = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      time =  getNthMinFromNow(-6)
      duration = ""
      steps = ""
      distance = "6"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType = "standard"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

    end

    it ". The system then displays a modal message based on whether any goal (if set) has just been met:" do
      #duration goal reached
      date = getDateNthDaysFromNow(-3, "%m/%d/%Y")
      time =  getNthMinFromNow(-4)
      duration = "320"
      steps = ""
      distance = "6"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType ="goal reached"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

      #steps goal reached
      date = getDateNthDaysFromNow(-4, "%m/%d/%Y")
      time =  getNthMinFromNow(-2)
      duration = "2"
      steps = "50000"
      distance = "6"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType ="goal reached"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

      #all target goals already met
      date = getDateNthDaysFromNow(-5, "%m/%d/%Y")
      time =  getNthMinFromNow(-15)
      duration = "4"
      steps = "2"
      distance = "1"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType ="standard"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)
    end
  end

  context "#AC CREHAB-1177|TC CREHAB-1188: Congratulatory and Reinforcing/Educational messagging in random order" do
    #manual testing - For each modal type, cycle sequentially through the list of content in order for congratulatory and reinforcing/educational messages below.
    it ". Edit to existing entries do not trigger messages." do
      @fitness_listview.tableList_elements[0].click

      @fitness_activity.primaryHeader_element.when_visible
      expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

      @fitness_activity.date_element.when_visible
      @fitness_activity.steps = 25
      @fitness_activity.save
      Watir::Wait.until {@fitness_listview.primaryHeader_element.text == "Fitness Activity"}
      expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")
    end

    it ". Delete all fitness activity and validate the modal messages" do
      count = @fitness_listview.tableList_elements.length
      for i in 0..(count -1)
        @fitness_listview.tableList_elements[0].click

        @fitness_activity.primaryHeader_element.when_visible
        expect(@fitness_activity.primaryHeader_element.text).to eq("Edit Fitness Activity Entry")

        @fitness_activity.delete_element.when_visible
        @fitness_activity.delete

        @modal.yes_element.when_visible
        expect(@modal.yes_element.text).to eq("Yes")
        @modal.yes
        @modal.yes_element.when_not_visible

        @fitness_listview.primaryHeader_element.when_visible
        expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")
        Watir::Wait.until {@fitness_listview.tableList_elements.length ==  (count - (i + 1))}

      end

      date = getDateNthDaysFromNow(0, "%m/%d/%Y")
      time =  getNthMinFromNow(-2)
      duration = "90"
      steps = "3"
      distance = ""
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType = "standard"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

      #distance goal reached
      date = getDateNthDaysFromNow(-1, "%m/%d/%Y")
      time =  getNthMinFromNow(-4)
      duration = "40"
      steps = "3"
      distance = ""
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType = "standard"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

      #target goal has already met
      date = getDateNthDaysFromNow(-2, "%m/%d/%Y")
      time =  getNthMinFromNow(-5)
      duration = "2"
      steps = ""
      distance = "6"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType = "standard"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

      #duration goal reached
      date = getDateNthDaysFromNow(-3, "%m/%d/%Y")
      time =  getNthMinFromNow(-6)
      duration = "2"
      steps = "3"
      distance = "6"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType ="standard"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

      #steps goal reached
      date = getDateNthDaysFromNow(-4, "%m/%d/%Y")
      time =  getNthMinFromNow(-7)
      duration = "2"
      steps = "50000"
      distance = "6"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType ="goal reached"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

      #all target goals already met
      date = getDateNthDaysFromNow(-5, "%m/%d/%Y")
      time =  getNthMinFromNow(-8)
      duration = "4"
      steps = "2"
      distance = "1"
      activity = "Walking"
      intensity = "Easy"
      notes = "N/A"
      msgType ="standard"
      addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)

    end


  end

  def addFitnessActivityAndVerifyModal(date, time, duration, steps, distance, activity, intensity, notes, msgType)
    count = @fitness_listview.tableList_elements.length

    @fitness_listview.add_element.when_visible
    @fitness_listview.add

    @fitness_activity.primaryHeader_element.when_visible
    expect(@fitness_activity.primaryHeader_element.text).to eq("Add Fitness Activity Entry")
    @fitness_activity.date = date
    @fitness_activity.time = time
    @fitness_activity.duration = duration
    @fitness_activity.steps = steps
    @fitness_activity.distance = distance
    @fitness_activity.activitySelect = activity
    @fitness_activity.intensitySelect = intensity
    @fitness_activity.notes = notes

    @fitness_activity.save

    Watir::Wait.until { @modal.continue2_element.exists? }
    @modal.continue2_element.when_visible
    congratulatory_msg_arr = nil
    education_msg_arr = nil

    if msgType == "standard"
      congratulatory_msg_arr =  @standard_congratulatory_messages
      education_msg_arr = @standard_education_messages
    elsif msgType == "goal reached"
      congratulatory_msg_arr =  @goal_reached_congratulatory_messages
      education_msg_arr = @goal_reached_education_messages
    end

    isDisplayed = false
    congratulatory_msg_arr.each_with_index do | congratulatory_message, i |
      if(@modal.modalTitle_element.text == congratulatory_message)
        isDisplayed = true
        break
      end
    end
    expect(isDisplayed).to eq(true)

    isDispalyed = false
    education_msg_arr.each_with_index do | education_message, i |
      if(@modal.section1_element.text == education_message)
        isDisplayed = true
        break
      end
    end
    expect(isDisplayed).to eq(true)

    #Continue button, clicking closes the modal and returns the user to the Fitness Activity table list view.
    @modal.continue2
    @modal.continue2_element.when_not_visible

    @fitness_listview.contentLoaded_element.when_present(timeout=10)
    Watir::Wait.until { @fitness_listview.tableList_elements.length == count + 1 }
    expect(@fitness_listview.tableList_elements.length).to eq(count + 1)

    @fitness_listview.primaryHeader_element.when_visible
    expect(@fitness_listview.primaryHeader_element.text).to eq("Fitness Activity")

  end

  def addWeeklyGoal(minutes, steps, miles, startDate)
    @fitness_listview.weeklyActivityGoals_element.when_visible
    @fitness_listview.weeklyActivityGoals

    @goal.modalTitle_element.when_visible
    expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

    @goal.minutes = minutes
    @goal.steps = steps
    @goal.miles = miles
    @goal.startDate = startDate

    @goal.save

    if (@modal.continue2_element.exists?)
      @modal.continue2
      @modal.continue2_element.when_not_visible
    end
  end

end
