require 'rspec'
require 'watir-webdriver'


require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../shared_context'
require_relative 'health_helper'


describe '[Story #CREHAB-380: Add]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @title = "Cholesterol"

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @detail = @site.health_detail
    @list_view = @site.list_view

    @error = @site.error
    @calendar = @site.calendar_picker

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")

    @EXPECTED_ROW_0_FORM =[\
            "* required field",
            "* Date:",
            "MM/DD/YYYY",
            "This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open Calendar Control.",
            "Day selected is " + getDayofWeek(@today),
            getDayofWeek(@today),
            "* Lab Results: (provide at least one result to save)",
            "Total Serum Cholesterol (mg/dl):",
            "Between 0 and 600",
            "Low-Density Lipoprotein (LDL) (mg/dl):",
            "Between 0 and 300",
            "High-Density Lipoprotein (HDL) (mg/dl):",
            "Between 0 and 300",
            "Triglycerides (mg/dl):",
            "Between 0 and 600",
            "Test done while fasting?:",
            "Select",
            "Yes",
            "No",
            "Unknown",
            "Note:",
            "250 character maximum",
            "250 characters left",
            "Notification:",
            "Status: Off",
            "",
            "To edit the Cholesterol notification, select the Notification button on the previous screen.",
            "Save Cancel"]
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      navigateToHealth(app_title)

      @health.cholesterol_element.when_visible
      @health.cholesterol_element.click

      Watir::Wait.until {@list_view.secondaryHeader_element.text == "Health"}
      Watir::Wait.until {@list_view.primaryHeader_element.text == @title}

      expect(@list_view.secondaryHeader_element.text).to eq("Health")
      expect(@list_view.primaryHeader_element.text).to eq(@title)
    end
  end

  context "#AC CREHAB-874|TC CREHAB-913: Add New Entry for FitHeart Cholesterol" do
    it_behaves_like "Add Button Validation"

    it_behaves_like  "Labels Validation"

    it_behaves_like "Date Validation"

    it_behaves_like "Note Validation"
  end

  context "#AC CREHAB-874|TC CREHAB-913: Add New Entry for FitHeart Cholesterol " do
    it ". Duplicate entries on the same date are not permitted." do
      @detail.date = @today
      @detail.totalSerumCholesterol = 345

      @detail.save

      Watir::Wait.until {@list_view.secondaryHeader_element.text == "Health"}
      Watir::Wait.until {@health.cholesterolAmount_element.exists?}

      Watir::Wait.until {@health.cholesterolAmount_element.text.strip == "345 mg/dl"}
      expect(@health.cholesterolAmount_element.text.strip).to eq("345 mg/dl")
      expect(@health.cholesterolDate_element.text).to eq("Today")

      @list_view.primaryHeader_element.when_visible
      Watir::Wait.until {@list_view.primaryHeader_element.text == @title}

      expect(@list_view.primaryHeader_element.text).to eq(@title)

      @list_view.tableList_elements[0].when_visible

      @list_view.addButton_element.when_visible
      @list_view.addButton_element.click

      Watir::Wait.until {@detail.primaryHeader_element.text == "Add " + @title + " Entry"}
      expect(@detail.primaryHeader_element.text).to eq("Add " + @title + " Entry")

      @detail.date_element.when_visible

      @detail.totalSerumCholesterol = 350

      @detail.save

      errorMessages = ["An entry has already been saved with the same date. Please change your date and save again."]
      verifyErrorMessage(errorMessages)

      clickCancelAndFocusToHeader
    end

    it ". Lab Results (primary label, show as required); " do
      @list_view.addButton

      Watir::Wait.until {@detail.primaryHeader_element.text == "Add " + @title + " Entry"}
      expect(@detail.primaryHeader_element.text).to eq("Add " + @title + " Entry")
      @detail.date_element.when_visible
      @detail.save_element.when_visible

      @detail.save
      errorMessages = ["Provide at least one Lab Result value to save your entry."]
      verifyErrorMessage(errorMessages)

    end

    it ". Total Serum Cholesterol (mg/dl) (secondary label, textbox, numeric, constrain format to whole number, show light grey guidance text: Between 0 and 600) " do
      expect(@detail.totalSerumCholesterol_element.attribute("placeholder")).to eq("Between 0 and 600")

      @detail.totalSerumCholesterol = 601
      @detail.save

      errorMessages = ["Total Serum Cholesterol (mg/dl) is outside the expected range. Please enter a value between 0 and 600."]
      verifyErrorMessage(errorMessages)

    end

    it ". Low-Density Lipoprotein (LDL) (mg/dl) (secondary label, textbox, numeric, constrain format to whole number, show light grey guidance text: Between 0 and 300) " do
      expect(@detail.ldLipoprotein_element.attribute("placeholder")).to eq("Between 0 and 300")
      @detail.ldLipoprotein = 301
      @detail.save

      errorMessages = ["Low-Density Lipoprotein (LDL) (mg/dl) is outside the expected range. Please enter a value between 0 and 300."]
      verifyErrorMessage(errorMessages)
    end

    it ". High-Density Lipoprotein (HDL) (mg/dl) (secondary label, textbox, numeric, constrain format to whole number, show light grey guidance text: Between 0 and 300) " do
      expect(@detail.hdLipoprotein_element.attribute("placeholder")).to eq("Between 0 and 300")
      @detail.hdLipoprotein = 301
      @detail.save

      errorMessages = ["High-Density Lipoprotein (HDL) (mg/dl) is outside the expected range. Please enter a value between 0 and 300."]
      verifyErrorMessage(errorMessages)
    end

    it ". Triglycerides (mg/dl) (secondary label, textbox, numeric, constrain format to whole number, show light grey guidance text: Between 0 and 600) " do
      expect(@detail.triglycerides_element.attribute("placeholder")).to eq("Between 0 and 600")
      @detail.triglycerides = 601
      @detail.save

      errorMessages = ["Triglycerides (mg/dl) is outside the expected range. Please enter a value between 0 and 600."]
      verifyErrorMessage(errorMessages)
    end

    it ". Test done while fasting? (single select, drop down, values: Select, Yes, No, Unknown, default to Select)" do
      expect(@detail.fastingSelect).to eq("Select")
      selectArr = ["Select", "Yes", "No", "Unknown"]
      expect(@detail.fastingSelect_options).to eq(selectArr)

      for i in 0..(selectArr.length - 1)
        @detail.fastingSelect = selectArr[i]
        expect(@detail.fastingSelect).to eq(selectArr[i])
      end
    end

    it ". Save, Cancel options are available" do
      expect(@detail.save_element.visible?).to eq(true)
      expect(@detail.cancel_element.visible?).to eq(true)
    end

  end


end
