require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/MongoUtility'
require_relative '../rspec_helper'


describe '[Story #CREHAB-376: Health Feature Options]', :regression => true do
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @health = @site.health
    @leftNav = @site.leftMenu
    @modal = @site.modal
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "#AC CREHAB-376|CREHAB-658: Health Features List Items" do
    it ". When a user opens Health feature, the user will see the following items in the main panel/column 2" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.learnMore_element.visible?).to eq(true)
      expect(@leftNav.learnMore_element.text).to eq("Learn More")

      @leftNav.health_element.when_visible
      @leftNav.health

      @health.primaryHeader_element.when_visible
      expect(@health.primaryHeader_element.text).to eq("Health")
      expect(@health.secondaryHeader_element.text).to eq("Health")

      expect(@health.bloodPressureAndPulse_element.text.strip).to eq(Health::BLOOD_PRESSURE_PULSE_LABEL)
      expect(@health.weight_element.text.strip).to eq("Weight")
      expect(@health.glucose_element.text.strip).to eq("Glucose")
      expect(@health.cholesterol_element.text.strip).to eq("Cholesterol")
      expect(@health.mood_element.text.strip).to eq("Mood")

      expect(@health.secondaryOptions_elements[0].text.strip).to eq(Health::BLOOD_PRESSURE_PULSE_LABEL)
      expect(@health.secondaryOptions_elements[1].text.strip).to eq("Weight")
      expect(@health.secondaryOptions_elements[2].text.strip).to eq("Glucose")
      expect(@health.secondaryOptions_elements[3].text.strip).to eq("Cholesterol")
      expect(@health.secondaryOptions_elements[4].text.strip).to eq("Mood")

    end

  end

  context "#AC CREHAB-653|TC CREHAB-658: Detail panel header Health" do
    it ". 	Detailed panel will have header Health and will display a message until an item is selected" do
      Watir::Wait.until {@health.placeholderMessage_element.visible?}
      @health.placeholderMessage_element.when_visible
      expect(@health.placeholderMessage_element.text).to eq("Select an item from the Health list to view details or add a new Health entry.")
    end

  end

end
