require 'rspec'
require 'watir-webdriver'


require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative 'health_helper'
require_relative '../shared_context'


describe '[Story #CREHAB-406: Mood - Add]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @list_view = @site.list_view
    @detail = @site.health_detail
    @select_mood = @site.select_mood
    @error = @site.error
    @calendar = @site.calendar_picker

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
    @title = "Mood"

    @EXPECTED_ROW_0_FORM = [\
                  "* required field",
                  "* Date:",
                  "MM/DD/YYYY",
                  "This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open Calendar Control.",
                  "Day selected is " +  getDayofWeek(@today),
                  getDayofWeek(@today),
                  "* Time:",
                  "hh:mm AM/PM",
                  "This time selection control is not optimized for screen readers. Please directly enter the time in the text box provided. Open Time Picker",
                  "* Mood:",
                  "Select Mood",
                  "Note:",
                  "250 character maximum",
                  "250 characters left",
                  "Notification:",
                  "Status: Off",
                  "",
                  "To edit the Mood notification, select the Notification button on the previous screen.",
                  "Save Cancel"]
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
      navigateToHealth(app_title)
      navigateTo(@title)
    end
  end

  context "#AC CREHAB-886|CREHAB-905: Add New Entry for FitHeart Mood" do
    it_behaves_like "Add Button Validation"

    it_behaves_like  "Labels Validation"

    it_behaves_like "Date Validation"

    it_behaves_like "Time Validation"

    it_behaves_like "Note Validation"

  end

  context "#AC CREHAB-886|CREHAB-905: Add New Entry for FitHeart Mood" do
    it ". Mood (required, display button below label, button, label: Select Mood), Save button, Smiley face icons" do
      @detail.selectMood_element.when_visible
      @detail.selectMood

      Watir::Wait.until {@select_mood.modalTitle_element.text== "Select Mood"}
      expect(@select_mood.modalTitle_element.text).to eq("Select Mood")

      expect(@select_mood.instructionalText_element.text).to eq("Rate your mood on a scale of 1 to 10.")
      expect(@select_mood.inputRangeKeyText_element.text).to eq("(1=Worst, 10=Best)")
      @select_mood.moveMoodRangeToRight(2)

      expect(@select_mood.veryBadIcon_element.visible?).to eq(true)
      expect(@select_mood.badIcon_element.visible?).to eq(true)
      expect(@select_mood.neutralIcon_element.visible?).to eq(true)
      expect(@select_mood.goodIcon_element.visible?).to eq(true)
      expect(@select_mood.veryGoodIcon_element.visible?).to eq(true)

      @select_mood.ok3

      Watir::Wait.until {@detail.moodRate_element.text == "7 / 10"}
      expect(@detail.moodRate_element.text).to eq("7 / 10")

      expect(@detail.smileyFace_element.attribute("class").include?(HealthDetail::SMILEY_FACE_GOOD)).to eq(true)

    end

    it ". Cancel button, selecting closes the modal, no changes are saved, user is returned to the add/edit screen with the previously saved mood rating still intact" do
      @detail.selectMood_element.when_visible
      @detail.selectMood

      Watir::Wait.until {@select_mood.modalTitle_element.text== "Select Mood"}
      expect(@select_mood.modalTitle_element.text).to eq("Select Mood")

      Watir::Wait.until {@select_mood.slider_element.attribute("aria-valuenow")== "7"}
      expect(@select_mood.slider_element.attribute("aria-valuenow")).to eq("7")

      @select_mood.cancel

      Watir::Wait.until {@detail.moodRate_element.text == "7 / 10"}
      expect(@detail.moodRate_element.text).to eq("7 / 10")

      expect(@detail.smileyFace_element.attribute("class").include?(HealthDetail::SMILEY_FACE_GOOD)).to eq(true)

    end

    it ". Standard Save and Cancel options are available.  Duplicate entries not allowed" do
      expect(@detail.save_element.exists?).to eq(true)
      expect(@detail.cancel_element.exists?).to eq(true)

      time = getNthHoursFromNow(0)
      date = @detail.date

      @detail.time = time
      @detail.save

      Watir::Wait.until {@list_view.secondaryHeader_element.text == "Health"}
      Watir::Wait.until {@health.moodAmount_element.exists?}
      Watir::Wait.until {@health.moodAmount_element.text == "7 / 10"}
      expect(@health.moodAmount_element.text).to eq("7 / 10")
      expect(@health.moodDate_element.text).to eq("Today")

      @list_view.primaryHeader_element.when_visible
      Watir::Wait.until {@list_view.primaryHeader_element.text == "Mood"}

      expect(@list_view.primaryHeader_element.text).to eq("Mood")

      @list_view.tableList_elements[0].when_visible

      @list_view.addButton_element.when_visible
      @list_view.addButton_element.click

      @detail.date_element.when_visible

      @detail.date = date
      @detail.time = time

      @detail.selectMood_element.when_visible
      @detail.selectMood

      Watir::Wait.until {@select_mood.modalTitle_element.text== "Select Mood"}
      expect(@select_mood.modalTitle_element.text).to eq("Select Mood")

      @select_mood.moveMoodRangeToLeft(3)
      @select_mood.ok3

      Watir::Wait.until {@detail.moodRate_element.text == "2 / 10"}
      expect(@detail.moodRate_element.text).to eq("2 / 10")

      expect(@detail.smileyFace_element.attribute("class").include?(HealthDetail::SMILEY_FACE_VERYBAD)).to eq(true)

      @detail.save
      errorMessages = ["An entry has already been saved with the same date and time. Please change your date and/or time and save again."]
      verifyErrorMessage(errorMessages)

      clickCancelAndFocusToHeader

    end
  end
end