require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../spec/shared_context'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../health/health_helper'

describe '[Story #CREHAB-405: Mood - Table and List View
          [Story #CREHAB-412: Mood - Graph]
          [Story #CREHAB-514: Phone User : Options control]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @list_view = @site.list_view
    @detail = @site.health_detail
    @select_mood = @site.select_mood
    @error = @site.error
    @calendar = @site.calendar_picker
    @learn_more = @site.learnMore
    @notification_detail = @site.notification_detail
    @date_filter = @site.date_filter
    @title = "Mood"

    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")
    @tomorrow = getDateNthDaysFromNow(1, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")
    @sixYearsAgo = getDateNthYearsFromNow(-6, "%m/%d/%Y")
    @towDaysAgo = getDateNthDaysFromNow(-2, "%m/%d/%Y")
    @fourDaysAgo = getDateNthDaysFromNow(-4, "%m/%d/%Y")
    @fiveDaysAgo = getDateNthDaysFromNow(-5, "%m/%d/%Y")
    @eightDaysAgo = getDateNthDaysFromNow(-8, "%m/%d/%Y")
    @threeMonthAgo = getDateNthMonthsFromNow(-3, "%m/%d/%Y")
    @hundredDaysAgo = getDateNthDaysFromNow(-100, "%m/%d/%Y")
    @sixYearsAndOneDayAgo = getDateNthDaysFromGivenDate(@sixYearsAgo, -1)
    @twoYearsAgo = getDateNthYearsFromNow(-2, "%m/%d/%Y")
    @fourYearsAgo = getDateNthYearsFromNow(-4, "%m/%d/%Y")
    @oneYearAgo = getDateNthYearsFromNow(-1, "%m/%d/%Y")

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
      navigateToHealth(app_title)
    end
  end

  context "#AC CREHAB-882|TC CREHAB-911: Sub Feature Selection for FitHeart Mood" do
    it ". When a user opens the Health feature, the system displays Mood in the Main screen (column 2):" do
      @health.primaryHeader_element.when_visible
      expect(@health.secondaryHeader_element.text).to eq("Health")
      expect(@health.primaryHeader_element.text).to eq("Health")

      @health.mood_element.when_visible
      expect(@health.leftSubMenulist_elements[4].text.strip).to eq("Mood")

      navigateTo(@title)
    end

    it ". Smile icon is displayed as left-justified on the button" do
      expect(@health.smileyIconForMood_element.visible?).to eq(true)
    end

    it ". The date of the most recent tracker entry is displayed below the Mood label. If date = today display Today if date = Today-1 display Yesterday, otherwise display the actual date (MM/DD/YYYY)
          The most recent mood entry is displayed as center aligned with other values in the far right column (format: <rating> / 10)" do

      moodRateSlide = 3
      moveLeftOrRight = "right"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_GOOD
      addMood(@fiveDaysAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      expect(@health.moodDate_element.text).to eq(@fiveDaysAgo)
      expect(@health.moodAmount_element.text).to eq("8 / 10")

      moodRateSlide = 1
      moveLeftOrRight = "left"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_BAD
      addMood(@yesterday, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      expect(@health.moodDate_element.text).to eq("Yesterday")
      expect(@health.moodAmount_element.text).to eq("4 / 10")

      moodRateSlide = 4
      moveLeftOrRight = "left"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_VERYBAD
      addMood(@today, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      expect(@health.moodDate_element.text).to eq("Today")
      expect(@health.moodAmount_element.text).to eq("1 / 10")
    end

  end

  context "#AC CREHAB-1186|TC CREHAB-1195: Phone View Option Button
           #AC CREAHB-1187|TC CREHAB-1195: Options control display for Phone user" do
    include_examples "Phone User: Options control"
  end

  context "#AC CREHAB-890|TC CREHAB-915: Graph Button & Features for Mood " do
    include_examples "Graph/Table Button Validation"
  end

  context "#AC CREHAB-883|TC CREHAB-911: Sub Featue Default view for fitheart Mood" do
    include_examples "Date Filter Validation"
  end

  context "#AC CREHAB-884|TC CREHAB-911: Ficter Action and Results display for FitHeart Mood" do
    it ". When a user selects the filter button, the system updates the table view to show records that match the filter options entered." do
      removeCollection("mood", "medicalhistorydb")
      @site.refreshBrowser
      Watir::Wait.until { @list_view.primaryHeader_element.text == "Mood" }
      expect(@list_view.primaryHeader_element.text).to eq("Mood")

      @date_filter.startDate = getDateNthYearsFromNow(-6, "%m/%d/%Y")
      @date_filter.endDate = @today
      @date_filter.filter

      Watir::Wait.until { @list_view.primaryHeader_element.text == "Mood" }
      expect(@list_view.primaryHeader_element.text).to eq("Mood")

      moodRateSlide = 3
      moveLeftOrRight = "right"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_GOOD
      addMood(@towDaysAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      moodRateSlide = 1
      moveLeftOrRight = "left"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_BAD
      addMood(@fourDaysAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      moodRateSlide = 4
      moveLeftOrRight = "left"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_VERYBAD
      addMood(@fiveDaysAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      moodRateSlide = 5
      moveLeftOrRight = "right"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_VERYGOOD
      addMood(@eightDaysAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      moodRateSlide = 1
      moveLeftOrRight = "right"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_NEUTRAL
      addMood(@threeMonthAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      moodRateSlide = 2
      moveLeftOrRight = "left"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_BAD
      addMood(@hundredDaysAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      moodRateSlide = 3
      moveLeftOrRight = "left"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_VERYBAD
      addMood(@oneYearAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      moodRateSlide = 4
      moveLeftOrRight = "right"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_VERYGOOD
      addMood(@fourYearsAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      moodRateSlide = 4
      moveLeftOrRight = "left"
      smileyFaceIcon = HealthDetail::SMILEY_FACE_VERYBAD
      addMood(@sixYearsAgo, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

      @date_filter.startDate = getDateNthMonthsFromNow(-3, "%m/%d/%Y")
      @date_filter.endDate = @today
      @date_filter.filter

      Watir::Wait.until { @list_view.primaryHeader_element.text == "Mood" }
      expect(@list_view.primaryHeader_element.text).to eq("Mood")

      Watir::Wait.until {@list_view.tableList_elements.length == 5}
      expect(@list_view.tableList_elements.length).to eq(5)
      expect(@list_view.getFocusedElement == @list_view.firstTableRow_element).to eq(true)

      verifyDefaultTableList

    end
    it ". After the filter button is selected, the focus moves to the returned results, first row, or the No results found message" do
      @date_filter.startDate = getDateNthYearsFromNow(-2, "%m/%d/%Y")
      @date_filter.endDate = @today
      @date_filter.filter

      Watir::Wait.until { @list_view.primaryHeader_element.text == "Mood" }
      expect(@list_view.primaryHeader_element.text).to eq("Mood")

      Watir::Wait.until {@list_view.tableList_elements.length == 7}
      expect(@list_view.tableList_elements.length).to eq(7)

      expect(@list_view.getFocusedElement == @list_view.firstTableRow_element).to eq(true)

      expect(@list_view.getTextForRowCol(6, 1)).to eq("3")
      expect(@list_view.getIconClassForRowCol(6, 1).include?(HealthDetail::SMILEY_FACE_BAD)).to eq(true)
      expect(@list_view.getTextForRowCol(6, 2).include?(@hundredDaysAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(6, 2), "%m/%d/%Y %I:%M %p"))

      expect(@list_view.getTextForRowCol(7, 1)).to eq("2")
      expect(@list_view.getIconClassForRowCol(7, 1).include?(HealthDetail::SMILEY_FACE_VERYBAD)).to eq(true)
      expect(@list_view.getTextForRowCol(7, 2).include?(@oneYearAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(7, 2), "%m/%d/%Y %I:%M %p"))

      @date_filter.startDate = getDateNthYearsFromNow(-6, "%m/%d/%Y")
      @date_filter.endDate = @today
      @date_filter.filter

      Watir::Wait.until { @list_view.primaryHeader_element.text == "Mood" }
      expect(@list_view.primaryHeader_element.text).to eq("Mood")

      Watir::Wait.until {@list_view.tableList_elements.length == 9}
      expect(@list_view.tableList_elements.length).to eq(9)

      expect(@list_view.getFocusedElement == @list_view.firstTableRow_element).to eq(true)
      expect(@list_view.getTextForRowCol(8, 1)).to eq("9")
      expect(@list_view.getIconClassForRowCol(8, 1).include?(HealthDetail::SMILEY_FACE_VERYGOOD)).to eq(true)
      expect(@list_view.getTextForRowCol(8, 2).include?(@fourYearsAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(8, 2), "%m/%d/%Y %I:%M %p"))

      expect(@list_view.getTextForRowCol(9, 1)).to eq("1")
      expect(@list_view.getIconClassForRowCol(9, 1).include?(HealthDetail::SMILEY_FACE_VERYBAD)).to eq(true)
      expect(@list_view.getTextForRowCol(9, 2).include?(@sixYearsAgo)).to eq(true)
      expect(isDateFormatValid?(@list_view.getTextForRowCol(9, 2), "%m/%d/%Y %I:%M %p"))

      verifyNoResultsMessage(@today, @today)
    end

    it ". When a user views a filter control, a Reset button is available located to the right of the Filter Button. (Alt text Reset)" do
      expect(@date_filter.reset_element.visible?).to eq(true)
      expect(@date_filter.reset_element.text).to eq("Reset")
    end

    it ". When the user selects the reset button, all filter field defaults are restored and the filter action is applied so that the table list view is updated per the default filter settings." do
      verifyReset(5)
      verifyDefaultTableList
    end
  end

  context "#AC CREHAB-885|TC CREHAB-911: Navigation for FitHeart Mood" do
    include_examples "Back button validation for Phone view"
  end

  context "#AC CREHAB-885|TC CREHAB-911: Reset for FitHeart Mood" do
    it ". When a user views a filter control, a Reset button is available located to the right of the Filter Button. (Alt text Reset)" do
      #covered in #AC CREHAB-884|TC CREHAB-911 test script
    end

    it ". When the user selects the reset button, all filter field defaults are restored and the filter action is applied so that the table list view is updated per the default filter settings." do
      #covered in #AC CREHAB-884|TC CREHAB-911
    end
  end


  def verifyDefaultTableList
    expect(@list_view.getTextForRowCol(1, 1)).to eq("8")
    expect(@list_view.getIconClassForRowCol(1, 1).include?(HealthDetail::SMILEY_FACE_GOOD)).to eq(true)
    expect(@list_view.getTextForRowCol(1, 2).include?(@towDaysAgo)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(1, 2), "%m/%d/%Y %I:%M %p"))

    expect(@list_view.getTextForRowCol(2, 1)).to eq("4")
    expect(@list_view.getIconClassForRowCol(2, 1).include?(HealthDetail::SMILEY_FACE_BAD)).to eq(true)
    expect(@list_view.getTextForRowCol(2, 2).include?(@fourDaysAgo)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(2, 2), "%m/%d/%Y %I:%M %p"))

    expect(@list_view.getTextForRowCol(3, 1)).to eq("1")
    expect(@list_view.getIconClassForRowCol(3, 1).include?(HealthDetail::SMILEY_FACE_VERYBAD)).to eq(true)
    expect(@list_view.getTextForRowCol(3, 2).include?(@fiveDaysAgo)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(3, 2), "%m/%d/%Y %I:%M %p"))

    expect(@list_view.getTextForRowCol(4, 1)).to eq("10")
    expect(@list_view.getIconClassForRowCol(4, 1).include?(HealthDetail::SMILEY_FACE_VERYGOOD)).to eq(true)
    expect(@list_view.getTextForRowCol(4, 2).include?(@eightDaysAgo)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(4, 2), "%m/%d/%Y %I:%M %p"))

    expect(@list_view.getTextForRowCol(5, 1)).to eq("6")
    expect(@list_view.getIconClassForRowCol(5, 1).include?(HealthDetail::SMILEY_FACE_NEUTRAL)).to eq(true)
    expect(@list_view.getTextForRowCol(5, 2).include?(@threeMonthAgo)).to eq(true)
    expect(isDateFormatValid?(@list_view.getTextForRowCol(5, 2), "%m/%d/%Y %I:%M %p"))
  end

end
