require 'rspec'
require 'watir-webdriver'


require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative 'health_helper'
require_relative '../shared_context'

describe '[Story #CREHAB-404: Weight - Add]
          [Story #CREHAB-853: Add Duplicate Entries for Fitness Activity, Weight, Glucose]', :regression => true do
  include DateUtility
  include MongoUtility
  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @list_view = @site.list_view
    @detail = @site.health_detail
    @error = @site.error
    @calendar = @site.calendar_picker

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
    @title = "Weight"
    @time = getNthHoursFromNow(0)

    @EXPECTED_ROW_0_FORM = [\
              "* required field",
              "* Date:",
              "MM/DD/YYYY",
              "This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open Calendar Control.",
              "Day selected is " + getDayofWeek(@today),
              getDayofWeek(@today),
              "* Time:",
              "hh:mm AM/PM",
              "This time selection control is not optimized for screen readers. Please directly enter the time in the text box provided. Open Time Picker",
              "* Weight (lbs):",
              "Between 0.0 and 999.9",
              "Note:",
              "250 character maximum",
              "250 characters left",
              "Notification:",
              "Status: Off",
              "",
              "To edit the Weight notification, select the Notification button on the previous screen.",
              "Save Cancel"
            ]
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
      navigateToHealth(app_title)
      navigateTo(@title)
    end
  end

  context "#AC CREHAB-660|CREHAB-684: Add New Entry for FitHeart Weigth" do
    it_behaves_like "Add Button Validation"

    it_behaves_like  "Labels Validation"

    it_behaves_like "Date Validation"

    it_behaves_like "Time Validation"

    it_behaves_like "Note Validation"
  end

  context "#AC CREHAB-660|CREHAB-684: Add New Entry for FitHeart Weigth" do
     it ". Weight (lbs) (required, numeric, textbox, show light grey guidance text: Between 0.0 and 999.9)" do
       @detail.date_element.when_visible
       expect(@detail.weight_element.attribute("required")).to eq("true")
       expect(@detail.weight_element.attribute("placeholder")).to eq("Between 0.0 and 999.9")

       @detail.save_element.click

       errorMessages = ["Weight (lbs) field is required."]
       verifyErrorMessage(errorMessages)

       @detail.weight = 9999.9
       expect(@detail.weight).to eq("999.9")
     end

     it ". Standard Save and Cancel options are available." do
       expect(@detail.save_element.exists?).to eq(true)
       expect(@detail.cancel_element.exists?).to eq(true)
       @detail.weight = "150.2"
       @detail.time = @time

       @detail.save

       Watir::Wait.until {@list_view.secondaryHeader_element.text == "Health"}
       Watir::Wait.until {@health.weightAmount_element.exists?}
       Watir::Wait.until {@health.weightAmount_element.text == "150.2 lbs"}
       expect(@health.weightAmount_element.text).to eq("150.2 lbs")

       @list_view.primaryHeader_element.when_visible
       Watir::Wait.until {@list_view.primaryHeader_element.text == "Weight"}

       expect(@list_view.primaryHeader_element.text).to eq("Weight")

       @list_view.tableList_elements[0].when_visible

       @list_view.addButton_element.when_visible
       @list_view.addButton_element.click

       @detail.date_element.when_visible
       @detail.cancel_element.when_visible
       @detail.cancel_element.click

       @list_view.primaryHeader_element.when_visible
       expect(@list_view.primaryHeader_element.text).to eq("Weight")

     end
  end

  context "#AC CREHAB-1073|TC CREHAB-1076: Display error message for duplicate entires" do
    it ". Duplicate Error validation" do
      @list_view.addButton_element.when_visible
      @list_view.addButton_element.click

      Watir::Wait.until { @detail.date_element.visible? }
      expect(@detail.primaryHeader_element.text).to eq("Add Weight Entry")

      @detail.date_element.when_visible
      @detail.weight = 224
      @detail.time = @time

      @detail.save_element.when_visible
      @detail.save_element.click

      errorMessages = ["An entry has already been saved with the same date and time. Please change your date and/or time and save again."]
      verifyErrorMessage(errorMessages)


    end
  end

end
