
######################################
# Common Methods for Health Features #
#    AddFeature and VerifyFeature    #
######################################

def navigateToHealth(app_title)
  expect(@landing.appTitle).to eq(app_title)
  @landing.welcomeTitleForAuthenticatedUser_element.when_visible
  expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

  @leftNav.leftMenu_element.when_visible
  @leftNav.leftMenu

  @leftNav.health_element.when_visible
  @leftNav.health

  Watir::Wait.until { @health.primaryHeader_element.text == "Health" }

  expect(@health.secondaryHeader_element.text).to eq("Health")
  expect(@health.primaryHeader_element.text).to eq("Health")
end

def addBP (date, systolic, diastolic, pulse, position, note)
  count = @list_view.tableList_elements.length

  @list_view.addButton_element.when_visible
  @list_view.addButton

  Watir::Wait.until { @detail.primaryHeader_element.text == "Add Blood Pressure & Pulse Entry" }
  expect(@detail.primaryHeader_element.text).to eq("Add Blood Pressure & Pulse Entry")

  @detail.date_element.when_visible

  @detail.date = date
  Watir::Wait.until { @detail.date == date }
  expect(@detail.date).to eq(date)

  @detail.systolic = systolic
  @detail.diastolic = diastolic
  @detail.pulse = pulse
  @detail.positionSelect = position
  @detail.note = note
  @detail.save
  @detail.save_element.when_not_visible

  Watir::Wait.until { @list_view.tableList_elements.length == count + 1 }
end

def verifyBP(date, systolic, diastolic, pulse, position, note)

  Watir::Wait.until { @list_view.getTextForRowCol(1, 1) != "" }

  index = @list_view.getRowIndex(systolic, 1)
  @list_view.tableList_elements[index.to_i - 1].when_visible
  @list_view.tableList_elements[index.to_i - 1].click

  Watir::Wait.until {@detail.primaryHeader_element.text == "Edit " + @title + " Entry"}
  expect(@detail.primaryHeader_element.text).to eq("Edit " + @title + " Entry")
  @detail.date_element.when_visible

  expect(@detail.date).to eq(date)
  expect(@detail.systolic).to eq(systolic)
  expect(@detail.diastolic).to eq(diastolic)
  expect(@detail.pulse).to eq(pulse)
  expect(@detail.positionSelect).to eq(position)
  expect(@detail.note).to eq(note)

  @detail.cancel_element.when_visible
  @detail.cancel

  Watir::Wait.until {@list_view.primaryHeader_element.text == @title}
  expect(@list_view.primaryHeader_element.text).to eq(@title)

end


def addWeight (date, weight, note)
  Watir::Wait.until { @list_view.addButton_element.visible? }

  count = @list_view.tableList_elements.length

  @list_view.addButton_element.when_visible
  @list_view.addButton_element.click

  Watir::Wait.until { @detail.date_element.visible? }
  expect(@detail.primaryHeader_element.text).to eq("Add Weight Entry")

  @detail.date_element.when_visible
  @detail.date = date
  @detail.weight = weight
  @detail.note = note

  @detail.save_element.when_visible
  @detail.save_element.click

  @detail.save_element.when_not_visible
  Watir::Wait.until { @list_view.tableList_elements.length == count + 1 }
end


def verifyWeight(date, note, weight)
  Watir::Wait.until { @detail.primaryHeader_element.text == "Edit Weight Entry" }
  expect(@detail.primaryHeader_element.text).to eq("Edit Weight Entry")

  Watir::Wait.until { @detail.date == date }

  expect(@detail.date).to eq(date)
  expect(@detail.weight).to eq(weight)
  expect(@detail.note).to eq(note)
end

def addGlucose(date, glucoseLevel, tested, note)
  Watir::Wait.until { @list_view.addButton_element.visible? }

  count = @list_view.tableList_elements.length

  @list_view.addButton_element.when_visible
  @list_view.addButton

  Watir::Wait.until {@detail.primaryHeader_element.text == "Add Glucose Entry"}
  expect(@detail.primaryHeader_element.text).to eq("Add Glucose Entry")

  Watir::Wait.until {@detail.date_element.visible? == true}
  @detail.date_element.when_visible

  @detail.date = date
  @detail.glucoseLevel = glucoseLevel
  @detail.testedSelect = tested
  @detail.note = note

  @detail.save
  @detail.save_element.when_not_visible
  Watir::Wait.until { @list_view.tableList_elements.length == count + 1 }

end

def verifyGlucose(date, glucoseLevel, tested, note)
  Watir::Wait.until { @list_view.getTextForRowCol(1, 1) != "" }
  index = @list_view.getRowIndex(glucoseLevel, 2)
  @list_view.tableList_elements[index.to_i - 1].when_visible
  @list_view.tableList_elements[index.to_i - 1].click

  Watir::Wait.until { @detail.primaryHeader_element.text == "Edit " + @title + " Entry" }
  expect(@detail.primaryHeader_element.text).to eq("Edit " + @title + " Entry")

  Watir::Wait.until { @detail.date == date }

  expect(@detail.date).to eq(date)
  expect(@detail.glucoseLevel).to eq(glucoseLevel)
  expect(@detail.testedSelect).to eq(tested)
  expect(@detail.note).to eq(note)

  clickCancelAndFocusToHeader

end

def addCholesterol (date, totalSerumCholesterol, ldLipo, hdLipo, tryiglycerides, fasting, note)
  Watir::Wait.until { @list_view.addButton_element.visible? }
  count = @list_view.tableList_elements.length

  @list_view.addButton_element.when_visible
  expect(@list_view.addButton_element.attribute("title")).to eq("Add")

  @list_view.addButton

  Watir::Wait.until { @detail.primaryHeader_element.text == "Add Cholesterol Entry" }
  expect(@detail.primaryHeader_element.text).to eq("Add Cholesterol Entry")

  @detail.date_element.when_visible
  @detail.date = date
  @detail.totalSerumCholesterol = totalSerumCholesterol
  @detail.ldLipoprotein = ldLipo
  @detail.hdLipoprotein = hdLipo
  @detail.triglycerides = tryiglycerides

  @detail.fastingSelect =fasting
  @detail.note = note
  @detail.save

  @detail.save_element.when_not_visible
  Watir::Wait.until { @list_view.tableList_elements.length == count + 1 }
end

def verifyCholesterol(date, totalSerumCholesterol, idLipo, hdLipo, tryiglycerides, fasting, note)
  Watir::Wait.until { @list_view.getTextForRowCol(1, 1) != "" }

  index = @list_view.getRowIndex(totalSerumCholesterol, 2)
  @list_view.tableList_elements[index.to_i - 1].when_visible
  @list_view.tableList_elements[index.to_i - 1].click

  Watir::Wait.until {@detail.primaryHeader_element.text == "Edit " + @title + " Entry"}
  expect(@detail.primaryHeader_element.text).to eq("Edit " + @title + " Entry")
  @detail.date_element.when_visible

  expect(@detail.date).to eq(date)
  expect(@detail.totalSerumCholesterol).to eq(totalSerumCholesterol)
  expect(@detail.ldLipoprotein).to eq(idLipo)
  expect(@detail.hdLipoprotein).to eq(hdLipo)
  expect(@detail.triglycerides).to eq(tryiglycerides)
  expect(@detail.fastingSelect).to eq(fasting)
  expect(@detail.note).to eq(note)

  @detail.cancel_element.when_visible
  @detail.cancel

  Watir::Wait.until {@list_view.primaryHeader_element.text == @title}
  expect(@list_view.primaryHeader_element.text).to eq(@title)
end

def addMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)
  count = @list_view.tableList_elements.length

  @list_view.addButton_element.when_visible
  @list_view.addButton

  Watir::Wait.until {@detail.primaryHeader_element.text == "Add Mood Entry"}
  expect(@detail.primaryHeader_element.text).to eq("Add Mood Entry")

  @detail.date_element.when_visible

  @detail.date = date
  @detail.selectMood_element.when_visible
  @detail.selectMood

  Watir::Wait.until { @select_mood.modalTitle_element.text== "Select Mood" }
  expect(@select_mood.modalTitle_element.text).to eq("Select Mood")

  if moveLeftOrRight == "left" then
    @select_mood.moveMoodRangeToLeft(moodRateSlide)
    @select_mood.ok3

    Watir::Wait.until { @detail.moodRate_element.text ==  (5 - moodRateSlide.to_i).to_s + " / 10" }
    expect(@detail.moodRate_element.text).to eq((5 - moodRateSlide.to_i).to_s + " / 10")

  else #right
    @select_mood.moveMoodRangeToRight(moodRateSlide)
    @select_mood.ok3

    Watir::Wait.until { @detail.moodRate_element.text ==  (5 + moodRateSlide.to_i).to_s + " / 10" }
    expect(@detail.moodRate_element.text).to eq((5 + moodRateSlide.to_i).to_s + " / 10")
  end

  expect(@detail.smileyFace_element.attribute("class").include?(smileyFaceIcon)).to eq(true)
  @detail.save

  @detail.save_element.when_not_visible
  Watir::Wait.until { @list_view.tableList_elements.length == count + 1 }
end

def verifyMood(date, moodRateSlide, moveLeftOrRight, smileyFaceIcon)

  index = -1
  selectedRate = -1

  Watir::Wait.until { @list_view.getTextForRowCol(1, 1) != "" }

  if moveLeftOrRight == "left" then
    selectedRate = 5 - moodRateSlide
    index = @list_view.getRowIndex(selectedRate, 1)
  else
    selectedRate = 5 +  moodRateSlide
    index = @list_view.getRowIndex(selectedRate, 1)
  end

  @list_view.tableList_elements[index.to_i - 1].when_visible
  @list_view.tableList_elements[index.to_i - 1].click

  Watir::Wait.until {@detail.primaryHeader_element.text == "Edit Mood Entry"}
  expect(@detail.primaryHeader_element.text).to eq("Edit Mood Entry")
  @detail.date_element.when_visible
  expect(@detail.date).to eq(date)
  expect(@detail.moodRate_element.text).to eq(selectedRate.to_s + " / 10")
  expect(@detail.smileyFace_element.attribute("class").include?(smileyFaceIcon)).to eq(true)

  @detail.selectMood_element.when_visible
  @detail.selectMood

  Watir::Wait.until {@select_mood.modalTitle_element.text== "Select Mood"}
  expect(@select_mood.modalTitle_element.text).to eq("Select Mood")

  Watir::Wait.until {@select_mood.slider_element.attribute("aria-valuenow")== selectedRate.to_s}
  expect(@select_mood.slider_element.attribute("aria-valuenow")).to eq(selectedRate.to_s)

  @select_mood.cancel

  Watir::Wait.until {@detail.moodRate_element.text == selectedRate.to_s + " / 10"}
  expect(@detail.moodRate_element.text).to eq(selectedRate.to_s + " / 10")

  @detail.cancel

  Watir::Wait.until {@list_view.primaryHeader_element.text == "Mood"}
  expect(@list_view.primaryHeader_element.text).to eq("Mood")

end


#####Common

def verifyBackButtonConfirmationPrompt
  @detail.backButton

  @modal.modalTitle_element.when_visible
  expect(@modal.modalTitle_element.text).to eq("Confirmation")
  expect(@modal.section1_element.text).to eq("You have not saved your changes. If you wish to continue without saving select CONTINUE, otherwise select RETURN to return to the entry screen.")
  @modal.return_element.when_visible
  expect(@modal.return_element.text).to eq("Return")
  @modal.return
  @modal.return_element.when_not_visible
  expect(@detail.getFocusedElement == @detail.primaryHeader_element).to eq(true)

  @detail.backButton_element.when_visible
  @detail.backButton

  @modal.modalTitle_element.when_visible
  expect(@modal.modalTitle_element.text).to eq("Confirmation")
  expect(@modal.section1_element.text).to eq("You have not saved your changes. If you wish to continue without saving select CONTINUE, otherwise select RETURN to return to the entry screen.")
  @modal.continue_element.when_visible
  expect(@modal.continue_element.text).to eq("Continue")
  @modal.continue
  @modal.continue_element.when_not_visible

  @list_view.primaryHeader_element.when_visible
  expect(@list_view.primaryHeader_element.text).to eq(@title)
end

def navigateAwayAndVerifyConfirmationPrompt
  @leftNav.leftMenu_element.when_visible
  @leftNav.leftMenu

  @leftNav.home_element.when_visible
  @leftNav.home

  @modal.modalTitle_element.when_visible
  expect(@modal.modalTitle_element.text).to eq("Confirmation")
  expect(@modal.section1_element.text).to eq("You have not saved your changes. If you wish to continue without saving select CONTINUE, otherwise select RETURN to return to the entry screen.")
  @modal.return_element.when_visible
  expect(@modal.return_element.text).to eq("Return")
  @modal.return
  @modal.return_element.when_not_visible
  expect(@detail.getFocusedElement() == @detail.primaryHeader_element).to eq(true)

  @leftNav.leftMenu_element.when_visible
  @leftNav.leftMenu

  @leftNav.learnMore_element.when_visible
  @leftNav.learnMore

  @modal.modalTitle_element.when_visible
  expect(@modal.modalTitle_element.text).to eq("Confirmation")
  expect(@modal.section1_element.text).to eq("You have not saved your changes. If you wish to continue without saving select CONTINUE, otherwise select RETURN to return to the entry screen.")
  @modal.continue_element.when_visible
  expect(@modal.continue_element.text).to eq("Continue")
  @modal.continue
  @modal.continue_element.when_not_visible

  @learn_more.primaryHeader_element.when_visible
  expect(@learn_more.primaryHeader_element.text).to eq("Learn More")

  @leftNav.leftMenu_element.when_visible
  @leftNav.leftMenu

  @leftNav.health_element.when_visible
  @leftNav.health

end

def verifyFocusMoveToFirstRow (numberOfEntries)
  @date_filter.startDate = @threeMonthAgo
  @date_filter.endDate = @today
  @date_filter.filter
  Watir::Wait.until { @list_view.primaryHeader_element.text == @title }
  expect(@list_view.primaryHeader_element.text).to eq(@title)
  Watir::Wait.until { @list_view.tableList_elements.length == numberOfEntries }
  expect(@list_view.tableList_elements.length).to eq(numberOfEntries)

  expect(@list_view.getFocusedElement() == @list_view.firstTableRow_element).to eq(true)
end

def verifyNoResultsMessage (startDate, endDate)
  @date_filter.startDate = startDate
  @date_filter.endDate = endDate
  @date_filter.filter

  Watir::Wait.until { @list_view.noResultsMessage_element.exists? }
  expect(@list_view.noResultsMessage_element.text).to eq("No results were found with the current filters. Use the Add button above to create a new record.")
  expect(@list_view.getFocusedElement() == @list_view.noResultsMessage_element).to eq(true)
end

def verifyReset(numberOfEntries)
  @date_filter.reset_element.when_visible
  @date_filter.reset

  Watir::Wait.until { @list_view.tableList_elements.length == numberOfEntries }
  expect(@list_view.tableList_elements.length).to eq(numberOfEntries)

  expect(@date_filter.startDate).to eq(@threeMonthAgo)
  expect(@date_filter.endDate).to eq(@today)
end

def verifySaveCancelDeleteButtons
  @list_view.tableList_elements[0].when_visible
  @list_view.tableList_elements[0].click

  Watir::Wait.until { @detail.primaryHeader_element.text == "Edit " + @title + " Entry" }
  expect(@detail.primaryHeader_element.text).to eq("Edit " + @title + " Entry")
  @detail.date_element.when_visible

  expect(@detail.save_element.visible?).to eq(true)
  expect(@detail.delete_element.visible?).to eq(true)
  expect(@detail.cancel_element.visible?).to eq(true)

  @detail.cancel_element.when_visible
  @detail.cancel

  Watir::Wait.until { @list_view.primaryHeader_element.text == @title }
  expect(@list_view.primaryHeader_element.text).to eq(@title)
end


def navigateTo(healthOption)
  if healthOption == "Blood Pressure & Pulse"
    @health.bloodPressureAndPulse_element.when_visible
    @health.bloodPressureAndPulse
  elsif healthOption == "Cholesterol"
    @health.cholesterol_element.when_visible
    @health.cholesterol
  elsif healthOption == "Glucose"
    @health.glucose_element.when_visible
    @health.glucose
  elsif healthOption == "Mood"
    @health.mood_element.when_visible
    @health.mood
  elsif healthOption == "Weight"
    @health.weight_element.when_visible
    @health.weight
  end

  Watir::Wait.until { @list_view.primaryHeader_element.visible?}
  Watir::Wait.until { @list_view.primaryHeader_element.text == healthOption }
  expect(@list_view.primaryHeader_element.text).to eq(healthOption)
end

def clickCancelAndFocusToHeader
  @detail.cancel_element.when_visible
  @detail.cancel

  Watir::Wait.until { @list_view.primaryHeader_element.text == @title }
  expect(@list_view.primaryHeader_element.text).to eq(@title)
  expect(@list_view.getFocusedElement() == @list_view.primaryHeader_element).to eq(true)
end


