require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'


describe '[Story #CREHAB-6: FitHeart - Landing Page Views]
          [Story #CREHAB-261: Landing Page - Weekly Activity Goals Display (status and instructional text)
          [Story #CREHAB-262: Set Weekly Activity Goals - Modal]
          [Story #CREHAB-678: Set Start Date to Today when a new goal is created]
          [Story #CREAHB-512: Phone User - Landing Page: Weekly Activity Goals progress graphs]
          [Story #CREHAB-699: Enhancing: Add number of units left under goal activities]', :acceptance => true do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeCollection("fitness", "healthfactordb")
    removeCollection("fitnessgoals", "healthfactordb")

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @lp = @site.launchpad
    @rightNav = @site.rightMenu
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @goal = @site.goal
    @error = @site.error
    @learnMore = @site.learnMore
    @carousel = @site.carousel

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
    @tomorrow = getDateNthDaysFromNow(1,  "%m/%d/%Y")
    @nintyDaysLater = getDateNthDaysFromNow(90,  "%m/%d/%Y")

    @EXPECTED_ROW_0_FORM = [\
                      "Activity Goal:",
                      "Minutes Per Week:",
                      "Steps Per Week:",
                      "Miles Per Week:",
                      "Start Date:",
                      "MM/DD/YYYY",
                      "This calendar control is not optimized for screen readers. Please directly enter dates in the text box provided. Open Calendar Control.",
                      ]
  end

  after(:all) do
    @site.quitDriver
  end

  context "#AC #CREHAB-68|TC CREHAB-185:  Unauthenticated User for FitHeart Landing Page" do
    it ".Landing page given a user has not authenticated - Page Header, Title, Current Date and Message" do
      Watir::Wait.until {@eula.accept_element.exists?}
      @eula.accept_element.when_visible
      expect(@eula.accept_element.text).to eq("Accept")
      @eula.accept

      @landing.appTitle_element.when_visible
      expect(@landing.appTitle).to eq(app_title)

      @landing.welcomeTitle_element.when_visible
      expect(@landing.welcomeTitle).to eq("Welcome to VA FitHeart")

      today = getDateNthDaysAgo(0, "%B %d, %Y")
      expect(@landing.currentDate).to eq(today)

      expectedDateAndMessage = today + "\nSelect Login to access the VA FitHeart app.\nLearn more about this application by selecting the user menu button above.\nLearn more about this application by selecting the button above.\nWishing you good health and welfare,\nYour VA Care Team"

      expect(@landing.dateAndMessage).to eq(expectedDateAndMessage)
    end

    it ". Right Menu" do
      @rightNav.rightMenu_element.when_visible
      @rightNav.rightMenu

      @rightNav.about_element.when_visible
      expect(@rightNav.about_element.text).to eq("About")
      @rightNav.login_element.when_visible
      expect(@rightNav.login_element.text).to eq("Login")
      @rightNav.launchpad_element.when_visible
      expect(@rightNav.launchpad_element.text).to eq("Launchpad")

      @rightNav.rightMenu

    end

    it ". Login Button" do
      @landing.login_element.when_visible
      expect(@landing.login_element.visible?).to eq(true)

      @landing.login
      Watir::Wait.until {@login.login_element.exists?}
      @login.userName_element.when_visible
      expect(@login.userName_element.visible?).to eq(true)
      @login.password_element.when_visible
      expect(@login.password_element.visible?).to eq(true)
    end

  end

  context "AC #CREHAB-69|TC CREHAB-185: Authenticated User for FitHeart Landing Page" do
    it ". Page Header, Welcome Title, Current Date" do
      @login.login_with("zztest.patient01", "pass", "D123401")

      #goal modal
      @modal.no_element.when_visible
      @modal.no

      @landing.appTitle_element.when_visible

      expect(@landing.appTitle).to eq(app_title)

      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      today = getDateNthDaysAgo(0, "%B %d, %Y")
      expect(@landing.dateForAuthenticatedUser).to eq(today)

    end

    it ". Right Menu" do
      @rightNav.rightMenu_element.when_visible
      @rightNav.rightMenu

      @rightNav.about_element.when_visible
      expect(@rightNav.about_element.text).to eq("About")
      @rightNav.logout_element.when_visible
      expect(@rightNav.logout_element.text).to eq("Logout")
      @rightNav.launchpad_element.when_visible
      expect(@rightNav.launchpad_element.text).to eq("Launchpad")
      @rightNav.help_element.when_visible
      expect(@rightNav.help_element.text).to eq("Help")

      @rightNav.rightMenu
    end

    it ". Left Menu: The App Options icon is displayed and includes available feature navigation buttons" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.title_element.text).to eq("App Options\nClose App Options Menu")

      expect(@leftNav.home_element.exists?).to eq(true)
      expect(@leftNav.fitness_activity_element.exists?).to eq(true)
      expect(@leftNav.health_element.exists?).to eq(true)
      expect(@leftNav.support_element.exists?).to eq(true)
      expect(@leftNav.learnMore_element.exists?).to eq(true)
      expect(@leftNav.notifications_element.exists?).to eq(true)

      @leftNav.leftMenu
    end

  end

  context "AC #CREHAB-360|TC CREHAB-388: Set Weekly Activity Goals - Modal" do
    it ". When the Set Weekly Activity Goals modal is invoked, the system displays the modal" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.home_element.when_visible
      @leftNav.home

      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

    end

    it ".Instructional text and Informational Text" do
      @goal.instructionalText_element.when_visible
      expect(@goal.instructionalText_element.text).to eq("Change activity goals or start a new weekly goal.")
      @goal.infoText1_element.when_visible
      expect(@goal.infoText1_element.text).to eq("One of the most important parts of staying healthy is being active.")
      expect(@goal.infoText2_element.text).to eq("Set at least one activity goal and a start date to begin tracking your progress.")

    end

    it ". Weekly Activity Goals Labels.  Activity Goals label displays above the minutes per week, steps per week, and miles per week fields" do
      weeklyGoalModalForm = @goal.getScreenContent().split("\n")
      @EXPECTED_ROW_0_FORM.each_with_index do | expectedScreenRow, i |
        expect(weeklyGoalModalForm[i]).to eq(expectedScreenRow)
      end

    end

    it ". Minutes Per Week (secondary label, numeric field, constrain to numeric values between 1 and 10080; do not display guidance text)" do
      @goal.minutes_element.when_visible
      @goal.minutes = 0
      @goal.save

      errorMessages = ["Minutes Per Week is outside the expected range. Please enter a value between 1 and 10080."]
      verifyErrorMessage(errorMessages)

      @goal.minutes = 0
      @goal.save
      verifyErrorMessage(errorMessages)

      @goal.minutes = 480
      @goal.startDate = @today
      @goal.save

      @landing.contentLoaded_element.when_present(timeout=10)

      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")
      expect(@goal.minutes).to eq("480")


    end

    it ". Steps Per Week (secondary label, numeric field, constrain to numeric values between 1 and 50000; do not display guidance text)" do
        @goal.steps_element.when_visible
        @goal.steps = 9999999
        @goal.save

        errorMessages = ["Steps Per Week is outside the expected range. Please enter a value between 1 and 50000."]
        verifyErrorMessage(errorMessages)

        @goal.steps = 50001
        @goal.save

        errorMessages = ["Steps Per Week is outside the expected range. Please enter a value between 1 and 50000."]
        verifyErrorMessage(errorMessages)

        @goal.steps = 50000
        @goal.save

        @landing.contentLoaded_element.when_present(timeout=10)

        @landing.weeklyGoals_element.when_visible
        @landing.weeklyGoals

        @goal.modalTitle_element.when_visible
        expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")
        expect(@goal.steps).to eq("50000")

    end

    it ". Miles Per Week (secondary label, numeric field, constrain to numeric values between 1 and 50000; do not display guidance text)" do
      @goal.miles_element.when_visible
      @goal.miles = 8888888
      @goal.save
      errorMessages = ["Miles Per Week is outside the expected range. Please enter a value between 0.1 and 50000."]
      verifyErrorMessage(errorMessages)

      @goal.miles = 50001
      @goal.save
      errorMessages = ["Miles Per Week is outside the expected range. Please enter a value between 0.1 and 50000."]
      verifyErrorMessage(errorMessages)

      @goal.miles = 50000.0
      @goal.save

      @landing.contentLoaded_element.when_present(timeout=10)
      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")
      expect(@goal.miles).to eq("50000")

      @goal.miles = 234.4
      @goal.save

      @landing.contentLoaded_element.when_present(timeout=10)
      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")
      expect(@goal.miles).to eq("234.4")

    end

    it ". Start Date (primary label), date, calendar picker with text input box, text input format: MM/DD/YYYY; value cannot be past date." do
     # todo: 90 day validation  - not implemented yet
      expect(@goal.startDate_element.attribute("placeholder")).to eq("MM/DD/YYYY")

      yesterday = getDateNthDaysFromNow(-1,  "%m/%d/%Y")

      @goal.startDate = yesterday
      @goal.save

      errorMessages = ["Start Date must be between " + @today.to_s + " and " + @nintyDaysLater.to_s + "."]
      verifyErrorMessage(errorMessages)

      nintyOneDaysLater = getDateNthDaysFromNow(91,  "%m/%d/%Y")
      @goal.startDate = nintyOneDaysLater
      @goal.save

      errorMessages = ["Start Date must be between " + @today.to_s + " and " + @nintyDaysLater.to_s + "."]
      verifyErrorMessage(errorMessages)

      nintySixDaysLater = getDateNthDaysFromNow(96,  "%m/%d/%Y")

      @goal.startDate = @nintyDaysLater
      @goal.save

      @goal.save_element.when_not_visible

      expect(@landing.status).to eq("Status: " + @nintyDaysLater.to_s + " - " + nintySixDaysLater.to_s)

    end

    it ". Instructional text is displayed below the label if the date is changed: Changing your start date will create new activity goal. " do
      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      expect(@goal.dateInstructionalText_element.text).to eq("")
      @goal.startDate = @today
      expect(@goal.dateInstructionalText_element.text).to eq("Changing your start date will create a new goal.")
    end

    it ". If a user enters one or more goal values, Start Date becomes required." do
      @goal.minutes = ""
      @goal.steps = ""
      @goal.miles = ""
      @goal.startDate = ""

      expect(@goal.startDateLabel).to eq("Start Date:")
      @goal.minutes = 200
      expect(@goal.startDateLabel).to eq("* Start Date:")

      @goal.minutes = ""
      @goal.steps = ""
      @goal.miles = ""
      @goal.startDate = ""

      expect(@goal.startDateLabel).to eq("Start Date:")
      @goal.steps = 40000
      expect(@goal.startDateLabel).to eq("* Start Date:")

      @goal.minutes = ""
      @goal.steps = ""
      @goal.miles = ""
      @goal.startDate = ""

      expect(@goal.startDateLabel).to eq("Start Date:")
      @goal.miles = 25
      expect(@goal.startDateLabel).to eq("* Start Date:")

      @goal.minutes = ""
      @goal.steps = ""
      @goal.miles = ""
      @goal.startDate = ""

      expect(@goal.startDateLabel).to eq("Start Date:")
      @goal.minutes = 200
      expect(@goal.startDateLabel).to eq("* Start Date:")
      @goal.steps = 30000
      expect(@goal.startDateLabel).to eq("* Start Date:")
      @goal.miles = 30
      expect(@goal.startDateLabel).to eq("* Start Date:")

      #todo

      @goal.startDate = ""
      @goal.save

      errorMessages = ["Start Date field is required."]
      verifyErrorMessage(errorMessages)

    end

    it ".  Instructional text is displayed below the Start Date field: Check with your provider if you have any questions about exercise." do
      @goal.checkWithProvider_element.when_visible
      expect(@goal.checkWithProvider_element.text).to eq("Check with your provider if you have any questions about exercise.")
    end

    it ". Reset (button, screen reader text matches button text)" do
      @goal.startDate = @today
      expect(@goal.reset_element.text).to eq("Reset")

      @goal.reset

      expect(@goal.minutes).to eq("480")
      expect(@goal.steps).to eq("50000")
      expect(@goal.miles).to eq("234.4")
      expect(@goal.startDate).to eq(@nintyDaysLater.to_s )

      @goal.save_element.when_visible
      @goal.save

      @landing.contentLoaded_element.when_present(timeout=10)
      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      expect(@goal.minutes).to eq("480")
      expect(@goal.steps).to eq("50000")
      expect(@goal.miles).to eq("234.4")
      expect(@goal.startDate).to eq(@nintyDaysLater.to_s )
    end

    it ". Save (button, screen reader text matches button text)" do
      expect(@goal.save_element.text).to eq("Save")

      @goal.minutes = 3000
      @goal.steps = 35000
      @goal.miles = 300
      @goal.startDate = @today

      @goal.save
      @landing.contentLoaded_element.when_present(timeout=10)
      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      expect(@goal.minutes).to eq("3000")
      expect(@goal.steps).to eq("35000")
      expect(@goal.miles).to eq("300")
      expect(@goal.startDate).to eq(@today)

      @goal.minutes = ""
      @goal.steps = ""
      @goal.miles = ""
      @goal.startDate = ""

      @goal.save
      @goal.save_element.when_not_visible

    end

    it ".  Changes to a target within a goal period result in an update to the goal. Changes to a start date set the previous goal to an inactive state and create a new goal." do

      removeCollection("fitness", "healthfactordb")
      removeCollection("fitnessgoals", "healthfactordb")
      @site.refreshBrowser

      @modal.no_element.when_visible
      @modal.no
      @modal.no_element.when_not_visible

      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      @goal.minutes = 111
      @goal.steps = 222
      @goal.miles = 333
      @goal.startDate = @today

      @goal.save
      @goal.save_element.when_not_visible
      sixDaysLater = getDateNthDaysFromNow(6,  "%m/%d/%Y")

      expect(@landing.status).to eq("Status: " + @today.to_s + " - " + sixDaysLater.to_s)

      origCount = getTotalDocumentCountByPatientIdentifier("fitnessgoals", "healthfactordb", "EDIPI", "D123401")

      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      @goal.minutes = 1
      @goal.steps = 2
      @goal.miles = 3

      @goal.save
      @goal.save_element.when_not_visible
      expect(@landing.status).to eq("Status: " + @today.to_s + " - " + sixDaysLater.to_s)

      newCount = getTotalDocumentCountByPatientIdentifier("fitnessgoals", "healthfactordb", "EDIPI", "D123401")

      expect(newCount).to eq(1)

      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      @goal.minutes = 3
      @goal.steps = 4
      @goal.miles = 5

      sevenDaysLater = getDateNthDaysFromNow(7,  "%m/%d/%Y")
      @goal.startDate = @tomorrow.to_s

      @goal.save
      @goal.save_element.when_not_visible
      expect(@landing.status).to eq("Status: " + @tomorrow.to_s + " - " + sevenDaysLater.to_s)
      newCount = getTotalDocumentCountByPatientIdentifier("fitnessgoals", "healthfactordb", "EDIPI", "D123401")
      expect(newCount).to eq(2)


    end

    it ". Cancel (button, screen reader text matches button text)" do
      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      @goal.cancel_element.when_visible
      expect(@goal.cancel_element.text).to eq("Cancel")
      @goal.minutes = 12
      @goal.steps = 111
      @goal.miles = 35
      @goal.startDate = @today

      @goal.cancel

      @landing.contentLoaded_element.when_present(timeout=10)
      @landing.weeklyGoals_element.when_visible
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      expect(@goal.minutes).to eq("3")
      expect(@goal.steps).to eq("4")
      expect(@goal.miles).to eq("5")
      expect(@goal.startDate).to eq(@tomorrow)
      @goal.save
      @landing.contentLoaded_element.when_present(timeout=10)

      @landing.weeklyGoals_element.when_visible
      endDate = getDateNthDaysFromNow(6, "%m/%d/%Y")
      puts "End date is " + endDate

    end
  end

  context "AC #CREHAB-358|TC CREHAB-387: Landing Page Display After Authentication" do
    it ". Weekly Goals button: Text, Alt Text, Icon, Clicking opens modal" do

      @landing.weeklyGoals_element.when_visible

      expect(@landing.weeklyGoalsAltText_element.text).to eq("Trophy icon ")
      expect(@landing.weeklyGoalsText_element.text).to eq("Weekly Activity Goals")
      expect(@landing.weeklyGoalsIcon_element.class_name).to include("icon-trophy")

      expect(@landing.goalsDialog_element.visible?).to eq(false)
      @landing.weeklyGoals

      @goal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Set Weekly Activity Goals")
    end

  end

  context "AC #CREHAB-359|TC CREHAB-387: User Without Set Goals" do
    it ". Modal displays when user has no set goals" do
      removeCollection("fitness", "healthfactordb")
      removeCollection("fitnessgoals", "healthfactordb")
      @site.refreshBrowser

      @modal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Weekly Activity Goals")
      expect(@modal.section1_element.text).to eq("You currently have no weekly activity goals. Are you ready to set your weekly activity goals?")
    end

    it ". Button: No (screen reader text matches button) " do
      @modal.no_element.when_visible
      @modal.no
      @modal.no_element.wait_while_present

      @landing.weeklyGoals_element.when_visible

    end

    it ". Status, Instructional text" do
      expect(@landing.weeklyGoals_element.text).to eq("Trophy icon \nWeekly Activity Goals")
      expect(@landing.infoText_element.text).to eq("One of the most important parts of staying healthy is being active. Use the Trophy icon above to set one or more activity goals.")
      expect(@landing.instructionalText_element.text).to eq("Use the menu buttons on the top left or right to explore the app.\nFrom the right upper menu, you can access the help section which has a user guide.\nSelect the Trophy icon to set and update activity goals.\nSelect the Trophy icon to set and update activity goals.")
    end

    it ". If a user does not set up goals and later returns to the home page within the same session the modal is not shown." do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible

      @leftNav.learnMore
      @learnMore.primaryHeader_element.when_visible

      expect(@learnMore.primaryHeader_element.text).to eq("Learn More")

      @leftNav.leftMenu
      @leftNav.title_element.when_visible
      @leftNav.home

      @landing.weeklyGoals_element.when_visible
      expect(@modal.modalTitle_element.exists?).to eq(false)

    end

    it ". Button: Yes (screen reader text matches button)" do
      removeCollection("fitness", "healthfactordb")
      removeCollection("fitnessgoals", "healthfactordb")
      @site.refreshBrowser
      @modal.yes_element.when_visible
      @modal.yes

      @goal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

    end

  end

  context "#AC CREHAB-857|TC CREHAB-896: Null value set start date to today " do
    it ". In the weekly activity goal modal, if start date is null, when a user enters a target (any of the three), set the start date to today." do
      @goal.minutes_element.when_visible

      expect(@goal.minutes).to eq("")
      expect(@goal.steps).to eq("")
      expect(@goal.miles).to eq("")
      expect(@goal.startDate).to eq("")

      @goal.minutes = 123
      expect(@goal.startDate).to eq(@today)

      @goal.reset
      expect(@goal.minutes).to eq("")
      expect(@goal.steps).to eq("")
      expect(@goal.miles).to eq("")
      expect(@goal.startDate).to eq("")

      @goal.steps = 2345
      expect(@goal.startDate).to eq(@today)

      @goal.reset
      expect(@goal.minutes).to eq("")
      expect(@goal.steps).to eq("")
      expect(@goal.miles).to eq("")
      expect(@goal.startDate).to eq("")

      @goal.miles = 10
      expect(@goal.startDate).to eq(@today)

    end
    it ". If a user nullifies a goal value after the default is set, the user must manually change the start date to another value or delete the date." do
      @goal.minutes = ""
      @goal.steps = ""
      @goal.miles = ""

      expect(@goal.startDate).to eq(@today)

      @goal.startDate = ""
      @goal.save

      @landing.contentLoaded_element.when_present(timeout=10)
      @landing.weeklyGoals_element.when_visible
    end

  end


  context "#CREHAB-699|TC CREHAB-1222: Number of remaining units" do
    it ". Display null value if a user has not set an activity goal" do
      Watir::Wait.until { @landing.minutesLeftUnits_element.text.strip == ""}

      expect(@landing.minutesLeftUnits_element.text.strip).to eq("")
      expect(@landing.stepsLeftUnits_element.text.strip).to eq("")
      expect(@landing.milesLeftUnits_element.text.strip).to eq("")
    end

    it ". Display  text <number of unit remaining> left if a user has an activity goal.  Display '0 left' if user completed more activity than the goal" do
      resetCollection("fitnessgoals", "healthfactordb", "weekly_goals")
      restoreCollection("fitness", "healthfactordb",  "spec/snapshots/fitness/fitness.bson")

      #update goals db
      startDate = Time.now - 3.days
      endDate = Time.now + 3.days
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "startDate", startDate)
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "endDate", endDate)

      #update fitness activity db

      threeDaysAgo = Time.now - 3.days
      twoDaysAgo = Time.now - 2.days
      updateDocument("fitness", "healthfactordb", "54b5a1d003644ae0cd231bf3", "activityDate", threeDaysAgo)
      updateDocument("fitness", "healthfactordb", "54b5a1d003644ae0cd231bf3", "createdDate", threeDaysAgo)
      updateDocument("fitness", "healthfactordb", "54b5a1ed03644ae0cd231bf4", "activityDate", twoDaysAgo)
      updateDocument("fitness", "healthfactordb", "54b5a1ed03644ae0cd231bf4", "createdDate", twoDaysAgo)

      @site.refreshBrowser

      #landing screen
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible

      expect(@landing.minutesLeftUnits_element.text.strip).to eq("70 left")
      expect(@landing.stepsLeftUnits_element.text.strip).to eq("0 left")
      expect(@landing.milesLeftUnits_element.text.strip).to eq("439.0 left")
    end
  end

  context "#AC CREAHB-1178|TC CREAHB-1193: Phone view : Weekly Activity Progress graphs" do
    it ". When a phone user opens the VA FitHeart landing page, the three Weekly Activity Progress graphs are shown one at a time" do
      @site.resizeWindowToPhone
      expect(@carousel.carousel_element.visible?).to eq(true)
      expect(@carousel.leftArrow_element.visible?).to eq(true)
      expect(@carousel.rightArrow_element.visible?).to eq(true)
      #@site.resizeWindowToDefault
    end
  end

  context "#AC CREHAB-1179|TC CREHAB-1193: First Image: Minute graph" do
    it ". First Graph: Minutes graph - pagination dot, alt text: Image 1 of e" do
      expect(@carousel.getPaginationAltText()).to eq("Image 1 of 3")
      expect(@carousel.getActiveGraphDataLabel()).to include("164\nminutes\n      70 left\nGoal:  234 minutes")
      expect(@carousel.leftArrow_element.attribute("title")).to eq("Previous Image")
      expect(@carousel.rightArrow_element.attribute("title")).to eq("Next Image")
    end
  end

  context "#AC CREHAB-1180|TC CREHAB-1193: Second image: Steps Graph " do
    it ". Second Graph: Steps Graph - pagination alt text: Image 2 of 3" do
      @carousel.rightArrow_element.click

      Watir::Wait.until {@carousel.getPaginationAltText() == "Image 2 of 3"}
      expect(@carousel.getPaginationAltText()).to eq("Image 2 of 3")
      expect(@carousel.getActiveGraphDataLabel()).to include("0 left")
      expect(@carousel.leftArrow_element.attribute("title")).to eq("Previous Image")
      expect(@carousel.rightArrow_element.attribute("title")).to eq("Next Image")
    end
  end

  context "#AC CREAHB-1181|TC CREHAB-1193:Third image: Miles Graph" do
    it ". Third Graph: Miles Graph - pagination alt text: Image 3 of 3" do
      @carousel.rightArrow_element.when_visible
      sleep 2
      @carousel.rightArrow_element.click

      Watir::Wait.until {@carousel.getPaginationAltText() == "Image 3 of 3"}
      Watir::Wait.until {@carousel.getActiveGraphDataLabel().include?("439.0 left")  }
      expect(@carousel.getPaginationAltText()).to eq("Image 3 of 3")
      expect(@carousel.getActiveGraphDataLabel()).to include("439.0 left")
      expect(@carousel.leftArrow_element.attribute("title")).to eq("Previous Image")
      expect(@carousel.rightArrow_element.attribute("title")).to eq("Next Image")
    end
  end
end
