require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../lib/module/DateUtility'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'


describe '[Story #CREHAB-262: Set Weekly Activity Goals - Modal : rules for Activity Goals]', :regression => true do
  include DateUtility
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeCollection("fitness", "healthfactordb")
    removeCollection("fitnessgoals", "healthfactordb")

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @goal = @site.goal
    @error = @site.error

    @today = getDateNthDaysAgo(0, "%m/%d/%Y")
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "AC #CREHAB-361|TC CREHAB-388: Rules for Activity Goals" do
    it ". 	Goal end date is stored and calculated as the Start Date + 6 days." do
        minutes = 111
        steps = 222
        miles = 333
        startDate = @today
        addWeeklyGoal(minutes, steps, miles, startDate)
        sixDaysLater = getDateNthDaysFromNow(6,  "%m/%d/%Y")

        expect(@landing.status).to eq("Status: " + @today.to_s + " - " + sixDaysLater.to_s)
    end

     it ". When the user saves a new start date, the previous goal is considered in inactive, the end date is set to today (the day the date change is saved), and the new start date and end date are used to define the active goal." do
       tomorrow = getDateNthDaysFromNow(1,  "%m/%d/%Y")
       sevenDaysLater = getDateNthDaysFromNow(7,  "%m/%d/%Y")

       minutes = 111
       steps = 222
       miles = 333
       startDate = tomorrow
       addWeeklyGoal(minutes, steps, miles, startDate)
       sixDaysLater = getDateNthDaysFromNow(6,  "%m/%d/%Y")

       expect(@landing.status).to eq("Status: " + tomorrow.to_s + " - " + sevenDaysLater.to_s)

       documentArray = retreiveAllDocumentsInCollection("fitnessgoals", "healthfactordb")
       #puts "documentArray 0 "  + documentArray[0].to_s
       previousGoalActiveFlag = documentArray[0]['activeFlag'].to_s
       previousGoalEndDate = documentArray[0]['endDate'].to_s

       expect(previousGoalActiveFlag).to eq("false")
       #end date is set to today
       expect(getFormatedDateStr(previousGoalEndDate, "%m/%d/%Y").to_s).to eq(convertTimeNowToUTCDate())

       #puts "documentArray 1 "  + documentArray[1].to_s
       newGoalActiveFlag = documentArray[1]['activeFlag'].to_s
       newGoalEndDate = documentArray[1]['endDate'].to_s
       newGoalStartDate = documentArray[1]['startDate'].to_s

       expect(newGoalActiveFlag).to eq("true")
       expect(getFormatedDateStr(newGoalEndDate, "%m/%d/%Y").to_s).to eq(sevenDaysLater.to_s)
       expect(getFormatedDateStr(newGoalStartDate, "%m/%d/%Y").to_s).to eq(tomorrow.to_s)

       target = documentArray[1]['target']
       expect(target['minutes'].to_s).to eq("111")
       expect(target['steps'].to_s).to eq("222")
       expect(target['miles'].to_i).to eq(333)

     end

    it ". If a user has not entered any activities into the last goal period, a new goal period is not automatically created. " do
      #no fitness activity is entered.
      resetCollection("fitnessgoals", "healthfactordb", "weekly_goals")

      yesterday = Time.now - 1.days
      sevenDaysAgo = Time.now - 7.days
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "startDate", sevenDaysAgo)
      updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "endDate", yesterday)

      @site.refreshBrowser
      @modal.yes_element.when_visible
      @modal.yes

      @goal.modalTitle_element.when_visible
      expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

      expect(@goal.steps).to eq("")
      expect(@goal.minutes).to eq("")
      expect(@goal.miles).to eq("")
      expect(@goal.startDate).to eq("")

      @goal.cancel_element.when_visible
      @goal.cancel

    end

     it ". If a user has entered fitness activities during the past goal period, a new goal range is automatically set for the following 7 days. " do
       resetCollection("fitnessgoals", "healthfactordb", "weekly_goals")
       resetCollection("fitness", "healthfactordb", "fitness_activity")

       twoDaysAgo = Time.now - 2.days
       eightDaysAgo = Time.now - 8.days
       updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "startDate", eightDaysAgo)
       updateDocument("fitnessgoals", "healthfactordb", "54b53a6203644ae0cd231bc5", "endDate", twoDaysAgo)

       threeDaysAgo = Time.now - 3.days
       fourDaysAgo = Time.now - 4.days
       updateDocument("fitness", "healthfactordb", "54b5a1d003644ae0cd231bf3", "activityDate", threeDaysAgo)
       updateDocument("fitness", "healthfactordb", "54b5a1d003644ae0cd231bf3", "createdDate", threeDaysAgo)
       updateDocument("fitness", "healthfactordb", "54b5a1ed03644ae0cd231bf4", "activityDate", fourDaysAgo)
       updateDocument("fitness", "healthfactordb", "54b5a1ed03644ae0cd231bf4", "createdDate", fourDaysAgo)

       @site.refreshBrowser

       Watir::Wait.until { @landing.status_element.exists?}
       @landing.status_element.when_visible
       yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")
       sixDaysLater = getDateNthDaysFromNow(5, "%m/%d/%Y")

       expect(@landing.status).to eq("Status: " + yesterday.to_s + " - " + sixDaysLater.to_s)

       @landing.weeklyGoals_element.when_visible
       @landing.weeklyGoals

       @goal.modalTitle_element.when_visible
       expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

       expect(@goal.minutes).to eq("234")
       expect(@goal.steps).to eq("345")
       expect(@goal.miles).to eq("456")
       expect(@goal.startDate).to eq(yesterday.to_s)

       @goal.cancel_element.when_visible
       @goal.cancel

     end


     it ". Only active goal start and end dates are shown in the Set Activity modal. Goal history is stored for historical purposes in the database." do
       removeCollection("fitness", "healthfactordb")
       removeCollection("fitnessgoals", "healthfactordb")

       @site.refreshBrowser
       @modal.yes_element.when_visible
       @modal.yes

       @goal.modalTitle_element.when_visible
       expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

       tomorrow = getDateNthDaysFromNow(1,  "%m/%d/%Y")
       sevenDaysLater = getDateNthDaysFromNow(7,  "%m/%d/%Y")
       @goal.startDate = tomorrow

       @goal.save
       @goal.save_element.when_not_visible

       minutes = ""
       steps = ""
       miles = ""
       startDate = ""
       addWeeklyGoal(minutes, steps, miles, startDate)


       expect(@landing.infoText_element.text).to eq("One of the most important parts of staying healthy is being active. Use the Trophy icon above to set one or more activity goals.")

     end

     it ". Users may change activity targets mid-week; changing targets does not reset the goal or create a new goal period." do
       removeCollection("fitness", "healthfactordb")
       removeCollection("fitnessgoals", "healthfactordb")

       @site.refreshBrowser
       @modal.no_element.when_visible
       @modal.no

       minutes = 22
       steps = 33
       miles = 44
       startDate = @today
       addWeeklyGoal(minutes, steps, miles, startDate)

       origCount = getTotalDocumentCountByPatientIdentifier("fitnessgoals", "healthfactordb", "EDIPI", "D123401")
       expect(origCount).to eq(1)

       minutes = 44
       steps = 55
       miles = 66
       startDate = @today
       addWeeklyGoal(minutes, steps, miles, startDate)
       newCount = getTotalDocumentCountByPatientIdentifier("fitnessgoals", "healthfactordb", "EDIPI", "D123401")
       expect(newCount).to eq(1)

       documentArray = retreiveAllDocumentsInCollection("fitnessgoals", "healthfactordb")
       puts "documentArray 0 "  + documentArray[0].to_s
       activeFlag = documentArray[0]['activeFlag'].to_s
       expect(activeFlag).to eq("true")

     end

     it ".  If a user changes a start date, the target values on the screen are not changed or nullified; instead they are copied to the new goal period." do
       #this test scenario is covered in Landing_Page_CREHAB-6_261_262_678_spec.rb
     end

     it ".  Goals may be saved with any or all of the target fields set to null." do
       minutes = ""
       steps = ""
       miles = ""
       startDate = ""
       addWeeklyGoal(minutes, steps, miles, startDate)
       expect(@landing.infoText_element.text).to eq("One of the most important parts of staying healthy is being active. Use the Trophy icon above to set one or more activity goals.")

     end
  end

  def addWeeklyGoal(minutes, steps, miles, startDate)
    @landing.weeklyGoals_element.when_visible
    @landing.weeklyGoals

    @goal.modalTitle_element.when_visible
    expect(@goal.modalTitle_element.text).to eq("Set Weekly Activity Goals")

    @goal.minutes = minutes
    @goal.steps = steps
    @goal.miles = miles
    @goal.startDate = startDate

    @goal.save
    @goal.save_element.when_not_visible
  end


end
