#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'


describe '[Story #CREHAB-169: Learn More - Health Accordion]', :regression => true do
  include MongoUtility

  app_title = "VA FitHeart"
  quick_tips = "Quick Tips"
  blood_pressure = "Blood Pressure"
  cholesterol = "Cholesterol"
  smoking = "Smoking"
  quit_smoking_apps = "Quit Smoking Apps"
  diabetes = "Diabetes"
  diabetes_apps = "Diabetes Apps"
  medications = "Medications"
  links = "Links"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @learnMore = @site.learnMore
    @health = @site.learnMore_health
    @modal = @site.modal
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "#AC #CREHAB-346|TC CREHAB-382: Health Option from App Options" do
    it ". When a user selects the Health option from the App Options list, the user will see the following items in the detail panel/column 3: " do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.learnMore_element.visible?).to eq(true)
      expect(@leftNav.learnMore_element.text).to eq("Learn More")

      @leftNav.learnMore
      @learnMore.primaryHeader_element.when_visible
      expect(@learnMore.primaryHeader_element.text).to eq("Learn More")

      expect(@learnMore.health_element.text).to eq("Health")

      @learnMore.health
      Watir::Wait.until {@health.primaryHeader_element.text == "Health"}

    end

    it ". Health Learn More sections displayed in an accordion control" do
      expect(@health.displayedInAccordionControl?).to eq(true)
    end

    it ". < icon is displayed in phone form factors, alt text: Return to Learn More options" do
      @site.resizeWindowToPhone

      expect(@health.backButton_element.visible?).to eq(true)
      expect(@health.backButtonIcon_element.visible?).to eq(true)
      expect(@health.backButton_element.attribute("title")).to eq("Return to Learn More options")
      @health.backButton

      @learnMore.secondaryHeader_element.when_visible
      expect(@learnMore.secondaryHeader_element.text).to eq("Learn More")

      expect(@learnMore.fitness_element.text).to eq("Fitness")
      expect(@learnMore.health_element.text).to eq("Health")
      expect(@learnMore.nutrition_element.text).to eq("Nutrition")
      expect(@learnMore.mood_element.text).to eq("Mood")

      @site.resizeWindowToDefault

    end

  end

  context "#AC CREHAB-347|TC CREHAB-382: Accordion Display for Health Accordion" do
    it ". Accordion display" do
      @learnMore.health
      Watir::Wait.until {@health.primaryHeader_element.text == "Health"}

      expect(@health.getAccordionLabelByHeading(quick_tips)).to eq("Quick Tips")
      expect(@health.getAccordionLabelByHeading(blood_pressure)).to eq("Blood Pressure")
      expect(@health.getAccordionLabelByHeading(cholesterol)).to eq("Cholesterol")
      expect(@health.getAccordionLabelByHeading(smoking)).to eq("Smoking")
      expect(@health.getAccordionLabelByHeading(quit_smoking_apps)).to eq("Quit Smoking Apps")
      expect(@health.getAccordionLabelByHeading(diabetes)).to eq("Diabetes")
      expect(@health.getAccordionLabelByHeading(diabetes_apps)).to eq("Diabetes Apps")
      expect(@health.getAccordionLabelByHeading(medications)).to eq("Medications")
      expect(@health.getAccordionLabelByHeading(links)).to eq("Links")
    end
  end

  context "#AC CREHAB-348|TC CREHAB-382: Accordion Behavior for Health Accordion" do
    it ". Accordion sections can be opened and closed" do

      accordionArr = [quick_tips, blood_pressure, cholesterol, smoking, quit_smoking_apps, diabetes, diabetes_apps, medications, links]

      accordionSize = accordionArr.length - 1

      #all accordions closed
      for i in 0..accordionSize
        expect(@health.isAccordionExpanded(accordionArr[i])).to eq(false)
        expect(@health.isRightArrowIconDisplayed(accordionArr[i])).to eq(true)
      end

      for j in 0..accordionSize
        expect(@health.getAccordionAltText(accordionArr[j])).to eq("Expand\n" + accordionArr[j])
        @health.clickAccordionByHeading(accordionArr[j])
        expect(@health.isAccordionExpanded(accordionArr[j])).to eq(true)
        expect(@health.getAccordionAltText(accordionArr[j])).to eq("Collapse\n" + accordionArr[j])
        expect(@health.isDownArrowIconDisplayed(accordionArr[j])).to eq(true)

        for k in 0..accordionSize
          if (k != j)
            expect(@health.isAccordionExpanded(accordionArr[k])).to eq(false)
            expect(@health.isRightArrowIconDisplayed(accordionArr[k])).to eq(true)
          end
        end

        @health.clickAccordionByHeading(accordionArr[j])
        expect(@health.isAccordionExpanded(accordionArr[j])).to eq(false)
        expect(@health.getAccordionAltText(accordionArr[j])).to eq("Expand\n" + accordionArr[j])
      end

    end
  end

  context "#AC CREHAB-349|TC CREHAB-382: Accordion section displays for Health Accordion" do
    it ". When a user expands the Quick Tips accordion" do
      @health.clickAccordionByHeading(quick_tips)
      expect(@health.getAccordionTextByHeading(quick_tips)).to eq("Quick Tips for staying healthy:\nPhysical activity\nMaintain a healthy weight\nEat healthy foods\nMonitor blood pressure\nLower cholesterol\nTake medications\nIf you smoke, quit\nIf you have Diabetes, monitor blood glucose")
    end

    it ".  When a user expands the Blood Pressure accordion" do
      @health.clickAccordionByHeading(blood_pressure)
      expect(@health.getAccordionTextByHeading(blood_pressure)).to eq("For most people with heart conditions, keep your blood pressure below 140/90. It may require medications to lower your blood pressure. Healthy behavior can also help:\nTake medications as directed\nExercise\nLimit salt intake\nMaintain a healthy diet\nLimit alcohol intake\nLearn more about high blood pressure.")

      @health.healthyDietLink_element.when_visible
      @health.healthyDietLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::HEALTHY_DIET_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::HEALTHY_DIET_URL)
      @site.closeBrowser

      @health.learnMoreHighBloodPressureLink_element.when_visible
      @health.learnMoreHighBloodPressureLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::LEARN_MORE_HIGH_BLOOD_PRESSURE_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::LEARN_MORE_HIGH_BLOOD_PRESSURE_URL)
      @site.closeBrowser

    end

    it ". When a user expands the Cholesterol accordion" do
      @health.clickAccordionByHeading(cholesterol)
      expect(@health.getAccordionTextByHeading(cholesterol)).to eq("There are several different types of cholesterol:\nLDL (“bad cholesterol”)\nHDL (“good cholesterol”)\nTriglycerides\nTotal cholesterol (a combination of the above)\nTalk to your health provider about developing an individual plan for managing your cholesterol.\nLearn more about cholesterol.")

      @health.learnMoreCholesterolLink_element.when_visible
      @health.learnMoreCholesterolLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::LEARN_MORE_CHOLESTEROL_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::LEARN_MORE_CHOLESTEROL_URL)
      @site.closeBrowser
    end

    it ". When a user expands the Smoking accordion" do
      @health.clickAccordionByHeading(smoking)
      expect(@health.getAccordionTextByHeading(smoking)).to eq("If you are already a non-smoker, then you are on the right track!\nResearch has shown that you will live longer and be less likely to have a heart attack if you quit smoking.\nLearn more about quitting smoking.")

      @health.learnMoreQuittingSmokingLink_element.when_visible
      @health.learnMoreQuittingSmokingLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::LEARN_MORE_QUITTING_SMOKING_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::LEARN_MORE_QUITTING_SMOKING_URL)
      @site.closeBrowser

    end

    it ".  When a user expands the Quit Smoking Apps accordion" do
      @health.clickAccordionByHeading(quit_smoking_apps)
      expect(@health.getAccordionTextByHeading(quit_smoking_apps)).to eq("Stay Quit Coach")

      @health.stayQuitCoachLink_element.when_visible
      @health.stayQuitCoachLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::STAY_QUIT_COACH_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::STAY_QUIT_COACH_URL)
      @site.closeBrowser
    end

    it ". When a user expands the Diabetes accordion" do
      @health.clickAccordionByHeading(diabetes)
      expect(@health.getAccordionTextByHeading(diabetes)).to eq("Diabetes can increase your risk of having heart disease. Many health providers routinely check for Diabetes in their patients with heart disease. Do you know if you have Diabetes?\nLearn more about diabetes.")

      @health.learnMoreDiabetesLink_element.when_visible
      @health.learnMoreDiabetesLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::LEARN_MORE_DIABETES_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::LEARN_MORE_DIABETES_URL)
      @site.closeBrowser

    end

    it ". When a user expands the Diabetes Apps accordion" do
      @health.clickAccordionByHeading(diabetes_apps)
      expect(@health.getAccordionTextByHeading(diabetes_apps)).to eq("Glooko Diabetes Management System")

      @health.diabetesAppLink_element.when_visible
      @health.diabetesAppLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::GLOOKO_DIABETES_MANAGEMENT_SYSTEM_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::GLOOKO_DIABETES_MANAGEMENT_SYSTEM_URL)
      @site.closeBrowser
    end

    it ". When a user expands the Medications accordion" do
      @health.clickAccordionByHeading(medications)
      expect(@health.getAccordionTextByHeading(medications)).to eq("Medications work in different ways to keep you healthy:\nAspirin\nPrevents blood clots from forming in arteries\nACE-inhibitor or ARB (generic name ends in “–pril” or “–sartan”)\nLowers blood pressure\nAnti-platelet drug (generic name has “grel” in the name)\nPrevents blood clots from forming in arteries\nBeta-blocker (generic name ends in “-lol”)\nLowers heart rate and blood pressure\nStatin\nLowers cholesterol")
    end

    it ". When a user expands the Links accordion" do
      @health.clickAccordionByHeading(links)
      expect(@health.getAccordionTextByHeading(links)).to eq("American Heart Association (AHA)\nAmerican College of Cardiology (ACC)\nAmerican Association of Cardiovascular and Pulmonary Rehabilitation (AACVPR)\nPreventive Cardiovascular Nurses Association\nQuit Smoking")

      @health.healthAHALink_element.when_visible
      @health.healthAHALink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::AHA_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::AHA_URL)
      @site.closeBrowser

      @health.healthACCLink_element.when_visible
      @health.healthACCLink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::ACC_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::ACC_URL)
      @site.closeBrowser

      @health.healthAACVPRLink_element.when_visible
      @health.healthAACVPRLink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::AACVPR_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::AACVPR_URL)
      @site.closeBrowser

      @health.healthPCNALink_element.when_visible
      @health.healthPCNALink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::PREVENTIVE_CARDIO_NURSE_ASSO_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::PREVENTIVE_CARDIO_NURSE_ASSO_URL)
      @site.closeBrowser

      @health.healthquitSmokingLink_element.when_visible
      @health.healthquitSmokingLink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreHealth::QUIT_SMOKING_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreHealth::QUIT_SMOKING_URL)
      @site.closeBrowser

    end
  end

end
