require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'


describe '[Story #CREHAB-167: Learn More Feature]', :regression => true do
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @learnMore = @site.learnMore
    @leftNav = @site.leftMenu
    @modal = @site.modal

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "#AC #CREHAB-340|TC #CREHAB-362:: List items for Learn More" do
    it ".When a user selects the Learn More feature from the App Options list, the user will see the following items  -  Fitness, Health, Nutrition, Mood" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.learnMore_element.visible?).to eq(true)
      expect(@leftNav.learnMore_element.text).to eq("Learn More")

      @leftNav.learnMore
      @learnMore.primaryHeader_element.when_visible
      expect(@learnMore.primaryHeader_element.text).to eq("Learn More")

      expect(@learnMore.fitness_element.text).to eq("Fitness")
      expect(@learnMore.health_element.text).to eq("Health")
      expect(@learnMore.nutrition_element.text).to eq("Nutrition")
      expect(@learnMore.mood_element.text).to eq("Mood")

      expect(@learnMore.learnMoreFitnessIcon_element.visible?).to eq(true)
      expect(@learnMore.learnMoreHealthIcon_element.visible?).to eq(true)
      expect(@learnMore.learnMoreNutritionIcon_element.visible?).to eq(true)
      expect(@learnMore.learnMoreMoodIcon_element.visible?).to eq(true)
    end

  end

  context "#AC CREHAB-341|TC #CREHAB-362: Learn More Display Message" do
    it ". Detailed panel will have header Learn More and display message until an item is selected" do
        expect(@learnMore.placeholderMessage).to eq("Select an item from the Learn More list to view details")
    end

  end

end
