#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'

describe '[Story #CREHAB-171: Learn More - Mood Accordion]', :regression => true do
  include MongoUtility

  app_title = "VA FitHeart"
  quick_tips = "Quick Tips"
  breathing = "Breathing"
  mindfulness = "Mindfulness"
  positive_events = "Positive Events"
  gratitude = "Gratitude"
  silver_lining = "Silver Lining"
  personal_strengths = "Personal Strengths"
  acts_of_kindness = "Acts of Kindness"
  video = "Video"
  links = "Links"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @learnMore = @site.learnMore
    @mood = @site.learnmore_mood
    @modal = @site.modal
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "#AC #CREHAB-354|TC CREHAB-384: Mood Accordion for App Options" do
    it ". When a user selects the Mood option from the App Options list, the user will see the following items in the detail panel/column 3:" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.learnMore_element.visible?).to eq(true)
      expect(@leftNav.learnMore_element.text).to eq("Learn More")

      @leftNav.learnMore
      @learnMore.primaryHeader_element.when_visible
      expect(@learnMore.primaryHeader_element.text).to eq("Learn More")

      expect(@learnMore.mood_element.text).to eq("Mood")

      @learnMore.mood
      Watir::Wait.until {@mood.primaryHeader_element.text == "Mood"}

    end

    it ". Mood Learn More sections displayed in an accordion control" do
      expect(@mood.displayedInAccordionControl?).to eq(true)
    end

    it ". < icon is displayed in phone form factors, alt text: Return to Learn More options" do
      @site.resizeWindowToPhone

      expect(@mood.backButton_element.visible?).to eq(true)
      expect(@mood.backButtonIcon_element.visible?).to eq(true)
      expect(@mood.backButton_element.attribute("title")).to eq("Return to Learn More options")
      @mood.backButton

      @learnMore.secondaryHeader_element.when_visible
      expect(@learnMore.secondaryHeader_element.text).to eq("Learn More")

      expect(@learnMore.fitness_element.text).to eq("Fitness")
      expect(@learnMore.health_element.text).to eq("Health")
      expect(@learnMore.nutrition_element.text).to eq("Nutrition")
      expect(@learnMore.mood_element.text).to eq("Mood")

      @site.resizeWindowToDefault
    end

  end

  context "#AC CREHAB-355|TC CREHAB-384: Accordion display for Mood Accordion" do
    it ". Accordion display" do
      @learnMore.mood
      Watir::Wait.until {@mood.primaryHeader_element.text == "Mood"}

      expect(@mood.getAccordionLabelByHeading(quick_tips)).to eq("Quick Tips")
      #expect(@mood.getAccordionLabelByHeading(breathing)).to eq("Breathing")
      #expect(@mood.getAccordionLabelByHeading(mindfulness)).to eq("Mindfulness")
      expect(@mood.getAccordionLabelByHeading(positive_events)).to eq("Positive Events")
      expect(@mood.getAccordionLabelByHeading(gratitude)).to eq("Gratitude")
      expect(@mood.getAccordionLabelByHeading(silver_lining)).to eq("Silver Lining")
      expect(@mood.getAccordionLabelByHeading(personal_strengths)).to eq("Personal Strengths")
      expect(@mood.getAccordionLabelByHeading(acts_of_kindness)).to eq("Acts of Kindness")
      expect(@mood.getAccordionLabelByHeading(video)).to eq("Video")
      expect(@mood.getAccordionLabelByHeading(links)).to eq("Links")

    end
  end

  context "#AC CREHAB-356|TC CREHAB-384: Accordion Behavior for Mood Accordion" do
    it ". Accordion sections can be opened and closed" do

      accordionArr = [quick_tips, positive_events, gratitude, silver_lining, personal_strengths, acts_of_kindness, video, links]

      accordionSize = accordionArr.length - 1

      #all accordions closed
      for i in 0..accordionSize
        expect(@mood.isAccordionExpanded(accordionArr[i])).to eq(false)
        expect(@mood.isRightArrowIconDisplayed(accordionArr[i])).to eq(true)
      end

      for j in 0..accordionSize
        expect(@mood.getAccordionAltText(accordionArr[j])).to eq("Expand\n" + accordionArr[j])
        @mood.clickAccordionByHeading(accordionArr[j])
        expect(@mood.isAccordionExpanded(accordionArr[j])).to eq(true)
        expect(@mood.getAccordionAltText(accordionArr[j])).to eq("Collapse\n" + accordionArr[j])
        expect(@mood.isDownArrowIconDisplayed(accordionArr[j])).to eq(true)

        for k in 0..accordionSize
          if (k != j)
            expect(@mood.isAccordionExpanded(accordionArr[k])).to eq(false)
            expect(@mood.isRightArrowIconDisplayed(accordionArr[k])).to eq(true)
          end
        end

        @mood.clickAccordionByHeading(accordionArr[j])
        expect(@mood.isAccordionExpanded(accordionArr[j])).to eq(false)
        expect(@mood.getAccordionAltText(accordionArr[j])).to eq("Expand\n" + accordionArr[j])
      end

    end
  end

  context "#AC CREHAB-357|TC CREHAB-384: Accordion section displays for Mood Accordion" do
    it ". When a user expands the Quick Tips accordion" do
      @mood.clickAccordionByHeading(quick_tips)
      expect(@mood.getAccordionTextByHeading(quick_tips)).to eq("Some quick tips for your mood:\nEnjoy positive experiences\nWrite about your feelings\nBe active!\nTake a few minutes for mindfulness\nAsk family and friends to help")
    end

    # it ". When a user expands the Breathing accordion" do
    #   @mood.clickAccordionByHeading(breathing)
    #   expect(@mood.getAccordionTextByHeading(breathing)).to eq("Sit in comfortable position.\nClose your eyes.\nImagine yourself in a peaceful place.\nBreathe in and out slowly and deeply.\nTry to take 5 to 10 minutes for deep breathing.")
    # end
    #
    # it ". When a user expands the Mindfulness accordion" do
    #   @mood.clickAccordionByHeading(mindfulness)
    #   expect(@mood.getAccordionTextByHeading(mindfulness)).to eq("This guided breathing exercise will help you with stress and mindfulness.\nListen Now.")
    # end

    it ". When a user expands the Positive Events accordion" do
      @mood.clickAccordionByHeading(positive_events)
      expect(@mood.getAccordionTextByHeading(positive_events)).to eq("Identifying and remembering positive events can affect your mood.\nCan you think of a positive event? How did that make you feel?")
    end

    it ". When a user expands the Gratitude accordion" do
      @mood.clickAccordionByHeading(gratitude)
      expect(@mood.getAccordionTextByHeading(gratitude)).to eq("Gratitude can refer to the way you feel when you’re thankful for something.\nAre you thankful for something today?")
    end

    it ". When a user expands the Silver Lining accordion" do
      @mood.clickAccordionByHeading(silver_lining)
      expect(@mood.getAccordionTextByHeading(silver_lining)).to eq("How you think about an event can make an event feel stressful, relaxing, wonderful, or meaningful.\nCan you find a way to think about a mildly stressful experience in a more positive way?")
    end

    it ". When a user expands the Personal Strengths accordion" do
      @mood.clickAccordionByHeading(personal_strengths)
      expect(@mood.getAccordionTextByHeading(personal_strengths)).to eq("You can use your personal strengths to help you overcome barriers.\nWhat are some of your strengths?")
    end

    it ". When a user expands the Acts of Kindness accordion" do
      @mood.clickAccordionByHeading(acts_of_kindness)
      expect(@mood.getAccordionTextByHeading(acts_of_kindness)).to eq("Doing nice things for other people can make you feel good, even if you don’t get anything in return.\nHave you done something nice for someone else today?")
    end

    it ". When a user expands the Video accordion" do
      @mood.clickAccordionByHeading(video)
      expect(@mood.getAccordionTextByHeading(video)).to eq("Watch a video on stress reduction.")

      @mood.stressVideoLink_element.when_visible
      @mood.stressVideoLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreMood::STRESS_VIDEO_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreMood::STRESS_VIDEO_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

    end

    it ". When a user expands the Links accordion" do
      @mood.clickAccordionByHeading(links)
      expect(@mood.getAccordionTextByHeading(links)).to eq("American Heart Association (AHA)\nAmerican College of Cardiology (ACC)\nAmerican Association of Cardiovascular and Pulmonary Rehabilitation (AACVPR)\nPreventive Cardiovascular Nurses Association")

      @mood.moodAHALink_element.when_visible
      @mood.moodAHALink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreMood::AHA_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreMood::AHA_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

      @mood.moodACCLink_element.when_visible
      @mood.moodACCLink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreMood::ACC_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreMood::ACC_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

      @mood.moodAACVPRLink_element.when_visible
      @mood.moodAACVPRLink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreMood::AACVPR_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreMood::AACVPR_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

      @mood.moodPCNALink_element.when_visible
      @mood.moodPCNALink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == LearnMoreMood::PREVENTIVE_CARDIO_NURSE_ASSO_URL}
      expect(@site.getCurrentURL).to eq(LearnMoreMood::PREVENTIVE_CARDIO_NURSE_ASSO_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

    end

  end


end
