#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'


describe '[Story #CREHAB-170: Learn More - Nutrition Accordion]', :acceptance => true do
  include MongoUtility

  app_title = "VA FitHeart"
  quick_tips = "Quick Tips"
  calories = "Calories"
  weight = "Weight"
  weight_apps = "Weight Apps"
  links = "Links"

  before(:all) do
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @learnMore = @site.learnMore
    @nutrition = @site.nutrition
    @modal = @site.modal

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")
    end
  end

  context "#AC #CREHAB-350|TC CREHAB-383: Nutrition Option from App Options" do
    it ". When a user selects the Nutrition option from the App Options list, the user will see the following items in the detail panel/column 3:" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.learnMore_element.visible?).to eq(true)
      expect(@leftNav.learnMore_element.text).to eq("Learn More")

      @leftNav.learnMore
      @learnMore.primaryHeader_element.when_visible
      expect(@learnMore.primaryHeader_element.text).to eq("Learn More")

      expect(@learnMore.nutrition_element.text).to eq("Nutrition")

      @learnMore.nutrition
      Watir::Wait.until {@nutrition.primaryHeader_element.text == "Nutrition"}

    end

    it ". Nutrition Learn More sections displayed in an accordion control" do
      expect(@nutrition.displayedInAccordionControl?).to eq(true)
    end

    it ". < icon is displayed in phone form factors, alt text: Return to Learn More options" do
      @site.resizeWindowToPhone

      expect(@nutrition.backButton_element.visible?).to eq(true)
      expect(@nutrition.backButtonIcon_element.visible?).to eq(true)
      expect(@nutrition.backButton_element.attribute("title")).to eq("Return to Learn More options")
      @nutrition.backButton

      @learnMore.secondaryHeader_element.when_visible
      expect(@learnMore.secondaryHeader_element.text).to eq("Learn More")

      expect(@learnMore.fitness_element.text).to eq("Fitness")
      expect(@learnMore.health_element.text).to eq("Health")
      expect(@learnMore.nutrition_element.text).to eq("Nutrition")
      expect(@learnMore.mood_element.text).to eq("Mood")

      @site.resizeWindowToDefault
    end

  end

  context "#AC CREHAB-351|TC CREHAB-383: Accordion Display for Nutrition Accordion" do
    it ". Accordion display" do
      @learnMore.nutrition
      Watir::Wait.until {@nutrition.primaryHeader_element.text == "Nutrition"}

      expect(@nutrition.getAccordionLabelByHeading(quick_tips)).to eq("Quick Tips")
      expect(@nutrition.getAccordionLabelByHeading(calories)).to eq("Calories")
      expect(@nutrition.getAccordionLabelByHeading(weight)).to eq("Weight")
      expect(@nutrition.getAccordionLabelByHeading(weight_apps)).to eq("Weight Apps")
      expect(@nutrition.getAccordionLabelByHeading(links)).to eq("Links")

    end
  end

  context "#AC CREHAB-352|TC CREHAB-383: Accordion Behavior for Nutrition Accordion" do
    it ". Accordion sections can be opened and closed" do

      accordionArr = [quick_tips, calories, weight, weight_apps, links]

      accordionSize = accordionArr.length - 1

      #all accordions closed
      for i in 0..accordionSize
        expect(@nutrition.isAccordionExpanded(accordionArr[i])).to eq(false)
        expect(@nutrition.isRightArrowIconDisplayed(accordionArr[i])).to eq(true)
      end

      for j in 0..accordionSize
        expect(@nutrition.getAccordionAltText(accordionArr[j])).to eq("Expand\n" + accordionArr[j])
        @nutrition.clickAccordionByHeading(accordionArr[j])
        expect(@nutrition.isAccordionExpanded(accordionArr[j])).to eq(true)
        expect(@nutrition.getAccordionAltText(accordionArr[j])).to eq("Collapse\n" + accordionArr[j])
        expect(@nutrition.isDownArrowIconDisplayed(accordionArr[j])).to eq(true)

        for k in 0..accordionSize
          if (k != j)
            expect(@nutrition.isAccordionExpanded(accordionArr[k])).to eq(false)
            expect(@nutrition.isRightArrowIconDisplayed(accordionArr[k])).to eq(true)
          end
        end

        @nutrition.clickAccordionByHeading(accordionArr[j])
        expect(@nutrition.isAccordionExpanded(accordionArr[j])).to eq(false)
        expect(@nutrition.getAccordionAltText(accordionArr[j])).to eq("Expand\n" + accordionArr[j])
      end

    end
  end

  context "#AC CREHAB-353|TC CREHAB-383: Accordion display for Nutrition Accordion" do
    it ". When a user expands the Quick Tips accordion" do
      @nutrition.clickAccordionByHeading(quick_tips)
      expect(@nutrition.getAccordionTextByHeading(quick_tips)).to eq("A few tips that apply for just about every one:\nEat more fresh fruits and vegetables\nLimit portion size\nReduce salt intake\nMinimize sugar-sweetened beverages (soda, juice, energy drinks)\nStay away from junk food (chips, candy, sweets)\nAvoid high fat foods\nLearn more about healthy eating.")

      @nutrition.learnMoreHealthEatingLink_element.when_visible
      @nutrition.learnMoreHealthEatingLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::LEARN_MORE_HEALTHY_EATING_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::LEARN_MORE_HEALTHY_EATING_URL)
      @site.closeBrowser

      @site.switchWindowToFirst

    end

    it ". When a user expands the Calories accordion" do
      @nutrition.clickAccordionByHeading(calories)
      expect(@nutrition.getAccordionTextByHeading(calories)).to eq("A calorie refers to how much energy you can get from food. About 2000 calories are needed per day for weight maintenance, but you would need to eat fewer calories to lose weight. Determine the number of calories in foods by looking through Nutrition Facts labels, cookbooks, or the internet.\nUse the following link to look up how many calories are in foods.\nUSDA Food-A-Pedia (Calorie Look-Up)\nHawaii Foods (Calorie Look-Up)")

      @nutrition.hawaiiFoodsLink_element.when_visible
      @nutrition.hawaiiFoodsLink

      @site.switchWindowToLast

      puts "Site.getCurrentURL " + @site.getCurrentURL
      puts "Hwaii food url " + Nutrition::HAWAII_FOODS_URL

      Watir::Wait.until {@site.getCurrentURL == Nutrition::HAWAII_FOODS_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::HAWAII_FOODS_URL)
      @site.closeBrowser

      @site.switchWindowToFirst

      @nutrition.nutritionUSDACalorieLookUpLink_element.when_visible
      @nutrition.nutritionUSDACalorieLookUpLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::USDA_FOOD_A_PEDIA_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::USDA_FOOD_A_PEDIA_URL)
      @site.closeBrowser

      @site.switchWindowToFirst


    end

    it ". When a user expands the Weight" do
      @nutrition.clickAccordionByHeading(weight)
      expect(@nutrition.getAccordionTextByHeading(weight)).to eq("Maintaining a healthy weight is an important part of staying healthy. If you are overweight or obese, losing weight can help lower blood pressure, lower cholesterol, and improve blood sugar.\nLearn more about weight management.")

      @nutrition.learnMoreWeightManagementLink_element.when_visible
      @nutrition.learnMoreWeightManagementLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::LEARN_MORE_WEIGHT_MANAGEMENT_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::LEARN_MORE_WEIGHT_MANAGEMENT_URL)
      @site.closeBrowser

      @site.switchWindowToFirst
    end

    it ". When a user expands the Weight Apps accordion" do
      @nutrition.clickAccordionByHeading(weight_apps)
      expect(@nutrition.getAccordionTextByHeading(weight_apps)).to eq("MOVE! Coach\nMyFitnessPal")

      @nutrition.moveCoachLink_element.when_visible
      @nutrition.moveCoachLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::MOVE_COACH_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::MOVE_COACH_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

      @nutrition.fitnessPalLink_element.when_visible
      @nutrition.fitnessPalLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::MY_FITNESS_PAL_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::MY_FITNESS_PAL_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

    end

    it ". When a user expands the Links accordion" do
      @nutrition.clickAccordionByHeading(links)
      expect(@nutrition.getAccordionTextByHeading(links)).to eq("American Heart Association (AHA)\nAmerican College of Cardiology (ACC)\nAmerican Association of Cardiovascular and Pulmonary Rehabilitation (AACVPR)\nPreventive Cardiovascular Nurses Association\nUSDA Food-A-Pedia (Calorie Look-Up)\nHawaii Foods (Calorie Look-Up)")

      @nutrition.nutritionAHALink_element.when_visible
      @nutrition.nutritionAHALink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::AHA_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::AHA_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

      @nutrition.nutritionACCLink_element.when_visible
      @nutrition.nutritionACCLink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::ACC_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::ACC_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

      @nutrition.nutritionAACVPRLink_element.when_visible
      @nutrition.nutritionAACVPRLink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::AACVPR_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::AACVPR_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

      @nutrition.nutritionPCNALink_element.when_visible
      @nutrition.nutritionPCNALink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::PREVENTIVE_CARDIO_NURSE_ASSO_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::PREVENTIVE_CARDIO_NURSE_ASSO_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

      @nutrition.nutritionUSDACalorieLookUpLink2_element.when_visible
      @nutrition.nutritionUSDACalorieLookUpLink2
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::USDA_FOOD_A_PEDIA_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::USDA_FOOD_A_PEDIA_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

      @nutrition.nutritionHawaiiFoodsCaLorieLookUpLink_element.when_visible
      @nutrition.nutritionHawaiiFoodsCaLorieLookUpLink
      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == Nutrition::HAWAII_FOODS_URL}
      expect(@site.getCurrentURL).to eq(Nutrition::HAWAII_FOODS_URL)
      @site.closeBrowser
      @site.switchWindowToFirst

    end

  end


end
