require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../spec/health/health_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../shared_context'
require_relative 'notification_helper'

describe 'Story #CREHAB-396: Notification Count - Home Page', :regression => true do
  include DBUtility
  include MongoUtility

  before(:all) do
    removeAllDocuments()
    resetQuartzNotificationDb()

    insertNotification("Update your Cholesterol information.", true, true)
    insertNotification("Update your Mood information.", true, false)
    insertNotification("Update your Weight information.", false, true)
    insertNotification("Update your Blood Pressure & Pulse information.", false, false)

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @notifications = @site.notifications
    @notification_setting = @site.notification_setting
  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
    end
  end

  context "#AC CREHAB-1070|TC CREHAB-1082: Display of unread notification" do
    it ". Verify the count is displayed below progress as XX Unread in last 30 days" do
      expect(@landing.unreadNotifications).to eq("2 Unread in last 30 days")
    end

    it ". Including bell icon, title 'Notifications'" do
      expect(@landing.hasUnreadNotificationsIcon())
      expect(@landing.unreadNotificationTitle).to eq("Notifications")
    end

    it ". Verify unread count only includes app notifications when external notifications are disabled" do
      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu
      @leftNav.notifications_element.when_visible
      @leftNav.notifications

      updateNotificationPreference("test@test.com", "No", "(-05:00) America/New_York (Eastern)")

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu
      @leftNav.home_element.when_visible
      @leftNav.home

      @landing.contentLoaded_element.when_present(timeout=15)

      Watir::Wait.until {@landing.unreadNotifications == "1 Unread in last 30 days"}
      expect(@landing.unreadNotifications).to eq("1 Unread in last 30 days")
    end
  end
end

def insertNotification(body, isRead, isStaff)
  notification = {
    "patientIdentifier" => {
        "uniqueId" => "D123401",
        "assigningAuthority" => "EDIPI"
    },
    "readFlag" => isRead,
    "source" => "VA FitHeart - Health",
    "type" => isStaff ? "VA-Health" : "VA-FitHeart",
    "timeZone" => "America/New_York",
    "body" => body,
    "createDate" => Time.now,
    "effectiveDisplayDate" => Time.now
  }
  insertDocument("notifications", "notificationsdb", notification)
end
