require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../../spec/rspec_helper'
require_relative '../../spec/health/health_helper'
require_relative '../../lib/module/DateUtility'
require_relative '../../lib/module/MongoUtility'
require_relative '../../lib/module/DBUtility'

describe '[Story #CREHAB-397: Health Notifications - Basic Implementation]
          [Story #CREHAB-953: FitnessActivity Notifications - Basic Implementation]
          [Story #CREHAB-1097: Update and to & Blood Pressure and Pulse]', :acceptance => true do
  include DateUtility
  include MongoUtility
  include DBUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeAllDocuments()
    resetQuartzNotificationDb()

    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @leftNav = @site.leftMenu
    @modal = @site.modal
    @health = @site.health
    @feature_detail = @site.health_detail
    @select_mood = @site.select_mood
    @list_view = @site.list_view
    @error = @site.error
    @calendar = @site.calendar_picker
    @notification_detail = @site.notification_detail
    @notifications = @site.notifications
    @notification_setting = @site.notification_setting

    @today = getDateNthDaysFromNow(0, "%m/%d/%Y")
    @yesterday = getDateNthDaysFromNow(-1, "%m/%d/%Y")

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to
          #AC CREHAB-1097|TC CREHAB-1191: Update text for BP & Pulse" do
    it ". Login and navigate" do
    acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")
    navigateToHealth(app_title)
    end
  end

  subFeaturesParam = [
      { :featureName => 'Blood Pressure & Pulse Notification', :featureOptionName => 'Blood Pressure & Pulse', :frequency => "Weekly", :timeStr => ["12:00 PM"], :delivery => ["In-App"] },
      { :featureName => 'Weight Notification', :featureOptionName => 'Weight', :frequency => "Monthly", :timeStr => ["08:00 AM"], :delivery => ["Email"]  },
      { :featureName => 'Glucose Notification', :featureOptionName => 'Glucose', :frequency => "Daily", :timeStr => ["08:00 AM", "12:00 PM", "02:00 PM", "06:00 PM", "09:00 PM"], :delivery => ["In-App"]  },
      { :featureName => 'Cholesterol Notification', :featureOptionName => 'Cholesterol', :frequency => "Weekly", :timeStr => ["02:00 PM"], :delivery => ["Email"]  },
      { :featureName => 'Mood Notification', :featureOptionName => 'Mood', :frequency => "Weekly", :timeStr => ["02:00 PM"], :delivery => ["Email"]  },
      { :featureName => 'Fitness Activity Notification', :featureOptionName => 'Fitness Activity', :frequency => "Weekly", :timeStr => ["02:00 PM"], :delivery => ["Email"]  }
  ]

  subFeaturesParam.each do |feature|

    context '#AC CREHAB-1058|TC CREHAB-1069:  ' + feature[:featureName] + ' - Notifications feature for the FitHeart' do
      it ". a Notification button is shown in the utility bar to the right of the Graph button and Notification status reflects current notifiation status setting" do

        if(feature[:featureOptionName] == "Fitness Activity")
          @list_view = @site.fitness_listview
          @feature_detail = @site.fitness_activity

          @leftNav.leftMenu_element.when_visible
          @leftNav.leftMenu

          @leftNav.fitness_activity_element.when_visible
          @leftNav.fitness_activity

          @list_view.contentLoaded_element.when_present(timeout=15)
          expect(@list_view.primaryHeader_element.text).to eq("Fitness Activity")

          @notification_detail.notificationButton_element.when_visible
          @notification_detail.notificationButton
          @feature_detail.contentLoaded_element.when_present(timeout=10)
          expect(@feature_detail.primaryHeader_element.text).to eq(feature[:featureName])

          @notification_detail.selectARadioButton("Off", "notificationOption")

          @notification_detail.save_element.when_visible
          @notification_detail.save
          @feature_detail.contentLoaded_element.when_present(timeout=10)

        else
          navigateTo(feature[:featureOptionName])
        end

        expect(@notification_detail.listBellIcon_element.visible?).to eq(true)
        Watir::Wait.until {@notification_detail.notificationButton_element.text == "Notification (Off)"}
        expect(@notification_detail.notificationButton_element.text).to eq("Notification (Off)")
      end

      it ". When the user views the " + feature[:featureName]+ " Detail screen (in Add or Edit mode), a Notification section is displayed below the fields: " do
          @list_view.addButton_element.when_visible
          @list_view.addButton

          @feature_detail.contentLoaded_element.when_present(timeout=10)
          @feature_detail.date_element.when_visible

          expect(@notification_detail.detailBellIcon_element.exists?).to eq(true)
          expect(@notification_detail.notificationLabelText_element.text).to eq("Notification:")
          expect(@notification_detail.instructionalText_element.text).to eq("To edit the " + feature[:featureOptionName] + " notification, select the Notification button on the previous screen.")

          expect(@notification_detail.notificationStatusLabel_element.text).to eq("Status:")
          expect(@notification_detail.notificationStatus_element.text).to eq("Off")

          @feature_detail.cancel_element.when_visible
          @feature_detail.cancel

          @feature_detail.contentLoaded_element.when_present(timeout=10)
      end

    end

    context "#AC CREHAB-1059|TC CREHAB-1069: " +  feature[:featureName]  + " Notification Button for Filter Screen FitHeart" do
     it ". Header " do
       @notification_detail.notificationButton_element.when_visible
       @notification_detail.notificationButton
       @feature_detail.contentLoaded_element.when_present(timeout=10)
       expect(@feature_detail.primaryHeader_element.text).to eq(feature[:featureName])
     end

      it ". Bell icon, Notification Label" do
        expect(@notification_detail.detailBellIcon_element.exists?).to eq(true)
        expect(@notification_detail.notificationLbTextOnDetail_element.text).to eq("Notification:")
      end

      it ". Status (single select, radio button, Yes, No, default to Off) and Start Date, Notify Me, and Notification Delivery fields are all disabled" do
        expect(@notification_detail.notificationStatusLbOnDetail_element.text).to eq("Status:")
        expect(@notification_detail.notificationStatusOnDetail_selected?).to eq("Off")
        expect(@notification_detail.startDateOnDetail_element.attribute("disabled")).to eq("true")
        expect(@notification_detail.notifyMe_element.attribute("disabled")).to eq("true")
        expect(@notification_detail.notificationTimeWithStatusOff_element.attribute("ng-disabled")).to eq("ngDisabled")
      end

      it ". Status : On  - Notify Me, and Notification Delivery fields are all enabled and required." do
        @notification_detail.selectARadioButton("On", "notificationOption")
        expect(@notification_detail.startDateRequired).to eq("*")
        expect(@notification_detail.startDateLabel).to eq("Start Date:")
        expect(@notification_detail.notifyMeRequired).to eq("*")
        expect(@notification_detail.notifyMeLabel).to eq("Notify Me:")
        expect(@notification_detail.notificationDeliveryLabel.include?("*")).to eq(true)
        expect(@notification_detail.notificationDeliveryLabel.include?("Notification Delivery")).to eq(true)
      end

      it ". Verify fields are required messages display when Save it without providing values." do
        @notification_detail.save_element.when_visible
        @notification_detail.save

        errorMessages = [
             "Start Date field is required.",
             "A Notification Delivery method is needed to save this record. Please select a method and save again.",
             "Notify Me field is required."
        ]

        verifyErrorMessage(errorMessages)

      end

      it ". Verify that Start Date can not be in the past." do
         @notification_detail.startDateOnDetail= @yesterday
         @notification_detail.save

         errorMessages = ["Start Date must be between " + @today + " and 12/31/2099."]
         verifyErrorMessage(errorMessages)
      end

      it ". Notify Me (required if enabled, single select, select list, values: Select, Daily, Weekly, Monthly)" do
        expect(@notification_detail.notifyMe_options).to eq(["Select", "Daily", "Weekly", "Monthly"])
        expect(@notification_detail.notifyMe).to eq("Select")
      end

      it ". Verify that when select Notify Me to Daily, Weekly or Monthly, Notification Time field becomes required" do

         notifyMeArry = [
             :method => "Weekly", :errMsg => "Notification Time field is required.",
             :method => "Monthly", :errMsg => "Notification Time field is required.",
             :method => "Daily", :errMsg => "Notification Time number 1 field is required."
         ]

         notifyMeArry.each do |value|

           @notification_detail.notifyMe= value[:method]
           sleep 0.5
           expect(@notification_detail.notificationTimeLabel.include?("* ")).to eq(true)
           expect(@notification_detail.notificationTimeLabel.include?("Notification Time:")).to eq(true)
           @notification_detail.save

           errorMsgArr = [value[:errMsg]]
           verifyErrorMessage(errorMsgArr)
         end

      end

     it ". Verify that when select Daily, Add Time button is visible and can add up to five time entries. Each time entry has a delete button. Click the delete button remove the corresponding time entry" do
       expect(@notification_detail.addTimeBttn?).to eq(true)
       @notification_detail.setNotificationTime(0, "12:00 PM")
       for i in 1..4
         @notification_detail.addTimeBttn
         sleep 0.5
         expect(@notification_detail.verifyTheNewTimeEntryAppears(i)).to eq(true)
         expect(@notification_detail.isNewTimeEntryFocused?(i)).to eq(true)
         @notification_detail.setNotificationTime(i, "12:" + i.to_s +  "0 PM")
       end
       expect(@notification_detail.addTimeBttn?).to eq(false)

       5.downto(2) { |j|
         expect(@notification_detail.getDeleteTimeButtonAltText(j)).to eq("Delete 12:" + (j - 1).to_s +  "0 PM")
         @notification_detail.clickDeleteTimeButton(j)
         expect(@notification_detail.verifyTheNewTimeEntryAppears(j)).to eq(false)
         expect(@notification_detail.addTimeBttn?).to eq(true)
       }

     end

      it ". Duplicate notification times cannot be saved and Notification time fields are required." do
        @notification_detail.addTimeBttn_element.when_visible
        @notification_detail.addTimeBttn
        sleep 0.5
        @notification_detail.setNotificationTime(1, "12:00 PM")
        @notification_detail.save

        errorMsgArr = ["Duplicate notification times are not permitted."]
        verifyErrorMessage(errorMsgArr)

        @notification_detail.setNotificationTime(1, "")
        @notification_detail.save

        errorMsgArr = ["Notification Time number 2 field is required."]
        verifyErrorMessage(errorMsgArr)

      end

     it ". Verify that Notification Delivery has two checkboxes and none was pre-checked as default" do
       expect(@notification_detail.notificationDeliveryCBX_checked?).to eq(false)
     end

     it ". 	If preferred email is null, the Email check box is disabled." do
       expect(@notification_detail.isEmailNotificationDisabled).to eq("true")
     end

     it ". Verify that email address displays when email is available and it displays an instructional text" do
       @notification_detail.cancel
       @list_view.contentLoaded_element.when_present(timeout=10)

       @leftNav.leftMenu_element.when_visible
       @leftNav.leftMenu

       @leftNav.notifications_element.when_visible
       @leftNav.notifications

       @notifications.contentLoaded_element.when_present(timeout=10)

       @notifications.primaryHeader_element.when_visible
       expect(@notifications.secondaryHeader_element.text).to eq("Notifications")
       expect(@notifications.primaryHeader_element.text).to eq("Notifications")

       @notifications.notificationSettingButton_element.when_visible
       @notifications.notificationSettingButton

       @notifications.contentLoaded_element.when_present(timeout=10)

       @notification_setting.modalTitle_element.when_visible
       expect(@notification_setting.modalTitle_element.text).to eq("Notification Settings")
       @notification_setting.email = "test@test.com"
       @notification_setting.save2_element.when_visible
       @notification_setting.save2

       @notifications.contentLoaded_element.when_present(timeout=10)

       @leftNav.leftMenu_element.when_visible
       @leftNav.leftMenu

       if (feature[:featureOptionName] != "Fitness Activity")

         @leftNav.health_element.when_visible
         @leftNav.health

         @health.primaryHeader_element.when_visible
         expect(@health.secondaryHeader_element.text).to eq("Health")
         expect(@health.primaryHeader_element.text).to eq("Health")

        navigateTo(feature[:featureOptionName])
       else # Fitness Activity
         @leftNav.fitness_activity_element.when_visible
         @leftNav.fitness_activity

         @list_view.contentLoaded_element.when_present(timeout=10)

         @list_view.primaryHeader_element.when_visible
         expect(@list_view.primaryHeader_element.text).to eq("Fitness Activity")

       end

       @notification_detail.notificationButton_element.when_visible
       @notification_detail.notificationButton
       @feature_detail.contentLoaded_element.when_present(timeout=10)
       expect(@feature_detail.primaryHeader_element.text).to eq(feature[:featureName])

       @notification_detail.selectARadioButton("On", "notificationOption")

       expect(@notification_detail.displayedEmail).to eq("test@test.com")
       expect(@notification_detail.instruTextOnDetail).to eq("You may update your Preferred Email address in the Notification feature by selecting the Notification Settings icon")

     end

     it ". After a notification is saved with On status, when the user reopens the notification the start date is shown as read only and is not validated." do

       deliveryMethod = feature[:delivery][0]
       @notification_detail.addANotification(@today, feature[:frequency], feature[:timeStr], deliveryMethod)

       Watir::Wait.until {@notification_detail.notificationButton_element.text == "Notification (On)"}
       expect(@notification_detail.notificationButton_element.text).to eq("Notification (On)")

       @notification_detail.notificationButton
       @feature_detail.contentLoaded_element.when_present(timeout=10)
       Watir::Wait.until {@notification_detail.notificationStatusOnDetail_selected? == "On"}
       expect(@feature_detail.primaryHeader_element.text).to eq(feature[:featureName])

       expect(@notification_detail.notificationStatusOnDetail_selected?).to eq("On")

       expect(@notification_detail.startDateOnDetail_element.attribute("disabled")).to eq("true")
       expect(@notification_detail.startDateOnDetail).to eq(@today)
       expect(@notification_detail.notifyMe).to eq(feature[:frequency])
       #todo: time is not displayed when weekly or monthly is selected and saved
       expect(@notification_detail.getNotificationTimeArray(feature[:frequency])).to eq(feature[:timeStr])
       expect(@notification_detail.isNotificationDeliveryChecked(deliveryMethod)).to eq(true)

       @notification_detail.selectARadioButton("Off", "notificationOption")
       expect(@notification_detail.startDateOnDetail_element.attribute("disabled")).to eq("true")
       expect(@notification_detail.notifyMe_element.attribute("disabled")).to eq("true")
       expect(@notification_detail.notificationTimeWithStatusOff_element.attribute("ng-disabled")).to eq("ngDisabled")

       @notification_detail.selectARadioButton("On", "notificationOption")
       expect(@notification_detail.startDateOnDetail_element.attribute("aria-disabled")).to eq("false")

       expect(@notification_detail.notificationStatusOnDetail_selected?).to eq("On")
       expect(@notification_detail.startDateOnDetail).to eq("")
       expect(@notification_detail.notifyMe).to eq("Select")
       expect(@notification_detail.notificationTimeWithStatusOff).to eq("")
       expect(@notification_detail.notificationDeliveryCBX_checked?).to eq(false)

     end

    end

    #AC CREHAB-1060 (In App notification) and  -- this will be covered in story crehab-394
    #AC CREHAB-1061 (Email notification) - Manual testing

    context "#AC CREHAB-1062|TC CREHAB-1069: Previous Button Navigation for Kidney Tracker Notification" do
      it ". Navigation: A ''<'' icon is shown on the page header, alt text: Return to previous screen; " do
        expect(@notification_detail.backButton_element.visible?).to eq(true)
        expect(@notification_detail.backButtonIcon_element.visible?).to eq(true)
        expect(@notification_detail.backButton_element.attribute("title")).to eq("Return to Previous Screen")
      end

      it ". selecting the icon returns the user to the <Cholesterol/Blood Pressure & Pulse/Weight/Mood> list/filter screen." do
        @notification_detail.backButton
        @modal.modalTitle_element.when_visible
        @modal.continue
        @list_view.contentLoaded_element.when_present(timeout=15)
        expect(@list_view.primaryHeader).to eq(feature[:featureOptionName])

        #reset data
        removeCollection("preferences", "notificationsdb")
        setDefaultPreferencesForPatient("D123401")
        removeCollection("reminders", "remindersdb")
      end
    end

  end #end of SubFeaturesParam

end
