#$LOAD_PATH.unshift(File.join(File.dirname(__FILE__), '..', 'lib'))
require 'rubygems'
require 'page-object'
require 'page-object/page_factory'
require 'rspec'
require 'watir-webdriver'


BASE_URL = "http://localhost:8080/fit-heart"
LP_URL = "http://localhost:8080/launchpad"

RSpec.configure do |c|
  # declare an exclusion filter
  c.filter_run_excluding :broken => true
  c.filter_run_excluding :regression => true
  c.filter_run_excluding :acceptance => true
  c.filter_run_excluding :smoketest => true
end

RSpec.configure do |config|

  fetch_current_example = RSpec.respond_to?(:current_example) ? proc { RSpec.current_example } : proc { |context| context.example }

  config.include PageObject::PageFactory

  config.after(:each) do
    example = fetch_current_example.call(self);

    if example.exception
      meta = example.metadata
      filename = File.basename(meta[:file_path])
      line_number = meta[:line_number]
      buildJobUrl = ENV["JOB_URL"]
      buildJobNumber = ENV["BUILD_NUMBER"]
      screenshot_name = "screenshot-#{filename}-#{line_number}.png"
      screenshot_dir = "screenshots" + (buildJobNumber != nil ? ("_" + buildJobNumber.to_s) : "")
      screenshot_path = screenshot_dir + "/#{screenshot_name}"


      if !Dir.exists?(screenshot_dir)
        Dir.mkdir screenshot_dir
      end

      @driver.screenshot.save screenshot_path

      puts "----Screenshot ------------------------------------------------------------------------"
      puts meta[:full_description] + "\n  Screenshot: #{screenshot_path}\n  " + (buildJobUrl != nil ? buildJobUrl.to_s : "") + "/ws/acceptance_test/watir/" + screenshot_path
      puts "---------------------------------------------------------------------------------------"
    end
  end

end


###########
# methods #
##########

def acceptEulaAndQuickLogin(userName, password, userId )
  firstName = userName.split('.')[1]
  lastName = userName.split('.')[0]

  setDefaultPreferencesForPatient(userId)
  setDefaultNotificationPreferencesForPatient(userId, firstName, lastName)

  Watir::Wait.until {@eula.accept_element.exists?}
  @eula.accept_element.when_visible
  @eula.accept

  @landing.login_element.when_visible
  @landing.login

  Watir::Wait.until {@login.login_element.exists?}

  expect(@landing.appTitle).to eq("Department of Veterans Affairs")

  @login.login_with(userName, password, userId)
  @landing.appTitle_element.when_visible

  #click No for Weekly Activity Goals if modal displays
  if (@modal.no_element.exists?)
    @modal.no
    @modal.no_element.when_not_visible
  end

end

def verifyErrorMessage (errorMessages)
  Watir::Wait.until {@error.errorSummaryHeader_element.exists?}
  Watir::Wait.until {@error.errorSummaryHeader_element.text == "The following errors were found:"}

  @error.errorSummaryHeader_element.when_visible
  expect(@error.errorSummaryHeader_element.text).to eq("The following errors were found:")
  #expect(@error.errorSummaryHeader_element.focused?).to eq(true)
  expect(@error.getFocusedElement() == @error.errorSummaryHeader_element).to eq(true)

  errorMessages.each_with_index do | expectedErrorMessage, i |

    expect(@error.isErrorMsgDisplayed?(expectedErrorMessage)).to eq(true)
  end

end



