require_relative '../lib/helper/table'

RSpec.shared_examples "Graph/Table Button Validation"  do
  it ".  When a user opens the summary view (list and filter) detail screen, a graph button is shown in the utility bar (right justified, shows graph icon and is labeled 'Graph', alt text matches label)" do
    puts "Starting Graph Button validation"
    expect(@list_view.graph_element.visible?).to eq(true)
    expect(@list_view.graph_element.text).to eq("Graph")
  end

  it ". When a user selects the graph icon, the table list is replaced by the graph view:" do
    @list_view.graph

    Watir::Wait.until { @list_view.tableReferenceMsg_element.text == "A text description of the information on this page is available in the table view." }
    expect(@list_view.tableReferenceMsg_element.text).to eq("A text description of the information on this page is available in the table view.")
  end

  it ". The Graph button in the utility bar toggles to the 'Table' button view (displays table icon, is labeled 'Table', alt text matches label)." do
    expect(@list_view.table_element.visible?).to eq(true)
    expect(@list_view.table_element.text).to eq("Table")
    @list_view.table

    Watir::Wait.until {@list_view.tableList_elements.length > 0}
    expect(@list_view.tableList_elements.length > 0).to eq(true)

  end
end


RSpec.shared_examples "Add Button Validation" do
  it ". When the user views the Blood Pressure & Pulse Table list, an Add button (+ icon, alt text Add), is available in the upper right corner of the Detail screen." do
    @list_view.addButton_element.when_visible
    expect(@list_view.addButton_element.attribute("title")).to eq("Add")
  end

  it ". When a user selects Add, the Add Blood Pressure & Pulse screen is shown, header: Add Blood Pressure & Pulse" do
    @list_view.addButton

    Watir::Wait.until {@detail.primaryHeader_element.text == "Add " + @title + " Entry"}
    expect(@detail.primaryHeader_element.text).to eq("Add " + @title + " Entry")
  end
end

RSpec.shared_examples "Labels Validation"  do
  it ". Detailed fields include the following: Note: Unless otherwise indicated, all values default to null (or select for drop lists), no guidance text (light gray placeholder text) is shown, and all fields are optional." do
    Watir::Wait.until {@detail.date_element.visible? == true}
    @detail.date_element.when_visible

    formContent = @detail.getScreenContent().split("\n")

    @EXPECTED_ROW_0_FORM.each_with_index do | expectedScreenRow, i |
    expect(formContent[i]).to eq(expectedScreenRow)
    end
  end

end

RSpec.shared_examples "Date Validation" do
  it ". Date: (required, textbox, date, date picker, MM/DD/YYYY format, show light gray text as format mask; format is read by screen reader, default to today)" do
    expect(@detail.date_element.attribute("required")).to eq("true")
    expect(@detail.date_element.attribute("placeholder")).to eq("MM/DD/YYYY")
    expect(@detail.date).to eq(@today)

    tomorrow = getDateNthDaysFromNow(1, "%m/%d/%Y")
    sixYearsAgo = getDateNthYearsFromNow(-6, "%m/%d/%Y")

    @detail.calendarPicker_element.when_visible
    @detail.calendarPicker_element.click

    Watir::Wait.until {@calendar.isNthCalendarDisplayed(1) == true}
    expect(@calendar.isNthCalendarDisplayed(1)).to eq(true)

    expect(@calendar.getcurrentYear()).to eq(getYear(@today))
    expect(@calendar.getCurrentMonth).to eq(getMonthOfYear(@today))
    expect(@calendar.getCurrentDay()).to eq(getDay(@today))

    @calendar.pick(tomorrow)
    expect(@calendar.isDayEnabled(getDay(tomorrow))).to eq(false)

    @calendar.pick(sixYearsAgo)
    Watir::Wait.until {@detail.date == sixYearsAgo}
    expect(@detail.date).to eq(sixYearsAgo)

    @detail.date = tomorrow
    @detail.save_element.when_visible

    @detail.save_element.click

    errorMessages = ["Date must be between " + sixYearsAgo + " and " + @today.to_s + "."]
    verifyErrorMessage(errorMessages)

    @detail.date = "01"
    @detail.save_element.click

    errorMessages = ["Date must be formatted MM/DD/YYYY."]
    verifyErrorMessage(errorMessages)

    @detail.date = ""
    @detail.save_element.click

    errorMessages = ["Date field is required."]
    verifyErrorMessage(errorMessages)

  end

end

RSpec.shared_examples "Time Validation" do
  it ". Time: (required, textbox, time, time picker, HH:MM AM/PM, default to now)" do
    expect(@detail.time_element.attribute("required")).to eq("true")
    expect(@detail.time_element.attribute("placeholder")).to eq("HH:MM AM/PM")

    @detail.date = @today
    @detail.time = getNthMinFromNow(2)

    @detail.save_element.click

    errorMessages = ["Time cannot be a future time."]
    verifyErrorMessage(errorMessages)

    @detail.time = ""

    @detail.save_element.click

    errorMessages = ["Time field is required."]
    verifyErrorMessage(errorMessages)

  end
end

RSpec.shared_examples "Note Validation" do
  it ".  Note (Alphanumeric, Text - 250 characters, show character countdown)" do
    notes_250 = "AAA4567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890"
    notes_260 = notes_250 + "ABCDEFGHIJ"
    expect(@detail.characterCounter_element.text).to eq("250 characters left")

    @detail.note  = notes_250
    expect(@detail.note).to eq(notes_250)
    expect(@detail.characterCounter_element.text).to eq("0 characters left")
    @detail.note = notes_260
    expect(@detail.note).to eq(notes_250)
    expect(@detail.characterCounter_element.text).to eq("0 characters left")
  end
end

RSpec.shared_examples "Date Filter Validation"  do
    it ". Filter defaults to an expanded view. Filter button alt text reads 'Collapse filter options' when expanded." do
     puts "Starting Date Filter Validation....."

      @date_filter.filterSection_element.when_visible
      expect(@date_filter.isFilterExpanded).to eq(true)
      expect(@date_filter.filterSection_element.attribute("title")).to eq("Collapse filter options")

      #Users can collapse the filter list by clicking on the Collapse filter icon. When collapsed, the filter icon alt text reads “Expand filter options.”
      @date_filter.filterSection
      expect(@date_filter.isFilterExpanded).to eq(false)
      expect(@date_filter.filterSection_element.attribute("title")).to eq("Expand filter options")

      @date_filter.filterSection
    end

    it ". Start Date (required, date value, date picker, MM/DD/YYYY format, show light grey text as format masks, and Constrain fields to match format; prevent future year entry and invalid month or day entry)" do

      expect(@date_filter.dateRangeLabel_element.text).to eq("Date Range:")

      expect(@date_filter.startDateLabel_element.text).to eq("* Start Date:\nMM/DD/YYYY")
      expect(@date_filter.startDate_element.attribute("required")).to eq("true")
      expect(@date_filter.startDate_element.attribute("placeholder")).to eq("MM/DD/YYYY")

      expect(@date_filter.startDate).to eq(@threeMonthAgo)

      @date_filter.startDate =  @tomorrow
      @date_filter.endDate = @today

      @date_filter.filter

      errorMessages = ["Start Date must occur before End Date.",  "Start Date must be between " + @sixYearsAgo + " and " + @today + "."]
      verifyErrorMessage(errorMessages)


      @date_filter.startDate = @sixYearsAndOneDayAgo
      @date_filter.filter

      #puts "Six Years and one day ago " + sixYearsAndOneDayAgo
      errorMessages = ["Start Date must be between " + @sixYearsAgo + " and " + @today + "."]
      verifyErrorMessage(errorMessages)

      @date_filter.startDate = "03/"
      @date_filter.filter

      #puts "Six Years and one day ago " + sixYearsAndOneDayAgo
      errorMessages = ["Start Date must be formatted MM/DD/YYYY."]
      verifyErrorMessage(errorMessages)

      @date_filter.startDatePicker
      expect(@date_filter.isStartDateCalendarDisplayed).to eq(true)
      @calendar.pick(@hundredDaysAgo)
      Watir::Wait.until {@date_filter.startDate == @hundredDaysAgo}
      expect(@date_filter.startDate).to eq(@hundredDaysAgo)

      @date_filter.startDatePicker
      expect(@date_filter.isStartDateCalendarDisplayed).to eq(true)
      @calendar.pick(@sixYearsAgo)
      Watir::Wait.until {@date_filter.startDate == @sixYearsAgo}
      expect(@date_filter.startDate).to eq(@sixYearsAgo)

      @date_filter.startDatePicker
      expect(@date_filter.isStartDateCalendarDisplayed).to eq(true)
      @calendar.pick(@sixYearsAndOneDayAgo)
      expect(@calendar.isDayEnabled(getDay(@sixYearsAndOneDayAgo))).to eq(false)

      @calendar.pick(@sixYearsAgo)
      Watir::Wait.until {@date_filter.startDate == @sixYearsAgo}
      expect(@date_filter.startDate).to eq(@sixYearsAgo)

      @date_filter.startDatePicker
      expect(@date_filter.isStartDateCalendarDisplayed).to eq(true)

      @calendar.pick(@tomorrow)
      expect(@calendar.isDayEnabled(getDay(@tomorrow))).to eq(false)

      @calendar.pick(@today)
      Watir::Wait.until {@date_filter.startDate == @today}
      expect(@date_filter.startDate).to eq(@today)
    end

    it ". End Date (required, date value, date picker, MM/DD/YYYY format, show light grey text as format masks, and Constrain fields to match format; prevent future year entry and invalid month or day entry) " do
      expect(@date_filter.endDateLabel_element.text).to eq("* End Date:\nMM/DD/YYYY")
      expect(@date_filter.endDate_element.attribute("required")).to eq("true")
      expect(@date_filter.endDate_element.attribute("placeholder")).to eq("MM/DD/YYYY")

      @date_filter.startDate = @tomorrow
      @date_filter.endDate = @today
      @date_filter.filter

      errorMessages = ["Start Date must occur before End Date."]
      verifyErrorMessage(errorMessages)

      threeDaysLater = getDateNthDaysFromNow(3, "%m/%d/%Y")
      @date_filter.startDate = @today
      @date_filter.endDate = threeDaysLater
      @date_filter.filter

      errorMessages = ["End Date must be between " + @sixYearsAgo + " and " + @today + "."]
      verifyErrorMessage(errorMessages)

      @date_filter.endDate = "03"
      @date_filter.filter

      errorMessages = ["End Date must be formatted MM/DD/YYYY."]
      verifyErrorMessage(errorMessages)

      expect(@date_filter.requiredFieldLabel_element.text).to eq("* required field")
    end

end

RSpec.shared_examples "Delete Validation" do
  it ". Delete -   When a user selects Delete, the system prompts the user to confirm the Delete by displaying a modal:" do
    count = @list_view.tableList_elements.length
    for i in 0..(count -1)
      @list_view.tableList_elements[0].click

      @detail.primaryHeader_element.when_visible
      expect( @detail.primaryHeader_element.text).to eq("Edit " + @title + " Entry")

      @detail.delete_element.when_visible
      @detail.delete

      @modal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Confirmation")
      expect(@modal.section1_element.text).to eq("Are you sure you want to remove this entry? Select YES to remove the entry or NO to return to the entry screen.")
      @modal.no_element.when_visible
      expect(@modal.no_element.text).to eq("No")
      @modal.no
      @modal.no_element.when_not_visible

      @detail.delete_element.when_visible
      expect(@detail.delete_element.focused?).to eq(true)
      @detail.delete

      @modal.modalTitle_element.when_visible
      expect(@modal.modalTitle_element.text).to eq("Confirmation")
      expect(@modal.section1_element.text).to eq("Are you sure you want to remove this entry? Select YES to remove the entry or NO to return to the entry screen.")
      @modal.yes_element.when_visible
      expect(@modal.yes_element.text).to eq("Yes")
      @modal.yes
      @modal.yes_element.when_not_visible

      @list_view.primaryHeader_element.when_visible
      expect(@list_view.primaryHeader_element.text).to eq(@title)
      Watir::Wait.until {@list_view.tableList_elements.length ==  (count - (i + 1))}

    end

  end
end

RSpec.shared_examples "Back button validation for Phone view" do
  it ". When a phone user views the Filter and Table List View Detail screen, a < is shown (alt text: Return to <feature name> options); when clicked the user is returned to the Main Screen for the feature where the group options for that feature are listed." do
    @site.resizeWindowToPhone
    expect(@list_view.backButton_element.visible?).to eq(true)
    expect(@list_view.backButton_element.attribute("Return to Health options"))
    @list_view.backButton_element.click

    @health.bloodPressureAndPulse_element.when_visible
    Watir::Wait.until { @list_view.secondaryHeader_element.text == "Health" }
    Watir::Wait.until { @health.bloodPressureAndPulse_element.text.strip == "Blood Pressure\nPulse" }

    expect(@health.bloodPressureAndPulse_element.text.strip).to eq("Blood Pressure\nPulse")
    @site.resizeWindowToDefault
  end
end

RSpec.shared_examples "Notification Fields Validation" do
  it ". Learn More Link" do
    expect(@notification_setting.learnMoreLink_element.visible?).to eq(true)
    @notification_setting.learnMoreLink

    @site.switchWindowToLast
    Watir::Wait.until {@user_guide.userGuideHeader_element.text == "VA FitHeart User Guide"}
    expect(@user_guide.userGuideHeader_element.text).to eq("VA FitHeart User Guide")
    @site.closeBrowser
    @site.switchWindowToFirst
  end

  it ". Notification Settings, Required Fields, 	Email Address (text box, 100 characters)" do
    expect(@notification_setting.requiredFieldsText_element.text).to eq("* required field")

    expect(@notification_setting.emailLabel_element.text).to eq("Email Address:")
    emailAddress_100_text = "1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890"

    @notification_setting.email = emailAddress_100_text +  "@test.com"
    expect(@notification_setting.email).to eq(emailAddress_100_text)
    @notification_setting.email = "test@test.com"
  end

  it ". Time Zone (required, standard time zone list, default to 'select' if user has not established a preference setting via another app, otherwise display user's set time zone preference)" do
    expect(@notification_setting.timeZoneSelect_element.attribute("required")).to eq("true")
    @notification_setting.timeZoneSelect = "Select"
    @notification_setting.save2_element.when_visible
    @notification_setting.save2

    errorMsgArr = ["Time Zone field is required."]
    verifyErrorMessage(errorMsgArr)
    @notification_setting.timeZoneSelect = "(-05:00) America/New_York (Eastern)"
  end

  it ". Receive messages to help improve your health (single select, radio button, values: Yes, No, default to yes)" do
    expect(@notification_setting.receiveMsgLegend_element.text).to include("Receive messages to help improve your health")
    expect(@notification_setting.receiveMsgOption_values).to eq(["Yes", "No"])
    @notification_setting.select_receiveMsgOption("No")
  end

end

RSpec.shared_examples "Phone User: Options control" do
  it ".Option control for Phone uer" do
      @site.resizeWindowToPhone
      @list_view.options_element.when_visible
      expect(@list_view.options_element.text).to eq("Options")
  end
  it ". When a user selects the Options button, a list of available options is displayed:" do
      @list_view.options
      expect(@list_view.graphOption_element.text).to eq("Graph")
      expect(@list_view.notificationOption_element.text).to include("Notification")
  end
  it ". When a user selects an option in the Options list the appropriate screen is displayed." do
      @list_view.graphOption_element.when_visible
      @list_view.graphOption

      Watir::Wait.until { @list_view.tableReferenceMsg_element.text == "A text description of the information on this page is available in the table view." }
      expect(@list_view.tableReferenceMsg_element.text).to eq("A text description of the information on this page is available in the table view.")

      @list_view.options
      expect(@list_view.tableOption_element.visible?).to eq(true)
      @list_view.notificationOption_element.when_visible
      @list_view.notificationOption

      @notification_detail.contentLoaded_element.when_present(timeout=10)
      expect(@notification_detail.primaryHeader_element.text).to include("Notification")
      @notification_detail.cancel

      @list_view.contentLoaded_element.when_present(timeout=10)

      @list_view.options_element.when_visible
      @list_view.options

      expect(@list_view.graphOption_element.text).to eq("Graph")
      expect(@list_view.notificationOption_element.text).to include("Notification")
      expect(@list_view.options_element.attribute("aria-label")).to eq("Collapse Options list button")

      @list_view.options
      Watir::Wait.until {@list_view.options_element.attribute("aria-label") == "Expand Options list button"}

      expect(@list_view.options_element.attribute("aria-label")).to eq("Expand Options list button")

      @site.resizeWindowToDefault
  end
end
