#!/bin/env ruby
# encoding: utf-8
require 'rspec'
require 'watir-webdriver'

require_relative '../../lib/browser_container/site'
require_relative '../rspec_helper'
require_relative '../../lib/module/MongoUtility'

describe '[Story #CREHAB-19: FitHeart - Support feature]
          [Story #CREHAB-20: FitHeart:Support - Mindfulness]
          [Story #CREHAB-21: FitHeart:Support - People Like You]
          [Story #CREHAB-22: FitHeart:Support - Family and Friends]
          [Story #CREHAB-23: FitHeart:Support - Health Providers]
          [Story #CREHAB-710: Change breathing and grounding from Learn to Support section]', :acceptance => true do
  include MongoUtility

  app_title = "VA FitHeart"

  before(:all) do
    removeCollection("fitness", "healthfactordb")
    removeCollection("fitnessgoals", "healthfactordb")
    @driver = Watir::Browser.new
    @site = Site.new(BASE_URL, @driver)

    @login = @site.loginPage
    @eula = @site.eula
    @roa = @site.roa
    @landing = @site.landing
    @support = @site.support
    @leftNav = @site.leftMenu
    @modal = @site.modal

    @accordionArr = ["What is Mindfulness?", "Mindfulness Meditation", "Compassion Meditation"]

  end

  after(:all) do
    @site.quitDriver
  end

  context "[Before] Log in and navigate to" do
    it ". Login and navigate to" do
      acceptEulaAndQuickLogin("zztest.patient01", "pass", "D123401")

      expect(@landing.appTitle).to eq(app_title)
      @landing.welcomeTitleForAuthenticatedUser_element.when_visible
      expect(@landing.welcomeTitleForAuthenticatedUser).to eq("Welcome to VA FitHeart")

      @leftNav.leftMenu_element.when_visible
      @leftNav.leftMenu

      @leftNav.title_element.when_visible
      expect(@leftNav.support_element.visible?).to eq(true)
      expect(@leftNav.support_element.text).to eq("Support")
    end
  end

  context "#AC #CREHAB-159|TC #CREHAB-187: Support Feature Selection" do
    it ". When the user selects the Supoprt feature from the left panel menu, the system displays the Feature grups in the Main Screen" do
      @leftNav.support

      Watir::Wait.until {@support.primaryHeader_element.text == "Support"}
      expect(@support.primaryHeader_element.text).to eq("Support")
      expect(@support.secondaryHeader_element.text).to eq("Support")

      @support.mindfulness_element.when_visible
      expect(@support.mindfulness_element.text).to eq("Mindfulness")
      expect(@support.pplLikeYou_element.text).to eq("People Like You")
      expect(@support.familyFriends_element.text).to eq("Family and Friends")
      expect(@support.healthProviders_element.text).to eq("Health Providers")

      expect(@support.getNthOption(1)).to eq("Mindfulness")
      expect(@support.getNthOption(2)).to eq("People Like You")
      expect(@support.getNthOption(3)).to eq("Family and Friends")
      expect(@support.getNthOption(4)).to eq("Health Providers")
    end

    it ". Instructional text is displayed in the detail screen " do
      expect(@support.placeholderMessage_element.text).to eq("Select an item from the Support list to view details.")

    end
  end

  context "#CREHAB-1043|TC CREHAB-1080: Accordions for Mindfulness" do
    it ". When a user selects the Mindfulness feature (column 2) from the Support feature, the user will see the following items in the detail panel/column 3:" do

      @support.mindfulness_element.when_visible
      @support.mindfulness

      @support.contentLoaded_element.when_present(timeout=10)
      expect(@support.primaryHeader_element.text).to eq("Mindfulness")
    end
    it ". <  (Return to Support button) on phone form factors" do
      verifyBackButtonOnPhoneView
    end
  end

  context "#CREHAB-1044|TC CREHAB-1080: Accordion Display" do
    it ". Accordion sections are shown labeled as follows; all sections are shown closed by default:" do
      @support.mindfulness_element.when_visible
      @support.mindfulness

      @support.contentLoaded_element.when_present(timeout=10)
      expect(@support.primaryHeader_element.text).to eq("Mindfulness")

      accordionSize = @accordionArr.length - 1
      for i in 0..accordionSize
        expect(@support.getAccordionLabelByHeading(@accordionArr[i])).to eq(@accordionArr[i])
      end

    end
  end

  context "#AC CREHAB-1045|TC CREHAB-1080: Accorion Section Display" do
    it ". When a user expands the What is Mindfulness accordion, the following content is displayed:" do
      @support.clickAccordionByHeading(@accordionArr[0])
      expect(@support.getAccordionTextByHeading(@accordionArr[0])).to eq("Mindfulness means paying attention, on purpose, to whatever is going on in the present moment without passing judgment on it. This practice involves noticing something and holding it in your attention lightly, with gentle acceptance, like a butterfly resting on your hand, until it moves away on its own. In mindfulness, you attend to a feeling, thought, or sensation without clinging to it, resisting it, or trying to change it.\nWhy Practice Mindfulness?\nIncrease sense of inner calm\nExperience both pleasant and unpleasant thoughts and feelings safely\nLearn through experience that everything changes; that thoughts and feelings come and go like the weather\nHave more balance, less emotional reactivity\nDevelop self-acceptance and self-compassion\nIncrease self-awareness\nFeel more connected to yourself, to others and to the world around you\nHave more direct contact with the world, rather than living through your thoughts\nBecome less judgmental of yourself and others\nReduce the struggle with anxiety and depression over time\nCope more effectively with chronic pain\nHow to practice mindfulness\nMindfulness can be practiced at any time, any place, anywhere, and no one has to know.\nUse mindfulness when faced with a trigger, anger, unwanted craving, or pain to help move past negative items.\nFocus on the present, not items in the past or those that may occur in the future.")
    end

    it ". When a user expands the Mindfulness Meditation accordion, the following content is displayed:" do
      @support.clickAccordionByHeading(@accordionArr[1])
      expect(@support.getAccordionTextByHeading(@accordionArr[1])).to eq("Goal: Be fully aware of what is going on in the present moment; let thoughts float in and out without any judgment.\nTo anchor yourself in the present, focus on your breath as you inhale and exhale. Breathing mindfully does not involve a conscious changing of your breath.\nTell yourself that you will not engage your thoughts. As thoughts pop up, just notice them and let them go.\nAs you notice your mind thinking about things, try to let those thoughts go without judging them and return your attention to the experience of breathing. The point is not to stop having thoughts. The point is to become more aware of your thoughts without automatically engaging them.\nBeing distracted is normal; our brains evolved to be vigilant. When you concentrate on your breath, don’t be upset when your mind wanders. Instead, simply observe the distraction. Your breath should remain your primary focus.\nIf focusing on breath does not work, you can focus on anything in the present (e.g., sounds, a picture, a candle), so long as you don’t have to think about it.")
    end

    it ". When a user expands the Compassion Meditation accordion, the following content is displayed:" do
      @support.clickAccordionByHeading(@accordionArr[2])
      expect(@support.getAccordionTextByHeading(@accordionArr[2])).to eq("Goal: Increase compassion for yourself and others.\nRepeat these phrases to yourself for a minute or two:\nMay I be happy.\nMay I be well.\nMay I be safe.\nMay I be peaceful and at ease.\nVisualize extending this energy to include somebody you care deeply about. Repeat the phrases, filling in the person’s name (instead of I).\nNow extend the energy to somebody for whom you have negative feelings. Repeat the phrases with his or her name.\nFinally, visualize extending this energy to the rest of the world.")
    end

  end

  context "AC #CREHAB-219|TC CREHAB-253: People Like You Sub-Feature Selection" do
    it ". When the user selects the People Like You option from the Support feature menu, the system displays the People Like You detail screen" do
      @support.pplLikeYou_element.when_visible
      @support.pplLikeYou
      Watir::Wait.until {@support.primaryHeader_element.text == "People Like You"}
      expect(@support.primaryHeader_element.text).to eq("People Like You")
      expect(@support.contentTitle_element.text).to eq("People Like You")

      @support.textSection_element.when_visible
      expect(@support.textSection_element.text).to eq("People Like You\nTalk to people like you about heart disease through an online support group.\nMended Hearts – Heart Disease Support Community")

      @support.mendedHeartsLink_element.when_visible
      @support.mendedHeartsLink

      @site.switchWindowToLast
      Watir::Wait.until {@site.getCurrentURL == "https://www.inspire.com/groups/mended-hearts-heart-disease/"}

      expect(@site.getCurrentURL).to eq("https://www.inspire.com/groups/mended-hearts-heart-disease/")
      @site.closeBrowser

      @site.switchWindowToFirst
    end
  end

  context "AC #CREHAB-220|TC #CREHAB-253: Return to Support button" do
    it ". <  (Return to Support button) on phone form factors" do
      verifyBackButtonOnPhoneView
    end
  end

  context "AC #CREHAB-222|TC #CREHAB-254: Famly and Friends Sub-feature Selection" do
    it ". When the user selects the Family and Friends option from the Support feature menu, the system displays the Family and Friends detail screen." do
      @support.familyFriends_element.when_visible
      @support.familyFriends
      Watir::Wait.until {@support.primaryHeader_element.text == "Family and Friends"}
      expect(@support.primaryHeader_element.text).to eq("Family and Friends")
      expect(@support.contentTitle_element.text).to eq("Family and Friends")

      @support.textSection_element.when_visible
      expect(@support.textSection_element.text).to eq("Family and Friends\nFamily and friends can be a great source of support and encouragement. Sometimes working with a buddy can help you stay healthy.")

    end

  end

  context "AC #CREHAB-226|TC #CREHAB-254: Return to Support for Family and Friends" do
    it ". <  (Return to Support button) on phone form factors" do
      verifyBackButtonOnPhoneView
    end
  end

  context "AC #CREHAB-229|TC CREHAB-255: health Providers Sub-Feature selection" do
      it ". When the user selects the Health Providers option from the Support feature menu, the system displays the Health Providers detail screen" do
        @support.healthProviders_element.when_visible
        @support.healthProviders
        Watir::Wait.until {@support.primaryHeader_element.text == "Health Providers"}
        expect(@support.primaryHeader_element.text).to eq("Health Providers")
        expect(@support.contentTitle_element.text).to eq("Health Providers")

        @support.textSection_element.when_visible
        expect(@support.textSection_element.text).to eq("Health Providers\nTalk to your health care provider about your health or related concerns.\nIt is good to:\nBe prepared\nAsk questions\nGive information\nRepeat back information and instructions\nTell your provider your preferences\nTell your provider your concerns\nYou can use the My VA Health App to help you prepare for your next appointment.\nMy VA Health")

        @support.myVaHealthLink_element.when_visible
        expect(@support.myVaHealthLink_element.text).to eq("My VA Health")
        @support.myVaHealthLink

        #todo: need to deploy my Va Health in MAE
        # @site.switchWindowToLast
        # Watir::Wait.until {@site.getCurrentURL == "http://localhost:8080/my-va-health/#/eula"}
        #
        # Watir::Wait.until {@eula.accept_element.exists?}
        # @eula.eulaTitle_element.when_visible
        #
        # @eula.accept_element.when_visible
        # @eula.accept
        # @support.appTitle_element.when_visible
        #
        # expect(@support.appTitle).to eq("My VA Health")
        # expect(@site.getCurrentURL).to eq("http://localhost:8080/my-va-health/#/home")
        #
        # @site.closeBrowser
        #
        # @site.switchWindowToFirst

      end
  end

  context "AC #CREHAB-230|TC #CREHAB-255: Return to Support for Health Provider" do
    it ". <  (Return to Support button) on phone form factors" do
      verifyBackButtonOnPhoneView
    end
  end


  def verifyBackButtonOnPhoneView
    @site.resizeWindowToPhone

    expect(@support.backButton_element.visible?).to eq(true)
    expect(@support.backButtonIcon_element.visible?).to eq(true)
    expect(@support.backButton_element.attribute("title")).to eq("Return to Support options")
    @support.backButton

    @support.secondaryHeader_element.when_visible
    expect(@support.secondaryHeader_element.text).to eq("Support")

    @support.mindfulness_element.when_visible
    expect(@support.mindfulness_element.text).to eq("Mindfulness")
    expect(@support.pplLikeYou_element.text).to eq("People Like You")
    expect(@support.familyFriends_element.text).to eq("Family and Friends")
    expect(@support.healthProviders_element.text).to eq("Health Providers")

    @site.resizeWindowToDefault
  end
end
