define(['FitnessEntryController', 'FitnessConstants'], function () {
	'use strict';

	describe("The Fitness Controller", function () {
		var controller,
			scope,
			globalDelayTrackerMock,
			focusServiceMock,
			modalServiceMock,
			goalsServiceMock,
			formatterMock,
			goalsServiceFetchSpy,
			createEmptySpy,
			openGoalsInfoModalSeen,
			goalsServiceSaveSpy,
			hasBeenPrompted = true,
			modal,
			q,
			createEmptyGoalReturn = {
				activeFlag : false,
				startDate : null,
				target : {
					minutes: null,
					steps : null,
					miles : null
				}
			},
			regularGoal = {
				startDate : "03/30/2015",
				activeFlag : true,
				target : {
					minutes : 20,
					steps : 2000,
					miles : 2
				}
			},
			goalServiceInstance,
			dataPointersMock = [regularGoal];

		beforeEach(function () {
			module('angularTemplateApp');

			globalDelayTrackerMock = jasmine.createSpyObj('globalDelayTracker', ['active']);
			focusServiceMock = jasmine.createSpyObj('focusService', ['focusMain']);
			modalServiceMock = jasmine.createSpyObj('modalService', ['showModal']);
			goalsServiceMock = jasmine.createSpyObj('WeeklyGoalsService', ['instance']);
			formatterMock = jasmine.createSpyObj('formatterMock', ['getFormattedFrontendDate', 'getDateTimeMinusDelta']);
			createEmptySpy = jasmine.createSpy('createEmptySpy').and.returnValue(createEmptyGoalReturn);

			formatterMock.getFormattedFrontendDate.and.returnValue(0);
			formatterMock.getDateTimeMinusDelta.and.returnValue(1);

			//goalsServiceMock.instance.and.returnValue({fetch: function () { return {then: function () {}}}});

			module(function ($provide) {
				$provide.value('globalDelayTracker', globalDelayTrackerMock);
				$provide.value('focusService', focusServiceMock);
				$provide.value('modalService', modalServiceMock);
				$provide.value('WeeklyGoalsService', goalsServiceMock);
				$provide.value('formatter', formatterMock);
			});

			inject(function ($controller, $rootScope, $modal, $q) {
				scope = $rootScope.$new();
				modal = $modal;
				q = $q;

				var deferred = q.defer();
				var saveDeferred = q.defer();
				var modalDeferred = q.defer();
				deferred.resolve({});
				modalDeferred.resolve({});
				saveDeferred.resolve({});

				goalsServiceFetchSpy = jasmine.createSpy('goalsServiceFetchSpy').and.returnValue(deferred.promise);
				goalsServiceSaveSpy = jasmine.createSpy('goalsServiceSaveSpy').and.returnValue(saveDeferred.promise);

				goalsServiceMock.instance.and.returnValue({
					fetch : goalsServiceFetchSpy,
					save : goalsServiceSaveSpy,
					promptedThisSession : hasBeenPrompted,
					dataPointers : {
						list : dataPointersMock
					},
					createEmpty : createEmptySpy
				});

				modalServiceMock.showModal.and.returnValue(modalDeferred.promise);

				controller = $controller('AuthenticatedUserController', {$scope: scope});
			});

		});

		xdescribe("when the page is done loading", function () {

			it("should focus on the main header", function () {
				expect(focusServiceMock.focusMain).toHaveBeenCalled();
			});

			it("should set details to first object in dataPointers list", function() {
				scope.$apply();
				expect(goalsServiceFetchSpy).toHaveBeenCalledWith({activeFlag : true});
				expect(scope.details).toEqual(regularGoal);
				dataPointersMock = [];
			});


			describe("setting of details", function () {
				it("should show text prompting the user to set goals", function () {
					scope.$apply();
					expect(scope.details).toEqual(createEmptyGoalReturn);
					expect(modalServiceMock.showModal).not.toHaveBeenCalled();
					hasBeenPrompted = false;
				});
				it("should then display a modal prompting to input goals or not", function () {
					var modalContent = {
						closeButtonText: 'No',
						actionButtonText: 'Yes',
						headerText: 'Weekly Activity Goals',
						bodyText: 'You currently have no weekly activity goals. Are you ready to set your weekly activity goals?'
					};

					scope.$apply();
					expect(scope.details).toEqual(createEmptyGoalReturn);
					// expect(goalsServiceMock.instance.promptedThisSession).toBe(true);
					expect(modalServiceMock.showModal).toHaveBeenCalledWith({}, modalContent);

				});
			});

			describe("When the user sets opens the set goal modal and enters values", function () {
				it("should save those new details", function () {
					var today = new Date();
					scope.details.startDate = "01/02/2015";
					scope.details.target = {
						minutes: 234,
						steps: 2000,
						miles: 10
					};
					scope.today = today;

					var editGoal = {
						startDate: "01/02/2015",
						minutes : 234,
						steps: 2000,
						miles : 10
					};

					var modalContent = {
						closeButtonText: 'Cancel',
						actionButtonText: 'Save',
						resetButtonText: 'Reset',
						headerText: 'Set Weekly Activity Goals',
						subHeaderText : 'Change activity goals or start a new weekly goal.',
						bodyContentUrl: 'src/modules/home/pages/authenticated-user/set-goals-modal_template.html',
						useEditForm: true,
						data: {goal: editGoal},
						metaData : {allowedDateRange: {min : "01/02/2015", max : 0}}
					};

					scope.openGoalsModal();
					scope.$apply();
					expect(modalServiceMock.showModal).toHaveBeenCalledWith({}, modalContent);
					expect(scope.details.activeFlag).toBe(true);
					expect(goalsServiceSaveSpy).toHaveBeenCalledWith(scope.details);

				});
			});
		});


	});
});