define(["angular", "app", "moment"], function (angular, app, moment) {
	"use strict";
	app.controller("FitnessActivityController", function ($scope, $state, $stateParams, focusService, $window,
		FitnessConstants, FitnessService, GoalGraphService, WeeklyGoalsService, globalDelayTracker, $timeout,
        GoalGraphAccessibilityService, FitnessActivityGraphService, FitnessRemindersService) {

        var goalService = WeeklyGoalsService.instance(),
			service = FitnessService.instance($state.current.data.sectionName);

		$scope.details = {};
		$scope.globalDelayTracker = globalDelayTracker;
		focusService.focusMain();
		$scope.today = new Date();
		$scope.sectionTitle = "Fitness Activity";
		$scope.isPhoneView = $window.innerWidth < 768;

        goalService.fetch({activeFlag: true}).then(function () {
            setDetails();
        });

		$scope.getDetails = function (item) {
			$stateParams.id = item ? service.idGetter(item) : null;
            $state.go(".entry", $stateParams);
            $stateParams.dateRange = $scope.filter.model;
		};

		$scope.openGoalsModal = function (focusOptionsButton) {
			FitnessService.openGoalsModal($scope.details).then(function (details) {
				goalService.save(details).then(function () {
				    setDetails(true);
				})
			}).finally(function () {
				if (focusOptionsButton) {
					focusService.focusElement(angular.element('span.dropdown button.dropdown-toggle'));
				} else {
					focusService.focusElement(angular.element('button[title*="Goals"]'));
				}
			});
		};

        var minDate = moment().subtract(6, 'year').format('L'),
            defaultMin = moment().subtract(3, 'month').format('L'),
            defaultMax = moment().format('L'),
            defaultFilters = {
                startDate: defaultMin,
                endDate: defaultMax
            };

        $scope.allowedDateRange = { min: minDate };
        $scope.filter = {
            service: service,
            defaults: defaultFilters,
            model: angular.copy(service.dataPointers.filters),
            callbacks : {
                filter : function() {
                    makeGraph();
                }
            }
        };

        var makeGraph = function() {
            $scope.activityGraphConfig = FitnessActivityGraphService.getGraphConfig($scope.filter);
        };

        service.fetch($stateParams.dateRange || $scope.filter.defaults).then(function() {
            makeGraph();
        });

        function setDetails(fetchNew) {
			if (goalService.dataPointers.list.length === 0) {
				$scope.details = goalService.createEmpty();
			} else {
				$scope.details = goalService.dataPointers.list[0];
				if (fetchNew) {
					goalService.fetch({activeFlag: true}).then(function () {
						setDetails();
					});
				}
			}

            $scope.statusMsg = $scope.details.endDate ? getStatusMsg($scope.details) : '';

            function getStatusMsg(details) {
                var msInDay = 1000 * 3600 * 24;
                var endDate = (new Date(details.endDate)).getTime();
                var d = new Date();
                var today = (new Date((d.getMonth()+1) + '/' + d.getDate() + '/' + d.getFullYear())).getTime();
                var daysRemain = Math.round((endDate - today) / msInDay) +1;

                return 'Status:  ' + details.startDate + ' - ' + details.endDate +
                    ' (' + daysRemain + ' days remaining)';
            }

            $scope.goalGraphConfig = GoalGraphService.getGraphConfig($scope.details);
            $scope.screenReaderText = GoalGraphAccessibilityService.getScreenReaderTexts($scope.details);
        }        

        $scope.notificationDetails = {
            notificationStatus: ''
        };

        FitnessRemindersService.init('fitness-activity');
        FitnessRemindersService.getReminder().then(function (response) {
            $scope.notificationDetails = angular.copy(response);
        });

        var trophyIconButton = {
            getLabel : function() {
                return "Weekly Activity Goals";
            },
			getPhoneLabel : function() {
				return "Goals";
			},
            getIcon : function() {
                return "icon-trophy";
            },
            onClick : function(isDropdownButton) {
                $scope.openGoalsModal(isDropdownButton);
            },
            getAriaLabel : function() {
                return "Trophy icon Weekly Activity Goals";
            },
			getPhoneAriaLabel : function() {
				return "Trophy icon Goals";
			}
        },
        tableGraphButton = {
            getLabel : function() {
                return $scope.isGraphView ? "Table" : "Graph";
            },
            getIcon : function() {
                return $scope.isGraphView ? "icon-list" : "icon-graph";
            },
            onClick : function(isDropdownButton) {
				if (isDropdownButton) {
					focusService.focusElement(angular.element('span.dropdown button.dropdown-toggle'));
				}
                $scope.isGraphView = !$scope.isGraphView;
                $scope.filter.noFocus = $scope.isGraphView;
            }
        },
        notificationButton = {
            getLabel: function() {
                return 'Notification (' + $scope.notificationDetails.notificationStatus + ')';
            },
            getIcon: function() {
                return 'icon-notification';
            },
            onClick: function() {
                $state.go('main.auth.fitness.notification', $stateParams);
            }
        };

        $scope.navButtons = {
			rightNoOptions : [trophyIconButton],
            right : [tableGraphButton, notificationButton]
        };

        var onViewChange = function () {
            focusService.focusPrimary();
        };
        $scope.$on('$viewContentLoaded', onViewChange);
        $scope.$on('$stateChangeSuccess', onViewChange);

		angular.extend($scope, FitnessConstants[$state.current.data.sectionName].summary);

		if (!navigator.userAgent.match(/(iPad|iPhone|iPod)/g)) {
			angular.element($window).bind('resize.fitnessGraphResize', function () {
				$timeout(function () {
					angular.forEach(Highcharts.charts, function (chart) {
						if (chart) {
							chart.setSize(chart.chartWidth, chart.chartHeight, false);
						}
					});
				});
			});

			$scope.$on("$destroy", function () {
				$(window).off("resize.fitnessGraphResize");
			});
		}
	});
});