define(['angular', 'app'], function (angular, app) {
    "use strict";
    app.service('HealthMoodRemindersService', function ($q, HealthMoodRemindersInstanceService, $state, $filter) {

        var HealthMoodRemindersService = {},
            remindersService,
            key,
            section;

        HealthMoodRemindersService.init = function(instanceKey) {
            remindersService = HealthMoodRemindersInstanceService.instance(instanceKey + "-reminders");
            key = instanceKey;
            section = $filter('filter')($state.current.data.paramSections, { paramValue: key }, true)[0].sectionName;
        };

        HealthMoodRemindersService.getReminder = function() {
            var defer = $q.defer();

            remindersService.fetch().then(function(response) {
                if(response.size === 0) {
                    defer.resolve(remindersService.createEmpty());
                } else {
                    defer.resolve(response.reminders[0]);
                }
            }, function(error) {
                defer.reject(error);
            });

            return defer.promise;
        };

        HealthMoodRemindersService.save = function(reminder) {
            reminder.title = "Update your " + section + " information now.";
            if (reminder.notificationStatus == "On") {
                reminder.healthTrackerId = key;
                reminder.auxData = createReminderAuxData();
            }

            var defer = $q.defer();

            remindersService.save(reminder).then(function() {
                defer.resolve();
            }, function(error){
                defer.reject(error);
            });

            return defer.promise;
        };

        function createReminderAuxData() {
            var subject = "Update your " + section + " information now.";

            var body = "<br/>This notice is provided to help you remember to update your " + section + " information.<br/><br/>";

            var updateMsg = "Update this information in the <appLink><br/><br/>" +
                "To modify or stop future notices, update the Notification settings for this Health feature inside the VA FitHeart App.";

            return [
                {key: "emailSubject", value: subject},
                {key: "emailBody", value: body},
                {key: "emailUpdateMessage", value: updateMsg},
                {key: "emailSignature", value: "VA FitHeart - Health"},
                {key: "source", value: "FIT_HEART"},
                {key: "appName", value: "VA FitHeart"},
                {key: "appUri", value: 'fit-heart'}
            ];
        }

        return HealthMoodRemindersService;
    });
});
