define(['angular', 'app'], function(angular, app) {
	app.constant('HealthMoodConstants', {
		glucose: {
			summary: {
				table : {
					columns : [
						{ label: 'Date Entered', keys: 'recorded' },
						{ label: 'Glucose Level (mg/dl)', keys: 'valueQuantity.value'},
						{ label: 'Tested', keys: 'method', class: {column: 'hidden-xs'} }
					]
				},
				testConditions: ['Unknown', 'Fasting (8 hours)', 'After Meal']
			},
			entry: {
				entryInputsTemplate : 'src/modules/health-mood/pages/entry/partials/_glucose.html',
				vitalObservationsMeta:[
					{
						label : 'Glucose Level',
						units : 'mg/dl',
						min : '0',
						max : '400',
						validationExpression : /(^0?[1-9][0-9]$)|(^[1-5][0-9]{2}$)|(^600$)/,
						userInputPattern : /(^[1-9][0-9]{0,2}$)|(^0$)/
					}
				],
				testConditions: ['Unknown', 'Fasting (8 hours)', 'After Meal']
			},
			name: 'Glucose'
		},
		'bp-pulse': {
			summary : {
				table : {
					columns : [
						{label: 'Systolic (mmHg)', keys: 'entry[0].valueQuantity.value'},
						{label: 'Diastolic (mmHg)', keys: 'entry[1].valueQuantity.value'},
						{label: 'Pulse (bpm)', keys: 'entry[2].valueQuantity.value'},
						{label: 'Position', keys: 'method', class: {column: 'hidden-xs'}},
						{label: 'Date Entered', keys: 'updated'}
					]
				}
			},
			entry : {
				entryInputsTemplate : 'src/modules/health-mood/pages/entry/partials/_bp-pulse.html',
				method : {
					label : 'Position',
					options : [ 'Sitting', 'Standing', 'Lying' ]
				},
				vitalObservationsMeta:[
					{
						label : 'Systolic',
						units : 'mmHg',
						min : '80',
						max : '200',
						validationExpression : /(^0?[89][0-9]$)|(^[12][0-9]{2}$)|(^300$)/,
						userInputPattern : /(^[1-9][0-9]{0,2}$)|(^0$)/
					},
					{
						label : 'Diastolic',
						units : 'mmHg',
						min : '50',
						max : '150',
						validationExpression : /(^0?[89][0-9]$)|(^[12][0-9]{2}$)/,
						userInputPattern : /(^[1-9][0-9]{0,2}$)|(^0$)/
					},
		         	{
						label : 'Pulse',
						units : 'bpm',
						min : '40',
						max : '250',
						validationExpression : /(^0?[4-9][0-9]$)|(^1[0-9]{2}$)|(^2[0-4][0-9]$)|(^250$)/,
						userInputPattern : /(^[1-9][0-9]{0,2}$)|(^0$)/
					}
				]
			},
            name: 'Blood Pressure & Pulse'
		},
		weight : {
			summary : {
				table : {
					columns : [
						{label: 'Date Entered', keys: 'recorded'},
						{label: 'Weight (lbs)', keys: 'valueQuantity.value'}
					]
				},
                filterTemplate: ''
			},
			entry : {
				entryInputsTemplate : 'src/modules/health-mood/pages/entry/partials/_weight.html',
		       	vitalObservationsMeta:[
		         	{
			            label : 'Weight',
			            units : 'lbs',
			            min : '0.0',
			            max : '999.9',
						validationExpression : /^[0-9]{0,3}([.][0-9])?$/,
						userInputPattern : /^[1-9][0-9]{0,2}([.][0-9]?)?$|(^0([.][0-9]?)?$)/
		        	}
		       	]
			},
            name: 'Weight'
		},
        "cholesterol" : {
            summary : {
                table : {
                    columns : [
                        {label: 'Date Entered', keys: 'inputEntryDate'},
                        {label: 'Total (mg/dl)', keys: 'entry[0].valueQuantity.value'},
                        {label: 'HDL (mg/dl)', keys: 'entry[2].valueQuantity.value', class: {column: 'hidden-xs'}},
                        {label: 'LDL (mg/dl)', keys: 'entry[1].valueQuantity.value', class: {column: 'hidden-xs'}},
                        {label: 'Triglycerides (mg/dl)', keys: 'entry[3].valueQuantity.value', class: {column: 'hidden-xs'}},
                        {label: 'Fasting?', keys: 'method', class: {column: 'hidden-xs'} }
                    ]
                },
                defaultFilter: {
                    name: 'Lipids'
                },
                filterTemplate: ''
            },
            entry : {
                entryInputsTemplate : 'src/modules/health-mood/pages/entry/partials/_lipids.html',
                vitalObservationsMeta:[
                    {
                        label : 'Cholesterol Level',
                        screenLabel: 'Total Serum Cholesterol',
                        units : 'mg/dl',
                        min : '0',
                        max : '600',
                        validationExpression : /(^0?[1-9][0-9]$)|(^[1-5][0-9]{2}$)|(^600$)/,
						userInputPattern : /(^[1-9][0-9]{0,2}$)|(^0$)/
                    },
                    {
                        label : 'LDL',
                        screenLabel: 'Low-Density Lipoprotein (LDL)',
                        units : 'mg/dl',
                        min : '0',
                        max : '300',
                        validationExpression : /(^0?[1-9][0-9]$)|(^[1-2][0-9]{2}$)|(^300$)/,
						userInputPattern : /(^[1-9][0-9]{0,2}$)|(^0$)/
                    },
                    {
                        label : 'HDL',
                        screenLabel: 'High-Density Lipoprotein (HDL)',
                        units : 'mg/dl',
                        min : '0',
                        max : '300',
                        validationExpression : /(^0?[1-9][0-9]$)|(^[1-2][0-9]{2}$)|(^300$)/,
						userInputPattern : /(^[1-9][0-9]{0,2}$)|(^0$)/
                    },
                    {
                        label : 'Triglycerides',
                        screenLabel: 'Triglycerides',
                        units : 'mg/dl',
                        min : '0',
                        max : '600',
                        validationExpression : /(^0?[1-9][0-9]$)|(^[1-5][0-9]{2}$)|(^600$)/,
						userInputPattern : /(^[1-9][0-9]{0,2}$)|(^0$)/
                    }
                ],
                fastingOptions: ['Yes', 'No', 'Unknown'],
                noTime: true
            },
            common: {},
            name: 'Cholesterol'

        },
		mood: {
			summary: {
				table : {
					columns : [{
							label: 'Mood Rating (1=Worst, 10=Best)',
							keys: 'valueQuantity.value',
							cellTemplate: "src/ui-components/table/partials/_icon.html",
							ngIconClass: function (item) { return { 
                                'icon-mood-verybad': item.valueQuantity.value < 3,
								'icon-mood-verygood': item.valueQuantity.value > 8,
								'icon-mood-neutral': (item.valueQuantity.value == 5 || item.valueQuantity.value == 6),
								'icon-mood-bad': (item.valueQuantity.value == 3 || item.valueQuantity.value == 4),
								'icon-mood-good': (item.valueQuantity.value == 7 || item.valueQuantity.value == 8) }
							}
						}, {label: 'Date Entered', keys: 'recorded' }
					]
				}
			},
			entry: {
				entryInputsTemplate : 'src/modules/health-mood/pages/entry/partials/_mood.html',
                vitalObservationsMeta:[
                    {
                        label : 'Mood Ratings',
                        units : '1 = Worst, 10 = Best',
                        min : '0',
                        max : '10',
                        validationExpression : /(^0?[1-9][0-9]$)|(^[1-3][0-9]{2}$)|(^10$)/,
                        userInputPattern : /(^[0-9]{0,2}$)/
                    }
                ]
			},
			name: 'Mood'
		}
	});
});