define(['angular', 'app', 'lodash', 'moment'], function (angular, app, lodash, moment) {
	'use strict';

	app.controller('HealthMoodEntryController', function ($scope, $state, $stateParams, $filter, HealthMoodConstants, HealthMoodService, HealthMoodRemindersService, modalService, focusService) {
		$scope.section = $stateParams.section;
		angular.extend($scope, HealthMoodConstants[$stateParams.section].entry);

		var service = HealthMoodService.instance($stateParams.section);

		$scope.notificationDetails = {
			notificationStatus: ''
		};

		HealthMoodRemindersService.init($stateParams.section);
		HealthMoodRemindersService.getReminder().then(function (response) {
			$scope.notificationDetails = angular.copy(response);
		});

		$scope.entryForm = {
			service: service,
			callbacks: {
				common: function () {
					$state.go($state.current.data.backState, $stateParams);
				}
			},
			allowedActions: {
				save: true,
				delete: !!$stateParams.id,
				cancel: true
			}
		};

		if($stateParams.section === "mood") {
			$scope.errorHandling = {
				'requiredMood': {
					message: "Mood field is required.",
					priority: 1
				}
			};

			$scope.$watch("[details.valueQuantity.value, form.specialErrors]", function () {
				if (angular.isDefined($scope.form) && angular.isDefined($scope.form.specialErrors)) {
					$scope.form.specialErrors.$setValidity("requiredMood", $scope.details.valueQuantity.value);
				}
			}, true);

			$scope.openMoodModal = function () {
				var modalContent = {
					closeButtonText: 'Cancel',
					actionButtonText: 'OK',
					bodyContentUrl: 'src/modules/health-mood/pages/entry/partials/mood-modal_template.html',
					headerText: 'Select Mood',
					data: {value: $scope.details.valueQuantity.value || 5}
				};

				modalService.showModal({}, modalContent).then(function () {
					$scope.details.valueQuantity.value = modalContent.data.value;
					$scope.form.$setDirty();
				}).finally(function () {
					focusService.focusElement('button[ng-click="openMoodModal()"]');
				});
			};
		}

		if ($stateParams.section === "cholesterol") {
			$scope.hidden = true;
			$scope.errorHandling = {
			    'Invalid-Date' : {
			        message : "An entry has already been saved with the same date. Please change your date and save again.",
			        priority : 1
			    },
			    'Lab-Result-Required' : {
			        message : "Provide at least one Lab Result value to save your entry.",
			        priority : 1
			    }
			};

			$scope.$watch("[details.entry, details.inputEntryDate, form]", function(newVal){
				if (angular.isDefined($scope.form) && angular.isDefined($scope.form.specialErrors)) {

					if($scope.details.inputEntryDate && $scope.details.inputEntryDate.length === 10 && $scope.form.specialErrors) {
					    var result = true;
					    for(var i = 0; i < service.dataPointers.unfilteredList.length; i++) {
					        if((moment(service.dataPointers.unfilteredList[i].updated, "MM/DD/YYYY").diff(moment($scope.details.inputEntryDate, "MM/DD/YYYY")) === 0) &&
					           (service.dataPointers.unfilteredList[i]._id !== $scope.details._id)){
					            result = false;
					            break;
					        }
					    } 
					    $scope.form.specialErrors.$setValidity("Invalid-Date", result);
					}		 			

					var validValue = false;
					if (newVal[0][0].valueQuantity.value || newVal[0][1].valueQuantity.value || newVal[0][2].valueQuantity.value || newVal[0][3].valueQuantity.value) {
						validValue = true;
					}

					$scope.form.specialErrors.$setValidity("Lab-Result-Required", validValue);

				}
			}, true);

		} else if ($stateParams.section === "bp-pulse") {
            $scope.errorHandling = {
                'Invalid-BP' : {
                    message : "The Systolic (mmHg) value must be greater than the Diastolic (mmHg) value. Please check the values and re-enter.",
                    priority : 1
                },
				'Lab-Result-Required' : {
					message : "Provide either both blood pressure values or a pulse value to save.",
					priority : 1
				},
				'BP-Required' : {
					message : "Both Systolic and Diastolic values are required to save a blood pressure reading.",
					priority : 1
				}
            };

            $scope.$watch("[details.entry[0].valueQuantity.value, details.entry[1].valueQuantity.value, details.entry[2].valueQuantity.value, form]", function(curVal){
				if ($scope.form && $scope.form.specialErrors) {
					var systolic = curVal[0] || 0,
						diastolic = curVal[1] || 0,
						pulse = curVal[2] || 0,
						labHasValue = systolic || diastolic || pulse,
						pulseIsOnlyValue = (!systolic && !diastolic && pulse);

					$scope.form.specialErrors.$setValidity("BP-Required", !labHasValue || (pulseIsOnlyValue || (systolic && diastolic)));
					$scope.form.specialErrors.$setValidity("Lab-Result-Required", labHasValue);
					$scope.form.specialErrors.$setValidity("Invalid-BP", !systolic || !diastolic || parseInt(systolic) > parseInt(diastolic));
				}
            }, true);
        }
		
		$scope.allowedDateRange = { min: moment().subtract(6, 'year').format('L') };
		$scope.sectionTitle = $filter('filter')($state.current.data.paramSections, { paramValue: $stateParams.section }, true)[0].sectionName;
		$scope.details = $stateParams.id ? angular.copy(service.localGetById($stateParams.id)) : service.createEmpty();
	});
});