define(['angular', 'app', 'moment'], function (angular, app, moment) {
	'use strict';

	app.controller('HealthMoodSummaryController', function ($scope, $filter, $state, $stateParams, HealthMoodService, HealthMoodConstants, HealthMoodGraphService, HealthMoodRemindersService, focusService) {

		var service = $stateParams.section.service || HealthMoodService.instance($stateParams.section);
		$scope.getDetails = function (item) {
			$stateParams.id = item ? service.idGetter(item) : null;
			$state.go('^.entry', $stateParams);
		};

		$scope.notificationDetails = {
			notificationStatus: ''
		};

		HealthMoodRemindersService.init($stateParams.section);
		HealthMoodRemindersService.getReminder().then(function (response) {
			$scope.notificationDetails = angular.copy(response);
		});

		$scope.currentSection = $stateParams.section;

		var defaultMin = moment().subtract(3, 'months').format('L'),
			defaultMax = moment().format('L'),
			defaultFilters = {
				startDate: defaultMin,
				endDate: defaultMax
			};
		
		$scope.allowedDateRange = { min: moment().subtract(6, 'year').format('L') };

		$scope.filter = {
			service: service,
			defaults: defaultFilters,
			model: defaultFilters,
			callbacks: {
				filter: function (items) {
					if ($scope.currentSection === 'glucose' && $scope.filter.model.tested) {
						service.dataPointers.list = $filter('filter')(items, function (item) {
							return item.method === $scope.filter.model.tested;
						});
					}
                    prepareData();
					if($scope.filter.noFocus) {
						focusService.focusElement(service.dataPointers.list.length === 0 ? ".no-results-message" : ".reference-table-message", true);
					}
				}
			}
		};
        
        var prepareData = function() {
            sortBundle();
            makeGraph();
        };

        var sortBundle = function() {
            if(($stateParams.section !== "cholesterol" &&
                $stateParams.section !== "bp-pulse") || !service.dataPointers.list) {return;}

            var temp = $stateParams.section === "cholesterol" ? [{}, {}, {}, {}] : [{}, {}, {}];

            for(var i = 0; i < service.dataPointers.list.length; i++) {
                for(var k = 0; k < service.dataPointers.list[i].entry.length; k++) {
                    switch (service.dataPointers.list[i].entry[k].name) {
                        case "Total Serum Cholesterol":
                            temp[0] = angular.copy(service.dataPointers.list[i].entry[k]);
                            break;
                        case "LDL":
                            temp[1] = angular.copy(service.dataPointers.list[i].entry[k]);
                            break;
                        case "HDL":
                            temp[2] = angular.copy(service.dataPointers.list[i].entry[k]);
                            break;
                        case "Triglycerides":
                            temp[3] = angular.copy(service.dataPointers.list[i].entry[k]);
                            break;
                        case "8480-6":
                            temp[0] = angular.copy(service.dataPointers.list[i].entry[k]);
                            break;
                        case "8462-4":
                            temp[1] = angular.copy(service.dataPointers.list[i].entry[k]);
                            break;
                        case "8867-4":
                            temp[2] = angular.copy(service.dataPointers.list[i].entry[k]);
                            break;
                    }
                }
                service.dataPointers.list[i].entry = angular.copy(temp);
            }
        };

		var makeGraph = function() {
			var graphMetaData = HealthMoodConstants[$stateParams.section].entry.vitalObservationsMeta;
			if (graphMetaData) {
				$scope.graphConfig = HealthMoodGraphService.getGraphConfig($stateParams.section,
                    service.dataPointers.filters || $scope.filter.model || $scope.filter.defaults,
                    service.dataPointers.list, graphMetaData, $scope.sectionTitle);
			}
		};

		var navGraphButton = {
			getLabel : function() {
				return $scope.summaryType === 'graph' ? 'Table' : 'Graph';
			},
			getIcon : function() {
				return $scope.summaryType === 'graph' ? 'icon-list' : 'icon-graph';
			},
			onClick : function(isDropdownButton) {
				if (isDropdownButton) {
					focusService.focusElement(angular.element('span.dropdown button.dropdown-toggle'));
				}
				$scope.summaryType = $scope.summaryType === 'graph' ? 'table' : 'graph';
				$scope.filter.noFocus = $scope.summaryType === 'graph';
			}
		};

		var notificationButton = {
			getLabel: function() {
				return 'Notification (' + $scope.notificationDetails.notificationStatus + ')';
			},
			getIcon: function() {
				return 'icon-notification';
			},
			onClick: function() {
				$state.go('^.notification', $stateParams);
			}
		};

		$scope.navButtons = {
			right: [navGraphButton, notificationButton]
		};

		$scope.summaryType = 'table';
		$scope.sectionTitle = $state.current.data.sectionName || $filter('filter')($state.current.data.paramSections, { paramValue: $stateParams.section }, true)[0].sectionName;
		angular.extend($scope, HealthMoodConstants[$stateParams.section].summary);

		if (!service.dataPointers.list) {
			service.fetch($scope.filter.defaults).then(function () {
                prepareData();
			});
		}
        else {
            prepareData();
        }
	});
});