define(['angular', 'app'], function (angular, app) {
	'use strict';

	app.controller('InitialSettingsController', function ($scope, $state, $q, patient, NotificationsPreferencesService, NotificationsConstants, FitnessRemindersService, focusService, formatter) {
		$scope.errors = [];

		(function init(){
			angular.extend($scope, NotificationsConstants['notification'].scope);
			$scope.preferences = {
				optInSelected: "Yes",
				emailAddress: null,
				timeZone: null
			};

			focusService.focusMain();

			patient.fetch().then(function (result) {
				$scope.patientData = result;
			});

			NotificationsPreferencesService.getPreferencesData().then(function (response) {
				$scope.preferences.timeZone = response.timeZone;
				$scope.isTimeZoneStored = !!response.timeZone;
			}, function () {
				$scope.isTimeZoneStored = false;
			});

			FitnessRemindersService.init('fitness-activity');			
		})();

		$scope.save = function () {
			this.initialForm.validationSummary.validate().then(function() {
				var promise1 = NotificationsPreferencesService.createNotificationsPreference($scope.preferences.optInSelected === 'Yes'),
					promise2 = NotificationsPreferencesService.createPreferencesData($scope.preferences.emailAddress, $scope.preferences.timeZone);

				$q.all([promise1, promise2]).then(function(){
					FitnessRemindersService.save(buildFitnessReminder()).then(function() {
						$state.go('main.auth.authenticated-user')
					});
				});
			});
        };

        $scope.showUserGuide = function () {
			window.open("src/ui-components/modals/help/guide/guide.html#/features/notifications", "_blank", "location=no,scrollbars=1,resizable=1");
		};

		function buildFitnessReminder() {
			return {
                source: 'VA FitHeart - Fitness Activity',
                eventDate: formatter.getFormattedFrontendDate(new Date()),
                frequency: 'Daily',
                notificationStatus: 'On',
                notificationTypes: $scope.preferences.emailAddress ? ['In-App', 'Email'] : ['In-App'],
                times: [{value: '09:00 AM'}]
            };			
		}
	});
});