define(['InitialSettingsController'], function() {
    'use strict';

    describe("The Initial Settings controller", function () {
        var controller,
            scope,
			windowMock = window,
			patientServiceMock,
            focusServiceMock,
			fitnessRemindersServiceMock,
			notificationsPreferencesServiceMock,
			state,
            q;

        beforeEach(function () {
            module('angularTemplateApp');

			patientServiceMock = jasmine.createSpyObj('patient', ['fetch']);
            focusServiceMock = jasmine.createSpyObj('focusService', ['focusMain']);
            notificationsPreferencesServiceMock = jasmine.createSpyObj('NotificationsPreferencesService', ['getPreferencesData', 'createNotificationsPreference', 'createPreferencesData']);
			fitnessRemindersServiceMock = jasmine.createSpyObj('FitnessRemindersService', ['init', 'save']);

			patientServiceMock.fetch.and.returnValue({then: function () {} });
			notificationsPreferencesServiceMock.getPreferencesData.and.returnValue({then: function () {}});

            module(function ($provide) {
                $provide.value('NotificationsPreferencesService', notificationsPreferencesServiceMock);
				$provide.value('patient', patientServiceMock);
                $provide.value('focusService', focusServiceMock);
				$provide.value('FitnessRemindersService', fitnessRemindersServiceMock);
            });

            inject(function($controller, $rootScope, $state, $q) {
                scope = $rootScope.$new();
				state = $state;
				q = $q;

				var defer = q.defer();
				defer.resolve('resolvedPatient');
				patientServiceMock.fetch.and.returnValue(defer.promise);

				var deferPreference = q.defer();
				deferPreference.resolve({timeZone: '(-05:00) America/New_York (Eastern)'});
				notificationsPreferencesServiceMock.getPreferencesData.and.returnValue(deferPreference.promise);
                notificationsPreferencesServiceMock.createNotificationsPreference.and.returnValue(deferPreference.promise);
                notificationsPreferencesServiceMock.createPreferencesData.and.returnValue(deferPreference.promise);
				fitnessRemindersServiceMock.save.and.returnValue(deferPreference.promise);

                controller = $controller('InitialSettingsController', {$scope: scope});
            });
        });

        it ( "should have scope values initialized", function () {
            expect(scope.errors).toEqual([]);
            expect(scope.timeZoneOptions).toEqual([ '(+00:00) WET (Western European Time)', '(+01:00) CET (Central European Time)', '(+02:00) EET (Eastern European Time)', '(+04:00) Europe/Moscow', '(+04:00) Asia/Dubai', '(+04:30) Asia/Kabul', '(+05:00) Asia/Aqtau', '(+05:00) Asia/Aqtobe', '(+05:45) Asia/Kathmandu', '(+06:00) Asia/Almaty', '(+06:30) Asia/Rangoon', '(+07:00) Asia/Bangkok', '(+07:00) Asia/Jakarta', '(+08:00) Asia/Singapore', '(+09:00) Asia/Jayapura', '(+09:00) Asia/Tokyo', '(+11:00) Asia/Sakhalin', '(+10:00) Australia/Sydney', '(+12:00) Asia/Anadyr', '(+12:00) Antarctica/McMurdo', '(+12:00) Pacific/Fiji', '(+13:00) Pacific/Enderbury', '(+14:00) Pacific/Kiritimati', '(-01:00) Atlantic/Cape_Verde', '(-01:00) America/Scoresbysund', '(-03:00) America/Argentina/San_Juan', '(-04:00) America/Argentina/San_Luis', '(-05:00) America/New_York (Eastern)', '(-06:00) America/Chicago (Central)', '(-07:00) America/Denver (Mountain)', '(-08:00) America/Los_Angeles (Pacific)', '(-09:00) America/Juneau', '(-10:00) Pacific/Honolulu', '(-11:00) Pacific/Midway', '(+00:00) Etc/GMT+12' ]);
			expect(scope.optInOptions).toEqual(["Yes", "No"]);
			expect(scope.preferences).toEqual({ optInSelected: "Yes", emailAddress : null, timeZone: null });
        });

        it ("should focus the header", function () {
			expect(focusServiceMock.focusMain).toHaveBeenCalled();
		});

		it ("should request patient data for username", function () {
			expect(patientServiceMock.fetch).toHaveBeenCalled();
		});

		it ("should request the current time zone preference", function() {
			expect(notificationsPreferencesServiceMock.getPreferencesData).toHaveBeenCalled();
		});

		describe("when patient fetch is called", function () {
			beforeEach(function () {
				scope.$digest();
			});

			it ("should set the resolved patient data", function() {
				expect(scope.patientData).toEqual('resolvedPatient');
			});
		});

		describe("when notification preferences is called", function () {
			beforeEach(function () {
				scope.$digest();
			});

			it ("should set the preferences data", function() {
				expect(scope.preferences.timeZone).toEqual('(-05:00) America/New_York (Eastern)');
				expect(scope.isTimeZoneStored).toBe(true);
			});
		});

        describe("when save is clicked and opting out", function () {
			beforeEach(function () {
				spyOn(state, 'go');
				scope.preferences.optInSelected = "No";
				var defer = q.defer();
				defer.resolve();
				scope.initialForm = {
					validationSummary: {
						validate: function () { return defer.promise; }
					}
				};

				scope.save();
				scope.$digest();
			});

			it ("should request to create or update a time zone preference", function() {
				expect(notificationsPreferencesServiceMock.createPreferencesData).toHaveBeenCalled();
			});

			it("should send a request to opt the user out", function () {
				expect(notificationsPreferencesServiceMock.createNotificationsPreference).toHaveBeenCalledWith(false);
			});

			it("should send the user to the main landing page", function () {
				expect(state.go).toHaveBeenCalledWith('main.auth.authenticated-user');
			});
		});

		describe("when save is clicked and opting in without selecting a time zone", function () {
			beforeEach(function () {
				spyOn(state, 'go');
				scope.preferences.optInSelected = "Yes";
				scope.preferences.timeZone = null;
				var defer = q.defer();
				defer.reject();
				scope.initialForm = {
					validationSummary: {
						validate: function () { return defer.promise; }
					}
				};
				scope.save();
				scope.$digest();
			});

			it ("should not request to create or update a time zone preference", function() {
				expect(notificationsPreferencesServiceMock.createPreferencesData).not.toHaveBeenCalled();
			});

			it("should not send a request to opt the user out", function () {
				expect(notificationsPreferencesServiceMock.createNotificationsPreference).not.toHaveBeenCalled();
			});

			it("should not send the user to the main landing page", function () {
				expect(state.go).not.toHaveBeenCalled();
			});
		});

		describe("when save is clicked and opting in and selecting a time zone (create)", function () {
			beforeEach(function () {
				spyOn(state, 'go');
				scope.preferences.optInSelected = "Yes";
				scope.preferences.timeZone = "(-05:00) America/New_York (Eastern)";
				var defer = q.defer();
				defer.resolve();
				scope.initialForm = {
					validationSummary: {
						validate: function () { return defer.promise; }
					}
				};
				scope.save();
				scope.$digest();
			});

			it ("should create a time zone preference", function() {
				expect(notificationsPreferencesServiceMock.createPreferencesData).toHaveBeenCalledWith(null, '(-05:00) America/New_York (Eastern)');
			});

			it("should send a request to opt the user out", function () {
				expect(notificationsPreferencesServiceMock.createNotificationsPreference).toHaveBeenCalledWith(true);
			});

			it("should send the user to the main landing page", function () {
				expect(state.go).toHaveBeenCalledWith('main.auth.authenticated-user');
			});
		});

		describe("when save is clicked and opting in and selecting a time zone (update)", function () {
			beforeEach(function () {
				spyOn(state, 'go');
				scope.preferences.optInSelected = "Yes";
				scope.preferences.timeZone = "(-05:00) America/New_York (Eastern)";
				scope.isTimeZoneStored = true;
				var defer = q.defer();
				defer.resolve();
				scope.initialForm = {
					validationSummary: {
						validate: function () { return defer.promise; }
					}
				};
				scope.save();
				scope.$digest();
			});

			it ("should create a time zone preference", function() {
				expect(notificationsPreferencesServiceMock.createPreferencesData).toHaveBeenCalledWith(null, '(-05:00) America/New_York (Eastern)');
			});

			it("should send a request to opt the user out", function () {
				expect(notificationsPreferencesServiceMock.createNotificationsPreference).toHaveBeenCalledWith(true);
			});

			it("should send the user to the main landing page", function () {
				expect(state.go).toHaveBeenCalledWith('main.auth.authenticated-user');
			});
		});

		describe("when Learn more link is clicked", function (){
			beforeEach( function () {
				spyOn(windowMock, 'open');
				scope.showUserGuide();
			});

			it ("should open a new browser window to the notification section of the user guide", function() {
				expect(windowMock.open).toHaveBeenCalledWith("src/ui-components/modals/help/guide/guide.html#/features/notifications", "_blank", "location=no,scrollbars=1,resizable=1");
			});
		});
    });

	describe("The Initial Settings controller - no stored time zone", function () {
		var controller,
			scope,
			patientServiceMock,
			focusServiceMock,
			notificationsPreferencesServiceMock,
			fitnessRemindersServiceMock,
			state,
			q;

		beforeEach(function () {
			module('angularTemplateApp');

			patientServiceMock = jasmine.createSpyObj('patient', ['fetch']);
			focusServiceMock = jasmine.createSpyObj('focusService', ['focusMain']);
			notificationsPreferencesServiceMock = jasmine.createSpyObj('NotificationsPreferencesService', ['getPreferencesData', 'createNotificationsPreference', 'createPreferencesData']);
			fitnessRemindersServiceMock = jasmine.createSpyObj('FitnessRemindersService', ['init', 'save']);

			patientServiceMock.fetch.and.returnValue({then: function () {} });
			notificationsPreferencesServiceMock.getPreferencesData.and.returnValue({then: function () {}});

			module(function ($provide) {
				$provide.value('NotificationsPreferencesService', notificationsPreferencesServiceMock);
				$provide.value('patient', patientServiceMock);
				$provide.value('focusService', focusServiceMock);
				$provide.value('FitnessRemindersService', fitnessRemindersServiceMock);
			});

			inject(function($controller, $rootScope, $state, $q) {
				scope = $rootScope.$new();
				state = $state;
				q = $q;

				var deferPreference = q.defer();
				deferPreference.reject('rejectedPromise');
				notificationsPreferencesServiceMock.getPreferencesData.and.returnValue(deferPreference.promise);

				controller = $controller('InitialSettingsController', {$scope: scope});
			});
		});

		describe("when notification preferences is called", function () {
			beforeEach(function () {
				scope.$digest();
			});

			it ("should set the preferences data", function() {
				expect(scope.isTimeZoneStored).toBe(false);
			});
		});
	});
});