define(['AuthenticatedUserController', 'HomePageConstants'], function () {
	'use strict';

	describe("The Authenticated User Controller", function () {
		var controller,
			scope,
			globalDelayTrackerMock,
			focusServiceMock,
			fitnessServiceMock,
			modalServiceMock,
			goalsServiceMock,
			goalGraphServiceMock,
			formatterMock,
			goalsServiceFetchSpy,
			createEmptySpy,
			initialSettingsMock,
			goalsServiceSaveSpy,
			hasBeenPrompted = true,
			modal,
			q,
			createEmptyGoalReturn = {
				activeFlag : false,
				startDate : null,
				target : {
					minutes: null,
					steps : null,
					miles : null
				}
			},
			regularGoal = {
				startDate : "03/30/2015",
				activeFlag : true,
				target : {
					minutes : 20,
					steps : 2000,
					miles : 2
				}
			},
			dataPointersMock = [regularGoal],
            goalGraphAccessibleMock;

		beforeEach(function () {
			module('angularTemplateApp');

			globalDelayTrackerMock = jasmine.createSpyObj('globalDelayTracker', ['active']);
			focusServiceMock = jasmine.createSpyObj('focusService', ['focusMain']);
			modalServiceMock = jasmine.createSpyObj('modalService', ['showModal']);
			goalsServiceMock = jasmine.createSpyObj('WeeklyGoalsService', ['instance']);
			goalGraphServiceMock = jasmine.createSpyObj('GoalGraphService', ['getGraphConfig']);
            goalGraphAccessibleMock = jasmine.createSpyObj('GoalGraphAccessibilityService', ['getScreenReaderTexts']);
			initialSettingsMock = jasmine.createSpyObj('NotificationsPreferencesService', ['getNotificationsPreferenceAndCount']);
            fitnessServiceMock = jasmine.createSpyObj('FitnessService', ['instance']);

			formatterMock = jasmine.createSpyObj('formatterMock', ['getFormattedFrontendDate', 'getDateTimeMinusDelta']);
			createEmptySpy = jasmine.createSpy('createEmptySpy').and.returnValue(createEmptyGoalReturn);

			formatterMock.getFormattedFrontendDate.and.returnValue(0);
			formatterMock.getDateTimeMinusDelta.and.returnValue(1);

			module(function ($provide) {
				$provide.value('globalDelayTracker', globalDelayTrackerMock);
				$provide.value('focusService', focusServiceMock);
				$provide.value('modalService', modalServiceMock);
				$provide.value('WeeklyGoalsService', goalsServiceMock);
				$provide.value('formatter', formatterMock);
				$provide.value('FitnessService', fitnessServiceMock);
				$provide.value('GoalGraphService', goalGraphServiceMock);
                $provide.value('GoalGraphAccessibilityService', goalGraphAccessibleMock);
				$provide.value('NotificationsPreferencesService', initialSettingsMock);
            });

			inject(function ($controller, $rootScope, $modal, $q) {
				scope = $rootScope.$new();
				modal = $modal;
				q = $q;

				var deferred = q.defer();
				var saveDeferred = q.defer();
				var modalDeferred = q.defer();
				deferred.resolve({});
				modalDeferred.resolve({});
				saveDeferred.resolve({});

				goalsServiceFetchSpy = jasmine.createSpy('goalsServiceFetchSpy').and.returnValue(deferred.promise);
				goalsServiceSaveSpy = jasmine.createSpy('goalsServiceSaveSpy').and.returnValue(saveDeferred.promise);

				goalsServiceMock.instance.and.returnValue({
					fetch : goalsServiceFetchSpy,
					save : goalsServiceSaveSpy,
					promptedThisSession : hasBeenPrompted,
					dataPointers : {
						list : dataPointersMock
					},
					createEmpty : createEmptySpy
				});

				modalServiceMock.showModal.and.returnValue(modalDeferred.promise);

				controller = $controller('AuthenticatedUserController', {$scope: scope});
			});
		});

		describe("when the page is done loading", function () {

			it("should focus on the main header", function () {
				expect(focusServiceMock.focusMain).toHaveBeenCalled();
			});

			it("should set details to first object in dataPointers list", function() {
				scope.$apply();
				expect(goalsServiceFetchSpy).toHaveBeenCalledWith({activeFlag : true});
				expect(scope.details).toEqual(regularGoal);
				dataPointersMock = [];
			});
		});
	});
});