define(['NotificationsService', 'NotificationsInstanceBuilderService', 'localResourceDirectoryService', 'CollectionCRUDService'], function() {
	'use strict';

	describe("The Notifications Service", function (){
		var service,
			focusServiceMock = jasmine.createSpy('focusService'),
			notificationsPreferencesServiceMock = jasmine.createSpy('NotificationsPreferencesService');

		beforeEach(function() {
			module('angularTemplateApp');

			module(function($provide){
				$provide.value('focusService', focusServiceMock);
				$provide.value('NotificationsPreferencesService', notificationsPreferencesServiceMock);
			});

			inject(function(NotificationsInstanceBuilderService) {
				service = NotificationsInstanceBuilderService.instance('all-notifications');
			});
		});

		describe("when instantiated", function() {
			it("service should be defined correctly", function() {
				expect(service.config).toBeDefined();
				expect(service.config.dependency.service).toBeDefined();
				expect(service.config.collection.filter).toBeDefined();
				expect(service.config.collection.sortComparator).toBeDefined();
				expect(service.config.object.uidKey).toBeDefined();
				expect(service.config.object.responseTransform).toBeDefined();
				expect(service.config.object.requestTransform).toBeDefined();
			});
		});

		describe("methods", function() {
			it("should transform the date in the response", function() {
				var item = {
					createDate: null,
					body: "test"
				};
				expect(service.config.object.responseTransform(item)).toEqual(item);

				item.createDate = "12/12/2015 01:01:01";
				var transformedResponse = service.config.object.responseTransform(item);
				expect(transformedResponse.createDateOnly).toEqual("12/12/2015");
				expect(transformedResponse.createTime).toEqual("01:01 AM");
				expect(transformedResponse.createDate).toEqual("12/12/2015 01:01 AM");
			});

			it("should transform the date in the request", function() {
				var item = {
					createDate: null
				};
				expect(service.config.object.requestTransform(item)).toEqual(item);

				item.createDate = '12/12/2015 01:01:00';
				expect(service.config.object.requestTransform(item)).toEqual({
					createDate: "12/12/2015 01:01:00"
				});
			});

			it("should provide a filter", function() {
				var filters = {status: 'all'},
					listItem1 = { readFlag: true },
					listItem2 = { readFlag: false },
					listItem3 = { readFlag: true },
					list = [listItem1, listItem2, listItem3];
				expect(service.config.collection.filter(list, filters)).toEqual([listItem1, listItem2, listItem3]);

				filters = { status: "unread" };
				expect(service.config.collection.filter(list, filters)).toEqual([listItem2]);

				filters = { status: "read" };
				expect(service.config.collection.filter(list, filters)).toEqual([listItem1, listItem3]);
			});
		});
	});
});