define(['NotificationsDetailsController'], function() {
	'use strict';

	describe("The Notifications Details Controller", function (){
		var controller,
			scope,
			stateParams,
			state,
			serviceMock,
			localGetByIdSpy,
			section = "all-notifications",
			itemId = "item_id",
			localGetByIdReturn = { id: itemId },
			allowedActionsMock = {
				save : false,
				delete : true,
				cancel : true
			};

		beforeEach(function() {
			module('angularTemplateApp');

			serviceMock = jasmine.createSpyObj('NotificationsInstanceBuilderService', ['instance']);
			localGetByIdSpy = jasmine.createSpy("localGetByIdSpy").and.returnValue(localGetByIdReturn);

			module(function($provide){
				$provide.value('NotificationsInstanceBuilderService', serviceMock);
			});

			inject(function($controller, $rootScope, $stateParams, $state) {
				scope = $rootScope.$new();
				state = $state;

				serviceMock.instance.and.returnValue({
					localGetById: localGetByIdSpy
				});

				stateParams = {
					section: section,
					id: itemId
				};

				state.current.data = {
					backState: "parentState",
					paramSections: [{
						paramValue: section
					}]
				};

				controller = $controller('NotificationsDetailsController', { $scope: scope, $state: state, $stateParams: stateParams });
			});
		});

		describe("when the page is loaded", function() {
			it("scope variables should be defined and correct", function() {
				expect(localGetByIdSpy).toHaveBeenCalledWith(itemId);
				expect(scope.formConfig).toEqual({service : { localGetById : localGetByIdSpy }, callbacks : { common : jasmine.any(Function) }, allowedActions : allowedActionsMock});
			});

			it("scope functions should be correct", function () {
				spyOn(state, 'go');
				scope.details = null;
				scope.formConfig.callbacks.common();
				expect(state.go).toHaveBeenCalledWith('parentState', { section : 'all-notifications', id : 'item_id' });
			});
		});
	});

	describe("The Notifications Details Controller - bad item", function (){
		var controller,
			scope,
			stateParams,
			state,
			serviceMock,
			localGetByIdSpy,
			section = "bad-section",
			itemId = "item_id",
			localGetByIdReturn = null;

		beforeEach(function() {
			module('angularTemplateApp');

			serviceMock = jasmine.createSpyObj('NotificationsInstanceBuilderService', ['instance']);
			localGetByIdSpy = jasmine.createSpy("localGetByIdSpy").and.returnValue(localGetByIdReturn);

			module(function($provide){
				$provide.value('NotificationsInstanceBuilderService', serviceMock);
			});

			inject(function($controller, $rootScope, $stateParams, $state) {
				scope = $rootScope.$new();
				state = $state;

				serviceMock.instance.and.returnValue({
					localGetById: localGetByIdSpy
				});

				stateParams = {
					section: section,
					id: itemId
				};

				state.current.data = {
					backState: "parentState",
					paramSections: [{
						paramValue: section
					}]
				};

				spyOn(state, 'go');

				controller = $controller('NotificationsDetailsController', { $scope: scope, $state: state, $stateParams: stateParams });
			});
		});

		describe("when the page is loaded", function() {
			it("should return to parent state", function () {
				expect(state.go).toHaveBeenCalledWith('parentState', { section : 'bad-section', id : 'item_id' });
			});
		});
	});
});