define(['NotificationsSummaryController', 'NotificationsConstants', 'moment'], function() {
	'use strict';

	describe("The Notifications Summary Controller", function (){
		var q,
			filter,
			deferred,
			scope,
			stateParams,
			notificationsPreferencesServiceMock,
			notificationsInstanceBuilderServiceMock,
			idGetterSpy,
			saveSpy,
			fetchSpy,
			dataPointers = {},
			dateRange = {
				startDate: moment().subtract(30, 'day').format('L'),
				endDate: moment().format('L')
			},
			section = "all-notifications",
			sectionName = "All Notifications",
			parsedItemId = "parsed_item_id";

		beforeEach(function() {
			module('angularTemplateApp');

			notificationsPreferencesServiceMock = jasmine.createSpyObj('NotificationsPreferencesService', ['dataPointers']);
			notificationsInstanceBuilderServiceMock = jasmine.createSpyObj('NotificationsInstanceBuilderService', ['instance']);
			idGetterSpy = jasmine.createSpy("idGetterSpy").and.returnValue(parsedItemId);
			saveSpy = jasmine.createSpy("saveSpy");
			fetchSpy = jasmine.createSpy("fetchSpy");

			module(function($provide){
				$provide.value('NotificationsPreferencesService', notificationsPreferencesServiceMock);
				$provide.value('NotificationsInstanceBuilderService', notificationsInstanceBuilderServiceMock);
			});

			inject(function($controller, $rootScope, $stateParams, $state, $q, $filter) {
				scope = $rootScope.$new();
				stateParams = $stateParams;
				filter = $filter;

				q = $q;
				deferred = q.defer();
				notificationsInstanceBuilderServiceMock.instance.and.returnValue({
					dataPointers: dataPointers,
					idGetter: idGetterSpy,
					save: saveSpy,
					fetch: fetchSpy
				});
				deferred.resolve('resolved');
				fetchSpy.and.returnValue(deferred.promise);

				stateParams.section = section;
				$state.current.data = {
					sectionName: sectionName,
					paramSections : [{
						paramValue : 'all-notifications',
						sectionName : 'All Notifications',
						sectionIndex : 1
					}, {
						paramValue : 'fit-heart-notices',
						sectionName : 'VA FitHeart App Notices',
						sectionIndex : 2
					}, {
						paramValue : 'va-health-messages',
						sectionName : 'VA Health Messages',
						sectionIndex : 3
					}]
				};

				$controller('NotificationsSummaryController', { $scope: scope, $stateParams: stateParams, $filter: filter });
			});
		});

		describe("scope variables", function() {
			it("should be defined and correct", function() {
				scope.$apply();
				expect(scope.filter.service).toEqual(notificationsInstanceBuilderServiceMock.instance(section));
				expect(scope.filter.defaults).toEqual({type : 'FIT_HEART,STAFF', status : 'All', startDate : dateRange.startDate, endDate : dateRange.endDate});
				expect(scope.sectionTitle).toBe(sectionName);
				expect(scope.allowedDateRange).toEqual({min: moment().subtract(1, 'year').format('L')});
				expect(scope.statusOptions).toEqual(['Unread', 'Read', 'All']);
				expect(scope.noResultsMessage).toBe('No results found.');
				expect(scope.typeOptions).toEqual({
					nullLabel: 'All Notifications',
					options: [
						{label: 'VA FitHeart App Notices', value: 'FIT_HEART'},
						{label: 'VA Health Messages', value: 'STAFF'}
					]
				});
			});
		});

		describe("table class function", function() {
			it("should be defined and correct", function() {
				scope.$apply();

				var item = {readFlag: true};
				expect(scope.table.columns[0].ngIconClass(item)).toEqual({ 'icon-dot-indicator': false });
				expect(scope.table.rows.ngClass(item)).toEqual({ 'bold-text': false });
			});
		});

		describe("when the call to get details is made", function() {
			var stateSpy;
			beforeEach(inject(function($state){
				stateSpy = spyOn($state, "go");
			}));

			it("for a read item, should get details and redirect for current entry", function() {
				var item = {readFlag: true};
				scope.getDetails(item);

				expect(idGetterSpy).toHaveBeenCalledWith(item);
				expect(stateParams.id).toBe(parsedItemId);
				expect(stateSpy).toHaveBeenCalledWith("^.detail", { section : 'all-notifications', id : 'parsed_item_id' });
			});

			it("for an unread item, should get details, redirect for current entry, and mark as read", function() {
				var item = {readFlag: false};
				scope.getDetails(item);

				expect(idGetterSpy).toHaveBeenCalledWith(item);
				expect(stateParams.id).toBe(parsedItemId);
				expect(stateSpy).toHaveBeenCalledWith("^.detail", { section : 'all-notifications', id : 'parsed_item_id' });
				expect(saveSpy).toHaveBeenCalledWith({ readFlag : true });
			});
		});
	});
});