define(['angular', 'angularMocks', 'AboutController', 'angularUiBootstrap'], function() {
    'use strict';

    describe("The About controller", function (){
        var controller, 
            scope,
            serviceMock,
			focusServiceMock,
            windowMock = window,
            fakeModal = {
                close: jasmine.createSpy('closeSpy')
            };

        beforeEach( function () {
            module('angularTemplateApp');

            serviceMock = jasmine.createSpyObj('pageService', ['getVersionNumber']);
			focusServiceMock = jasmine.createSpyObj('focusService', ['focusMain']);
            serviceMock.getVersionNumber.and.callFake(function(){
                return '1.0.0';
            });

            module(function($provide){
                $provide.value('pageService', serviceMock);
				$provide.value('focusService', focusServiceMock);
            });

            inject(function($controller, $rootScope, $modal) {
                scope = $rootScope.$new();
				spyOn($modal, 'open').and.returnValue(fakeModal);
                controller = $controller('AboutController', {$scope:scope, $modalInstance:fakeModal});
				scope.$digest();
            });
        });

        it ("should get the version number from the global service", function() {
            expect(serviceMock.getVersionNumber).toHaveBeenCalled();
            expect(scope.version).toEqual('1.0.0');
        });

		it ("should have four groups defined for the accordion content", function() {
			expect(scope.groups.length).toEqual(4);
		});

        describe("when the OK button is clicked", function (){
            beforeEach( function () {
                scope.ok();
            });

            it ("should close the modal", function() {
                expect(fakeModal.close).toHaveBeenCalled();
            });
        });

        describe("when EULA link is clicked", function (){
            beforeEach( function () {
                spyOn(windowMock, 'open');
                scope.showEula();
            });

            it ("should open a new browser window", function() {
                expect(windowMock.open).toHaveBeenCalledWith('src/ui-components/modals/about/eula.html', '_blank');
            });
        });
    });
});