define(['angular', 'app', 'global'], function (angular, app) {
	'use strict';

	app.controller('SetGoalsModalController', function ($scope, $modalInstance, $q, focusService, formatter, data) {

		$scope.$watch(function () {
			return $('div.modal-header h3').is(":visible");
		}, function (isVisible, wasVisible) {
			if (isVisible && !wasVisible) {
				focusService.focusElement($('div.modal-header h3'));
			}
		});

		$scope.data = angular.copy(data);
		$scope.data.target = $scope.data.target || {};
		$scope.originalData = angular.copy($scope.data);

		var todaysDate = new Date();
		function getMinAllowedDate() {
			return data.startDate && new Date(data.startDate).getTime() < todaysDate.getTime() ? data.startDate : formatter.getFormattedFrontendDate(todaysDate);
		}

		$scope.allowedDateRange = {
			min : getMinAllowedDate(),
			max : formatter.getFormattedFrontendDate(formatter.getDateTimeMinusDelta(todaysDate, {days:90}, "front"))
		};

		$scope.ok = function () {
			allowModalToClose().then(function () {
				$modalInstance.close($scope.data);
			});
		};

		$scope.close = function () {
			$modalInstance.dismiss('cancel');
		};

		$scope.reset = function () {
			$scope.data = angular.copy($scope.originalData);
		};

		$scope.$watch('[data.target.minutes, data.target.steps, data.target.miles]', function(newValues, oldValues) {
			if (!$scope.data.startDate) {
				for (var i = 0; i < newValues.length; i++) {
					if (angular.isString(newValues[i]) && newValues[i].length > 0 && newValues[i] !== oldValues[0]) {
						$scope.data.startDate = formatter.getFormattedFrontendDate(new Date());
					}
				}
			}
		}, true);

		function allowModalToClose() {
			var defer = $q.defer();
			if (angular.element('.modal-content validation-summary')
				&& angular.element('.modal-content validation-summary').controller('form')) {
				angular.element('.modal-content validation-summary').controller('form').validationSummary.validate().then( function () {
					defer.resolve();
				}, function () {
					defer.reject();
				});
			} else {
				defer.resolve();
			}
			return defer.promise;
		}
	})
});