define(['angular', 'angularMocks', 'SetGoalsModalController', 'angularUiBootstrap'], function() {
    'use strict';

    describe("The Set Goals Modal controller", function (){
        var controller, 
            scope,
            serviceMock,
			focusServiceMock,
			dataMock = {
				startDate: null,
				target: null
			},
            fakeModal = {
                close: jasmine.createSpy('closeSpy'),
                dismiss: jasmine.createSpy('dismissSpy')
            };

        beforeEach( function () {
            module('angularTemplateApp');

            serviceMock = jasmine.createSpyObj('pageService', ['getVersionNumber']);
			focusServiceMock = jasmine.createSpyObj('focusService', ['focusMain']);

            module(function($provide){
                $provide.value('pageService', serviceMock);
				$provide.value('focusService', focusServiceMock);
				$provide.value('data', dataMock);
            });

            inject(function($controller, $rootScope, $modal) {
                scope = $rootScope.$new();
				spyOn($modal, 'open').and.returnValue(fakeModal);
                controller = $controller('SetGoalsModalController', {$scope:scope, $modalInstance:fakeModal});
				scope.$digest();
            });
        });

        it ("should initialize scope variables", function() {
            expect(scope.data).toEqual({startDate : null, target : {}});
			expect(scope.originalData).toEqual(scope.data);
			expect(scope.allowedDateRange).toBeDefined();
			expect(scope.allowedDateRange.min).toBeDefined();
			expect(scope.allowedDateRange.max).toBeDefined();
        });

		describe("when the OK button is clicked", function (){
			beforeEach( function () {
				scope.ok();
				scope.$digest();
			});

			it ("should close the modal", function() {
				expect(fakeModal.close).toHaveBeenCalled();
			});
		});

		describe("when the Cancel button is clicked", function (){
			beforeEach( function () {
				scope.close();
			});

			it ("should close the modal", function() {
				expect(fakeModal.dismiss).toHaveBeenCalled();
			});
		});

		describe("when the Reset button is clicked", function (){
			it ("should reset the values to original data", function() {
				scope.data.startDate = "01/01/2016";
				expect(scope.data).not.toEqual(scope.originalData);
				scope.reset();
				expect(scope.data).toEqual(scope.originalData);
			});
		});
    });
});