define(['app', 'angular'], function (app) {
	app.directive('navBar', function (focusService, $timeout, $window) {
		return {
			restrict: 'E',
			transclude : true,
			scope : {
				'buttons' : '=',
				'filter' : '=' //service, disable, defaults, model, callbacks (success, failure)
			},
			templateUrl: 'src/ui-components/nav-bar/nav-bar_template.html',
			link: function(scope) {
				scope.collapseThreshold = false;
				scope.collapseButtons = false;
				scope.status = {
					toggleOpen: false
				};

				var setupButtonDropdown = function () {
					var leftButtonCount = scope.buttons.left ? scope.buttons.left.length : 0,
						rightButtonCount = scope.buttons.right ? scope.buttons.right.length : 0;

					if (leftButtonCount >= 2 || rightButtonCount >= 2 || (leftButtonCount + rightButtonCount >= 2)) {
						scope.collapseThreshold = true;
						scope.collapseButtons = $window.innerWidth < 768;
					}
					scope.$watch('status.toggleOpen', function (newVal, oldVal) {
						if (newVal === false && oldVal === true) {
							$timeout(function () {
								focusService.focusElement('button.dropdown-toggle');
							}, 300);
						}
					}, true);
				};

				if(scope.buttons) {
					setupButtonDropdown();
				}

				var unbindButtonWatch = scope.$watch('buttons', function (newVal, oldVal) {
					if (newVal && newVal !== oldVal) {
						setupButtonDropdown();
						unbindButtonWatch();
					}
				}, true);

				scope.showCollapseContent = function() {
					return !scope.isFilterCollapsed;
				};

				if (scope.collapseThreshold) {
					angular.element($window).bind('resize.doResize', function() {
						$timeout(function() {
							scope.collapseButtons = $window.innerWidth < 768;
						});
					});

					scope.$on("$destroy",function (){
						$(window).off("resize.doResize");
					});
				}
			},
			controller: function ($scope) {
				$scope.isFilterCollapsed = false;
				$scope.toggleFilterState = function() {
					$scope.isFilterCollapsed = !$scope.isFilterCollapsed;
				};
			}
		};
	});
});